@@ -3,41 +3,43 @@
  * found in the LICENSE file.
  */
 
+#include <ctype.h>
 #include <stddef.h>
 #include <stdio.h>
 #include <string.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <unistd.h>
-#include <ctype.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <sys/wait.h>
-#include <fcntl.h>
 #include <unistd.h>
 
-#include "host_common.h"
-
-#include "crossystem.h"
+#include "2api.h"
+#include "2common.h"
+#include "2nvstorage.h"
+#include "2sysincludes.h"
+#include "chromeos_config.h"
 #include "crossystem_arch.h"
+#include "crossystem.h"
 #include "crossystem_vbnv.h"
-#include "utility.h"
-#include "vboot_common.h"
-#include "vboot_nvstorage.h"
+#include "host_common.h"
+#include "flashrom.h"
+#include "subprocess.h"
 #include "vboot_struct.h"
 
 /* Filename for kernel command line */
 #define KERNEL_CMDLINE_PATH "/proc/cmdline"
 
-#define MOSYS_CROS_PATH "/usr/sbin/mosys"
-#define MOSYS_ANDROID_PATH "/system/bin/mosys"
+/* Filename for the mount-encrypted key */
+/* TODO(b/174807059): Remove this after we land driver-level TPM simulator on
+ * all VM boards */
+#define MOUNT_ENCRYPTED_KEY_PATH "/mnt/stateful_partition/encrypted.key"
+
+/* Filename for the TPM simulator NV data */
+#define TPM_SIMULATOR_NVCHIP_PATH \
+	"/mnt/stateful_partition/unencrypted/tpm2-simulator/NVChip"
 
 /* Fields that GetVdatString() can get */
 typedef enum VdatStringField {
-	VDAT_STRING_TIMERS = 0,           /* Timer values */
-	VDAT_STRING_LOAD_FIRMWARE_DEBUG,  /* LoadFirmware() debug information */
-	VDAT_STRING_LOAD_KERNEL_DEBUG,    /* LoadKernel() debug information */
-	VDAT_STRING_MAINFW_ACT            /* Active main firmware */
+	VDAT_STRING_DEPRECATED_TIMERS = 0,  /* Timer values */
+	VDAT_STRING_LOAD_FIRMWARE_DEBUG,  /* LoadFirmware() debug info */
+	VDAT_STRING_DEPRECATED_LOAD_KERNEL_DEBUG,  /* LoadKernel() debug info */
+	VDAT_STRING_MAINFW_ACT  /* Active main firmware */
 } VdatStringField;
 
 
@@ -46,7 +48,6 @@ typedef enum VdatIntField {
 	VDAT_INT_FLAGS = 0,           /* Flags */
 	VDAT_INT_HEADER_VERSION,      /* Header version for VbSharedData */
 	VDAT_INT_DEVSW_BOOT,          /* Dev switch position at boot */
-	VDAT_INT_DEVSW_VIRTUAL,       /* Dev switch is virtual */
 	VDAT_INT_RECSW_BOOT,          /* Recovery switch position at boot */
 	VDAT_INT_HW_WPSW_BOOT,        /* Hardware WP switch position at boot */
 
@@ -69,7 +70,7 @@ typedef enum VbBuildOption {
 } VbBuildOption;
 
 static const char *fw_results[] = {"unknown", "trying", "success", "failure"};
-static const char *default_boot[] = {"disk", "usb", "legacy"};
+static const char *default_boot[] = {"disk", "usb", "altfw"};
 
 /* Masks for kern_nv usage by kernel. */
 #define KERN_NV_FWUPDATE_TRIES_MASK 0x000F
@@ -91,77 +92,92 @@ int FwidStartsWith(const char *start)
 	return 0 == strncmp(fwid, start, strlen(start));
 }
 
+static struct vb2_context *get_fake_context(void)
+{
+	static uint8_t fake_workbuf[sizeof(struct vb2_shared_data) + 16]
+		__attribute__((aligned(VB2_WORKBUF_ALIGN)));
+	static struct vb2_context *fake_ctx;
+
+	if (fake_ctx)
+		return fake_ctx;
+
+	vb2api_init(fake_workbuf, sizeof(fake_workbuf), &fake_ctx);
+
+	return fake_ctx;
+}
+
 static int vnc_read;
 
-int VbGetNvStorage(VbNvParam param)
+int vb2_get_nv_storage(enum vb2_nv_param param)
 {
-	uint32_t value;
-	int retval;
-	static VbNvContext cached_vnc;
+	VbSharedDataHeader* sh = VbSharedDataRead();
+	struct vb2_context *ctx = get_fake_context();
+
+	if (!sh)
+		return -1;
 
 	/* TODO: locking around NV access */
 	if (!vnc_read) {
-		if (0 != VbReadNvStorage(&cached_vnc))
+		if (sh && sh->flags & VBSD_NVDATA_V2)
+			ctx->flags |= VB2_CONTEXT_NVDATA_V2;
+		if (0 != vb2_read_nv_storage(ctx)) {
+			free(sh);
 			return -1;
+		}
+		vb2_nv_init(ctx);
+
+		/* TODO: If vnc.raw_changed, attempt to reopen NVRAM for write
+		 * and save the new defaults.  If we're able to, log. */
+
 		vnc_read = 1;
 	}
 
-	if (0 != VbNvSetup(&cached_vnc))
-		return -1;
-	retval = VbNvGet(&cached_vnc, param, &value);
-	if (0 != VbNvTeardown(&cached_vnc))
-		return -1;
-	if (0 != retval)
-		return -1;
-
-	/* TODO: If vnc.raw_changed, attempt to reopen NVRAM for write and
-	 * save the new defaults.  If we're able to, log. */
-	/* TODO: release lock */
-
-	return (int)value;
+	free(sh);
+	return (int)vb2_nv_get(ctx, param);
 }
 
-int VbSetNvStorage(VbNvParam param, int value)
+int vb2_set_nv_storage(enum vb2_nv_param param, int value)
 {
-	VbNvContext vnc;
-	int retval = -1;
-	int i;
+	VbSharedDataHeader* sh = VbSharedDataRead();
+	struct vb2_context *ctx = get_fake_context();
 
-	if (0 != VbReadNvStorage(&vnc))
+	if (!sh)
 		return -1;
 
-	if (0 != VbNvSetup(&vnc))
-		goto VbSetNvCleanup;
-	i = VbNvSet(&vnc, param, (uint32_t)value);
-	if (0 != VbNvTeardown(&vnc))
-		goto VbSetNvCleanup;
-	if (0 != i)
-		goto VbSetNvCleanup;
+	/* TODO: locking around NV access */
+	if (sh && sh->flags & VBSD_NVDATA_V2)
+		ctx->flags |= VB2_CONTEXT_NVDATA_V2;
+	if (0 != vb2_read_nv_storage(ctx)) {
+		free(sh);
+		return -1;
+	}
+	vb2_nv_init(ctx);
+	vb2_nv_set(ctx, param, (uint32_t)value);
 
-	if (vnc.raw_changed) {
+	if (ctx->flags & VB2_CONTEXT_NVDATA_CHANGED) {
 		vnc_read = 0;
-		if (0 != VbWriteNvStorage(&vnc))
-			goto VbSetNvCleanup;
+		if (0 != vb2_write_nv_storage(ctx)) {
+			free(sh);
+			return -1;
+		}
 	}
 
 	/* Success */
-	retval = 0;
-
-VbSetNvCleanup:
-	/* TODO: release lock */
-	return retval;
+	free(sh);
+	return 0;
 }
 
 /*
- * Set a param value, and try to flag it for persistent backup.
- * It's okay if backup isn't supported. It's best-effort only.
+ * Set a param value, and try to flag it for persistent backup.  It's okay if
+ * backup isn't supported (which it isn't, in current designs). It's
+ * best-effort only.
  */
-static int VbSetNvStorage_WithBackup(VbNvParam param, int value)
+static int vb2_set_nv_storage_with_backup(enum vb2_nv_param param, int value)
 {
 	int retval;
-	retval = VbSetNvStorage(param, value);
+	retval = vb2_set_nv_storage(param, value);
 	if (!retval)
-		VbSetNvStorage(VBNV_BACKUP_NVRAM_REQUEST, 1);
+		vb2_set_nv_storage(VB2_NV_BACKUP_NVRAM_REQUEST, 1);
 	return retval;
 }
 
@@ -193,14 +209,14 @@ static VbBuildOption VbScanBuildOption(void)
 
 /* Determine whether the running OS image was built for debugging.
  * Returns 1 if yes, 0 if no or indeterminate. */
-int VbGetDebugBuild(void)
+static vb2_error_t VbGetDebugBuild(void)
 {
 	return VB_BUILD_OPTION_DEBUG == VbScanBuildOption();
 }
 
 /* Determine whether OS-level debugging should be allowed.
  * Returns 1 if yes, 0 if no or indeterminate. */
-int VbGetCrosDebug(void)
+static int VbGetCrosDebug(void)
 {
 	/* If the currently running system specifies its debug status, use
 	 * that in preference to other indicators. */
@@ -219,8 +235,8 @@ int VbGetCrosDebug(void)
 	return 0;
 }
 
-char *GetVdatLoadFirmwareDebug(char *dest, int size,
-                               const VbSharedDataHeader *sh)
+static char *GetVdatLoadFirmwareDebug(char *dest, int size,
+				      const VbSharedDataHeader *sh)
 {
 	snprintf(dest, size,
 		 "Check A result=%d\n"
@@ -236,102 +252,7 @@ char *GetVdatLoadFirmwareDebug(char *dest, int size,
 	return dest;
 }
 
-#define TRUNCATED "\n(truncated)\n"
-
-char *GetVdatLoadKernelDebug(char *dest, int size,
-                             const VbSharedDataHeader *sh)
-{
-	int used = 0;
-	int first_call_tracked = 0;
-	int call;
-
-	/* Make sure we have space for truncation warning */
-	if (size < strlen(TRUNCATED) + 1)
-		return NULL;
-	size -= strlen(TRUNCATED) + 1;
-
-	used += snprintf(
-	    dest + used, size - used,
-	    "Calls to LoadKernel()=%d\n",
-	    sh->lk_call_count);
-	if (used > size)
-		goto LoadKernelDebugExit;
-
-	/* Report on the last calls */
-	if (sh->lk_call_count > VBSD_MAX_KERNEL_CALLS)
-		first_call_tracked = sh->lk_call_count - VBSD_MAX_KERNEL_CALLS;
-	for (call = first_call_tracked; call < sh->lk_call_count; call++) {
-		const VbSharedDataKernelCall* shc = sh->lk_calls +
-				(call & (VBSD_MAX_KERNEL_CALLS - 1));
-		int first_part_tracked = 0;
-		int part;
-
-		used += snprintf(dest + used, size - used,
-				 "Call %d:\n"
-				 "  Boot flags=0x%02x\n"
-				 "  Boot mode=%d\n"
-				 "  Test error=%d\n"
-				 "  Return code=%d\n"
-				 "  Debug flags=0x%02x\n"
-				 "  Drive sectors=%" PRIu64 "\n"
-				 "  Sector size=%d\n"
-				 "  Check result=%d\n"
-				 "  Kernel partitions found=%d\n",
-				 call + 1,
-				 shc->boot_flags,
-				 shc->boot_mode,
-				 shc->test_error_num,
-				 shc->return_code,
-				 shc->flags,
-				 shc->sector_count,
-				 shc->sector_size,
-				 shc->check_result,
-				 shc->kernel_parts_found);
-		if (used > size)
-			goto LoadKernelDebugExit;
-
-		/* If we found too many partitions, only prints ones where the
-		 * structure has info. */
-		if (shc->kernel_parts_found > VBSD_MAX_KERNEL_PARTS)
-			first_part_tracked = shc->kernel_parts_found -
-					VBSD_MAX_KERNEL_PARTS;
-
-		/* Report on the partitions checked */
-		for (part = first_part_tracked; part < shc->kernel_parts_found;
-		     part++) {
-			const VbSharedDataKernelPart* shp = shc->parts +
-					(part & (VBSD_MAX_KERNEL_PARTS - 1));
-
-			used += snprintf(dest + used, size - used,
-					 "  Kernel %d:\n"
-					 "    GPT index=%d\n"
-					 "    Start sector=%" PRIu64 "\n"
-					 "    Sector count=%" PRIu64 "\n"
-					 "    Combined version=0x%08x\n"
-					 "    Check result=%d\n"
-					 "    Debug flags=0x%02x\n",
-					 part + 1,
-					 shp->gpt_index,
-					 shp->sector_start,
-					 shp->sector_count,
-					 shp->combined_version,
-					 shp->check_result,
-					 shp->flags);
-			if (used > size)
-				goto LoadKernelDebugExit;
-		}
-	}
-
-LoadKernelDebugExit:
-
-	/* Warn if data was truncated; we left space for this above. */
-	if (used > size)
-		strcat(dest, TRUNCATED);
-
-	return dest;
-}
-
-char *GetVdatString(char *dest, int size, VdatStringField field)
+static char *GetVdatString(char *dest, int size, VdatStringField field)
 {
 	VbSharedDataHeader *sh = VbSharedDataRead();
 	char *value = dest;
@@ -340,27 +261,10 @@ char *GetVdatString(char *dest, int size, VdatStringField field)
 		return NULL;
 
 	switch (field) {
-		case VDAT_STRING_TIMERS:
-			snprintf(dest, size,
-				 "LFS=%" PRIu64 ",%" PRIu64
-				 " LF=%" PRIu64 ",%" PRIu64
-				 " LK=%" PRIu64 ",%" PRIu64,
-				 sh->timer_vb_init_enter,
-				 sh->timer_vb_init_exit,
-				 sh->timer_vb_select_firmware_enter,
-				 sh->timer_vb_select_firmware_exit,
-				 sh->timer_vb_select_and_load_kernel_enter,
-				 sh->timer_vb_select_and_load_kernel_exit);
-			break;
-
 		case VDAT_STRING_LOAD_FIRMWARE_DEBUG:
 			value = GetVdatLoadFirmwareDebug(dest, size, sh);
 			break;
 
-		case VDAT_STRING_LOAD_KERNEL_DEBUG:
-			value = GetVdatLoadKernelDebug(dest, size, sh);
-			break;
-
 		case VDAT_STRING_MAINFW_ACT:
 			switch(sh->firmware_index) {
 				case 0:
@@ -386,7 +290,7 @@ char *GetVdatString(char *dest, int size, VdatStringField field)
 	return value;
 }
 
-int GetVdatInt(VdatIntField field)
+static int GetVdatInt(VdatIntField field)
 {
 	VbSharedDataHeader* sh = VbSharedDataRead();
 	int value = -1;
@@ -416,6 +320,7 @@ int GetVdatInt(VdatIntField field)
 			break;
 		case VDAT_INT_FW_BOOT2:
 			value = (sh->flags & VBSD_BOOT_FIRMWARE_VBOOT2 ? 1 : 0);
+			VBOOT_FALLTHROUGH;
 		default:
 			break;
 	}
@@ -427,10 +332,6 @@ int GetVdatInt(VdatIntField field)
 				value = (sh->flags &
 					 VBSD_BOOT_DEV_SWITCH_ON ? 1 : 0);
 				break;
-			case VDAT_INT_DEVSW_VIRTUAL:
-				value = (sh->flags &
-					 VBSD_HONOR_VIRT_DEV_SWITCH ? 1 : 0);
-				break;
 			case VDAT_INT_RECSW_BOOT:
 				value = (sh->flags &
 					 VBSD_BOOT_REC_SWITCH_ON ? 1 : 0);
@@ -468,63 +369,73 @@ int VbGetSystemPropertyInt(const char *name)
 
 	/* NV storage values */
 	else if (!strcasecmp(name,"kern_nv")) {
-		value = VbGetNvStorage(VBNV_KERNEL_FIELD);
+		value = vb2_get_nv_storage(VB2_NV_KERNEL_FIELD);
 	} else if (!strcasecmp(name,"nvram_cleared")) {
-		value = VbGetNvStorage(VBNV_KERNEL_SETTINGS_RESET);
+		value = vb2_get_nv_storage(VB2_NV_KERNEL_SETTINGS_RESET);
 	} else if (!strcasecmp(name,"recovery_request")) {
-		value = VbGetNvStorage(VBNV_RECOVERY_REQUEST);
+		value = vb2_get_nv_storage(VB2_NV_RECOVERY_REQUEST);
+	} else if (!strcasecmp(name,"diagnostic_request")) {
+		value = vb2_get_nv_storage(VB2_NV_DIAG_REQUEST);
 	} else if (!strcasecmp(name,"dbg_reset")) {
-		value = VbGetNvStorage(VBNV_DEBUG_RESET_MODE);
+		value = vb2_get_nv_storage(VB2_NV_DEBUG_RESET_MODE);
 	} else if (!strcasecmp(name,"disable_dev_request")) {
-		value = VbGetNvStorage(VBNV_DISABLE_DEV_REQUEST);
+		value = vb2_get_nv_storage(VB2_NV_DISABLE_DEV_REQUEST);
 	} else if (!strcasecmp(name,"clear_tpm_owner_request")) {
-		value = VbGetNvStorage(VBNV_CLEAR_TPM_OWNER_REQUEST);
+		if (TPM2_SIMULATOR && VTPM_PROXY)
+			/* Check TPM simulator NVChip status */
+			value = access(TPM_SIMULATOR_NVCHIP_PATH, F_OK) != 0;
+		else if (TPM2_SIMULATOR)
+			/* Check mount-encrypted key status */
+			value = access(MOUNT_ENCRYPTED_KEY_PATH, F_OK) != 0;
+		else
+			value = vb2_get_nv_storage(
+				VB2_NV_CLEAR_TPM_OWNER_REQUEST);
 	} else if (!strcasecmp(name,"clear_tpm_owner_done")) {
-		value = VbGetNvStorage(VBNV_CLEAR_TPM_OWNER_DONE);
+		value = vb2_get_nv_storage(VB2_NV_CLEAR_TPM_OWNER_DONE);
 	} else if (!strcasecmp(name,"tpm_rebooted")) {
-		value = VbGetNvStorage(VBNV_TPM_REQUESTED_REBOOT);
-	} else if (!strcasecmp(name,"fwb_tries")) {
-		value = VbGetNvStorage(VBNV_TRY_B_COUNT);
+		value = vb2_get_nv_storage(VB2_NV_TPM_REQUESTED_REBOOT);
+	} else if (!strcasecmp(name,"fwb_tries") ||
+		   !strcasecmp(name,"fw_try_count")) {
+		value = vb2_get_nv_storage(VB2_NV_TRY_COUNT);
 	} else if (!strcasecmp(name,"fw_vboot2")) {
 		value = GetVdatInt(VDAT_INT_FW_BOOT2);
-	} else if (!strcasecmp(name,"fw_try_count")) {
-		value = VbGetNvStorage(VBNV_FW_TRY_COUNT);
 	} else if (!strcasecmp(name,"fwupdate_tries")) {
-		value = VbGetNvStorage(VBNV_KERNEL_FIELD);
+		value = vb2_get_nv_storage(VB2_NV_KERNEL_FIELD);
 		if (value != -1)
 			value &= KERN_NV_FWUPDATE_TRIES_MASK;
 	} else if (!strcasecmp(name,"block_devmode")) {
-		value = VbGetNvStorage(VBNV_KERNEL_FIELD);
+		value = vb2_get_nv_storage(VB2_NV_KERNEL_FIELD);
 		if (value != -1) {
 			value &= KERN_NV_BLOCK_DEVMODE_FLAG;
 			value = !!value;
 		}
 	} else if (!strcasecmp(name,"tpm_attack")) {
-		value = VbGetNvStorage(VBNV_KERNEL_FIELD);
+		value = vb2_get_nv_storage(VB2_NV_KERNEL_FIELD);
 		if (value != -1) {
 			value &= KERN_NV_TPM_ATTACK_FLAG;
 			value = !!value;
 		}
 	} else if (!strcasecmp(name,"loc_idx")) {
-		value = VbGetNvStorage(VBNV_LOCALIZATION_INDEX);
+		value = vb2_get_nv_storage(VB2_NV_LOCALIZATION_INDEX);
 	} else if (!strcasecmp(name,"backup_nvram_request")) {
-		value = VbGetNvStorage(VBNV_BACKUP_NVRAM_REQUEST);
+		value = vb2_get_nv_storage(VB2_NV_BACKUP_NVRAM_REQUEST);
 	} else if (!strcasecmp(name,"dev_boot_usb")) {
-		value = VbGetNvStorage(VBNV_DEV_BOOT_USB);
-	} else if (!strcasecmp(name,"dev_boot_legacy")) {
-		value = VbGetNvStorage(VBNV_DEV_BOOT_LEGACY);
+		value = vb2_get_nv_storage(VB2_NV_DEV_BOOT_EXTERNAL);
+	} else if (!strcasecmp(name,"dev_boot_altfw") ||
+		   !strcasecmp(name,"dev_boot_legacy")) {
+		value = vb2_get_nv_storage(VB2_NV_DEV_BOOT_ALTFW);
 	} else if (!strcasecmp(name,"dev_boot_signed_only")) {
-		value = VbGetNvStorage(VBNV_DEV_BOOT_SIGNED_ONLY);
-	} else if (!strcasecmp(name,"dev_boot_fastboot_full_cap")) {
-		value = VbGetNvStorage(VBNV_DEV_BOOT_FASTBOOT_FULL_CAP);
-	} else if (!strcasecmp(name,"oprom_needed")) {
-		value = VbGetNvStorage(VBNV_OPROM_NEEDED);
+		value = vb2_get_nv_storage(VB2_NV_DEV_BOOT_SIGNED_ONLY);
+	} else if (!strcasecmp(name,"dev_enable_udc")) {
+		value = vb2_get_nv_storage(VB2_NV_DEV_ENABLE_UDC);
+	} else if (!strcasecmp(name,"display_request")) {
+		value = vb2_get_nv_storage(VB2_NV_DISPLAY_REQUEST);
 	} else if (!strcasecmp(name,"recovery_subcode")) {
-		value = VbGetNvStorage(VBNV_RECOVERY_SUBCODE);
+		value = vb2_get_nv_storage(VB2_NV_RECOVERY_SUBCODE);
 	} else if (!strcasecmp(name,"wipeout_request")) {
-		value = VbGetNvStorage(VBNV_FW_REQ_WIPEOUT);
+		value = vb2_get_nv_storage(VB2_NV_REQ_WIPEOUT);
 	} else if (!strcasecmp(name,"kernel_max_rollforward")) {
-		value = VbGetNvStorage(VBNV_KERNEL_MAX_ROLLFORWARD);
+		value = vb2_get_nv_storage(VB2_NV_KERNEL_MAX_ROLLFORWARD);
 	}
 	/* Other parameters */
 	else if (!strcasecmp(name,"cros_debug")) {
@@ -533,11 +444,10 @@ int VbGetSystemPropertyInt(const char *name)
 		value = VbGetDebugBuild();
 	} else if (!strcasecmp(name,"devsw_boot")) {
 		value = GetVdatInt(VDAT_INT_DEVSW_BOOT);
-	} else if (!strcasecmp(name,"devsw_virtual")) {
-		value = GetVdatInt(VDAT_INT_DEVSW_VIRTUAL);
 	} else if (!strcasecmp(name, "recoverysw_boot")) {
 		value = GetVdatInt(VDAT_INT_RECSW_BOOT);
-	} else if (!strcasecmp(name, "wpsw_boot")) {
+	} else if (!strcasecmp(name, "wpsw_cur")) {
+		/* Use "write-protect at boot" as a fallback value. */
 		value = GetVdatInt(VDAT_INT_HW_WPSW_BOOT);
 	} else if (!strcasecmp(name,"vdat_flags")) {
 		value = GetVdatInt(VDAT_INT_FLAGS);
@@ -549,14 +459,12 @@ int VbGetSystemPropertyInt(const char *name)
 		value = GetVdatInt(VDAT_INT_TRIED_FIRMWARE_B);
 	} else if (!strcasecmp(name,"recovery_reason")) {
 		value = GetVdatInt(VDAT_INT_RECOVERY_REASON);
-	} else if (!strcasecmp(name, "fastboot_unlock_in_fw")) {
-		value = VbGetNvStorage(VBNV_FASTBOOT_UNLOCK_IN_FW);
 	} else if (!strcasecmp(name, "boot_on_ac_detect")) {
-		value = VbGetNvStorage(VBNV_BOOT_ON_AC_DETECT);
+		value = vb2_get_nv_storage(VB2_NV_BOOT_ON_AC_DETECT);
 	} else if (!strcasecmp(name, "try_ro_sync")) {
-		value = VbGetNvStorage(VBNV_TRY_RO_SYNC);
+		value = vb2_get_nv_storage(VB2_NV_TRY_RO_SYNC);
 	} else if (!strcasecmp(name, "battery_cutoff_request")) {
-		value = VbGetNvStorage(VBNV_BATTERY_CUTOFF_REQUEST);
+		value = vb2_get_nv_storage(VB2_NV_BATTERY_CUTOFF_REQUEST);
 	} else if (!strcasecmp(name, "inside_vm")) {
 		/* Detect if the host is a VM. If there is no HWID and the
 		 * firmware type is "nonchrome", then assume it is a VM. If
@@ -566,22 +474,36 @@ int VbGetSystemPropertyInt(const char *name)
 		if (!VbGetSystemPropertyString("hwid", hwid, sizeof(hwid))) {
 			char fwtype_buf[VB_MAX_STRING_PROPERTY];
 			const char *fwtype = VbGetSystemPropertyString(
-			    "mainfw_type", fwtype_buf, sizeof(fwtype_buf));
+				"mainfw_type", fwtype_buf, sizeof(fwtype_buf));
 			if (fwtype && !strcasecmp(fwtype, "nonchrome")) {
 				value = 1;
 			}
 		} else {
 			value = 0;
 		}
+	} else if (!strcasecmp(name, "post_ec_sync_delay")) {
+		value = vb2_get_nv_storage(VB2_NV_POST_EC_SYNC_DELAY);
 	}
 
 	return value;
 }
 
 const char *VbGetSystemPropertyString(const char *name, char *dest,
-                                      size_t size)
+				      size_t size)
 {
-	/* Check architecture-dependent properties first */
+	/* Check for HWID override via cros_config */
+	if (!strcasecmp(name, "hwid")) {
+		char *hwid_override;
+
+		if (chromeos_config_get_string("/", "hwid-override",
+					       &hwid_override) == VB2_SUCCESS) {
+			StrCopy(dest, hwid_override, size);
+			free(hwid_override);
+			return dest;
+		}
+	}
+
+	/* Check architecture-dependent properties */
 	if (VbGetArchPropertyString(name, dest, size))
 		return dest;
 
@@ -596,37 +518,35 @@ const char *VbGetSystemPropertyString(const char *name, char *dest,
 		}
 	} else if (!strcasecmp(name, "mainfw_act")) {
 		return GetVdatString(dest, size, VDAT_STRING_MAINFW_ACT);
-	} else if (!strcasecmp(name, "vdat_timers")) {
-		return GetVdatString(dest, size, VDAT_STRING_TIMERS);
 	} else if (!strcasecmp(name, "vdat_lfdebug")) {
 		return GetVdatString(dest, size,
 				     VDAT_STRING_LOAD_FIRMWARE_DEBUG);
-	} else if (!strcasecmp(name, "vdat_lkdebug")) {
-		return GetVdatString(dest, size, VDAT_STRING_LOAD_KERNEL_DEBUG);
 	} else if (!strcasecmp(name, "fw_try_next")) {
-		return VbGetNvStorage(VBNV_FW_TRY_NEXT) ? "B" : "A";
+		return vb2_get_nv_storage(VB2_NV_TRY_NEXT) ? "B" : "A";
 	} else if (!strcasecmp(name, "fw_tried")) {
-		return VbGetNvStorage(VBNV_FW_TRIED) ? "B" : "A";
+		return vb2_get_nv_storage(VB2_NV_FW_TRIED) ? "B" : "A";
 	} else if (!strcasecmp(name, "fw_result")) {
-		int v = VbGetNvStorage(VBNV_FW_RESULT);
+		int v = vb2_get_nv_storage(VB2_NV_FW_RESULT);
 		if (v < ARRAY_SIZE(fw_results))
 			return fw_results[v];
 		else
 			return "unknown";
 	} else if (!strcasecmp(name, "fw_prev_tried")) {
-		return VbGetNvStorage(VBNV_FW_PREV_TRIED) ? "B" : "A";
+		return vb2_get_nv_storage(VB2_NV_FW_PREV_TRIED) ? "B" : "A";
 	} else if (!strcasecmp(name, "fw_prev_result")) {
-		int v = VbGetNvStorage(VBNV_FW_PREV_RESULT);
+		int v = vb2_get_nv_storage(VB2_NV_FW_PREV_RESULT);
 		if (v < ARRAY_SIZE(fw_results))
 			return fw_results[v];
 		else
 			return "unknown";
 	} else if (!strcasecmp(name,"dev_default_boot")) {
-		int v = VbGetNvStorage(VBNV_DEV_DEFAULT_BOOT);
+		int v = vb2_get_nv_storage(VB2_NV_DEV_DEFAULT_BOOT);
 		if (v < ARRAY_SIZE(default_boot))
 			return default_boot[v];
 		else
 			return "unknown";
+	} else if (!strcasecmp(name, "minios_priority")) {
+		return vb2_get_nv_storage(VB2_NV_MINIOS_PRIORITY) ? "B" : "A";
 	}
 
 	return NULL;
@@ -644,82 +564,110 @@ int VbSetSystemPropertyInt(const char *name, int value)
 	if (!strcasecmp(name,"nvram_cleared")) {
 		/* Can only clear this flag; it's set inside the NV storage
 		 * library. */
-		return VbSetNvStorage(VBNV_KERNEL_SETTINGS_RESET, 0);
+		return vb2_set_nv_storage(VB2_NV_KERNEL_SETTINGS_RESET, 0);
 	} else if (!strcasecmp(name,"recovery_request")) {
-		return VbSetNvStorage(VBNV_RECOVERY_REQUEST, value);
+		return vb2_set_nv_storage(VB2_NV_RECOVERY_REQUEST, value);
+	} else if (!strcasecmp(name,"diagnostic_request")) {
+		return vb2_set_nv_storage(VB2_NV_DIAG_REQUEST, value);
 	} else if (!strcasecmp(name,"recovery_subcode")) {
-		return VbSetNvStorage(VBNV_RECOVERY_SUBCODE, value);
+		return vb2_set_nv_storage(VB2_NV_RECOVERY_SUBCODE, value);
 	} else if (!strcasecmp(name,"dbg_reset")) {
-		return VbSetNvStorage(VBNV_DEBUG_RESET_MODE, value);
+		return vb2_set_nv_storage(VB2_NV_DEBUG_RESET_MODE, value);
 	} else if (!strcasecmp(name,"disable_dev_request")) {
-		return VbSetNvStorage(VBNV_DISABLE_DEV_REQUEST, value);
+		return vb2_set_nv_storage(VB2_NV_DISABLE_DEV_REQUEST, value);
 	} else if (!strcasecmp(name,"clear_tpm_owner_request")) {
-		return VbSetNvStorage(VBNV_CLEAR_TPM_OWNER_REQUEST, value);
+		if (TPM2_SIMULATOR) {
+			/* We don't support to set clear_tpm_owner_request to 0
+			 * on simulator */
+			if (value == 0)
+				return -1;
+			const char *tpm_path =
+				VTPM_PROXY ? TPM_SIMULATOR_NVCHIP_PATH
+					   : MOUNT_ENCRYPTED_KEY_PATH;
+			/* Check TPM simulator data status */
+			if (!access(tpm_path, F_OK)) {
+				/* Remove the TPM2.0 simulator data */
+				return remove(tpm_path);
+			} else {
+				/* Return success when the file is already
+				 * removed */
+				return 0;
+			}
+		} else {
+			return vb2_set_nv_storage(
+				VB2_NV_CLEAR_TPM_OWNER_REQUEST, value);
+		}
 	} else if (!strcasecmp(name,"clear_tpm_owner_done")) {
 		/* Can only clear this flag; it's set by firmware. */
-		return VbSetNvStorage(VBNV_CLEAR_TPM_OWNER_DONE, 0);
-	} else if (!strcasecmp(name,"fwb_tries")) {
-		return VbSetNvStorage(VBNV_TRY_B_COUNT, value);
-	} else if (!strcasecmp(name,"fw_try_count")) {
-		return VbSetNvStorage(VBNV_FW_TRY_COUNT, value);
-	} else if (!strcasecmp(name,"oprom_needed")) {
-		return VbSetNvStorage(VBNV_OPROM_NEEDED, value);
+		return vb2_set_nv_storage(VB2_NV_CLEAR_TPM_OWNER_DONE, 0);
+	} else if (!strcasecmp(name,"fwb_tries") ||
+		   !strcasecmp(name,"fw_try_count")) {
+		return vb2_set_nv_storage(VB2_NV_TRY_COUNT, value);
+	} else if (!strcasecmp(name,"display_request")) {
+		return vb2_set_nv_storage(VB2_NV_DISPLAY_REQUEST, value);
 	} else if (!strcasecmp(name,"wipeout_request")) {
 		/* Can only clear this flag, set only by firmware. */
-		return VbSetNvStorage(VBNV_FW_REQ_WIPEOUT, 0);
+		return vb2_set_nv_storage(VB2_NV_REQ_WIPEOUT, 0);
 	} else if (!strcasecmp(name,"backup_nvram_request")) {
 		/* Best-effort only, since it requires firmware and TPM
 		 * support. */
-		return VbSetNvStorage(VBNV_BACKUP_NVRAM_REQUEST, value);
+		return vb2_set_nv_storage(VB2_NV_BACKUP_NVRAM_REQUEST, value);
 	} else if (!strcasecmp(name,"fwupdate_tries")) {
-		int kern_nv = VbGetNvStorage(VBNV_KERNEL_FIELD);
+		int kern_nv = vb2_get_nv_storage(VB2_NV_KERNEL_FIELD);
 		if (kern_nv == -1)
 			return -1;
 		kern_nv &= ~KERN_NV_FWUPDATE_TRIES_MASK;
 		kern_nv |= (value & KERN_NV_FWUPDATE_TRIES_MASK);
-		return VbSetNvStorage_WithBackup(VBNV_KERNEL_FIELD, kern_nv);
+		return vb2_set_nv_storage_with_backup(
+			VB2_NV_KERNEL_FIELD, kern_nv);
 	} else if (!strcasecmp(name,"block_devmode")) {
-		int kern_nv = VbGetNvStorage(VBNV_KERNEL_FIELD);
+		int kern_nv = vb2_get_nv_storage(VB2_NV_KERNEL_FIELD);
 		if (kern_nv == -1)
 			return -1;
 		kern_nv &= ~KERN_NV_BLOCK_DEVMODE_FLAG;
 		if (value)
 			kern_nv |= KERN_NV_BLOCK_DEVMODE_FLAG;
-		return VbSetNvStorage_WithBackup(VBNV_KERNEL_FIELD, kern_nv);
+		return vb2_set_nv_storage_with_backup(
+			VB2_NV_KERNEL_FIELD, kern_nv);
 	} else if (!strcasecmp(name,"tpm_attack")) {
 		/* This value should only be read and cleared, but we allow
 		 * setting it to 1 for testing. */
-		int kern_nv = VbGetNvStorage(VBNV_KERNEL_FIELD);
+		int kern_nv = vb2_get_nv_storage(VB2_NV_KERNEL_FIELD);
 		if (kern_nv == -1)
 			return -1;
 		kern_nv &= ~KERN_NV_TPM_ATTACK_FLAG;
 		if (value)
 			kern_nv |= KERN_NV_TPM_ATTACK_FLAG;
-		return VbSetNvStorage_WithBackup(VBNV_KERNEL_FIELD, kern_nv);
+		return vb2_set_nv_storage_with_backup(
+			VB2_NV_KERNEL_FIELD, kern_nv);
 	} else if (!strcasecmp(name,"loc_idx")) {
-		return VbSetNvStorage_WithBackup(VBNV_LOCALIZATION_INDEX,
-						 value);
+		return vb2_set_nv_storage_with_backup(
+			VB2_NV_LOCALIZATION_INDEX, value);
 	} else if (!strcasecmp(name,"dev_boot_usb")) {
-		return VbSetNvStorage_WithBackup(VBNV_DEV_BOOT_USB, value);
-	} else if (!strcasecmp(name,"dev_boot_legacy")) {
-		return VbSetNvStorage_WithBackup(VBNV_DEV_BOOT_LEGACY, value);
+		return vb2_set_nv_storage_with_backup(
+			VB2_NV_DEV_BOOT_EXTERNAL, value);
+	} else if (!strcasecmp(name,"dev_boot_altfw") ||
+		   !strcasecmp(name,"dev_boot_legacy")) {
+		return vb2_set_nv_storage_with_backup(
+			VB2_NV_DEV_BOOT_ALTFW, value);
 	} else if (!strcasecmp(name,"dev_boot_signed_only")) {
-		return VbSetNvStorage_WithBackup(VBNV_DEV_BOOT_SIGNED_ONLY,
-						 value);
-	} else if (!strcasecmp(name,"dev_boot_fastboot_full_cap")) {
-		return VbSetNvStorage_WithBackup(
-				VBNV_DEV_BOOT_FASTBOOT_FULL_CAP, value);
-	} else if (!strcasecmp(name, "fastboot_unlock_in_fw")) {
-		return VbSetNvStorage_WithBackup(VBNV_FASTBOOT_UNLOCK_IN_FW,
-						 value);
+		return vb2_set_nv_storage_with_backup(
+			VB2_NV_DEV_BOOT_SIGNED_ONLY, value);
+	} else if (!strcasecmp(name, "dev_enable_udc")) {
+		return vb2_set_nv_storage_with_backup(
+			VB2_NV_DEV_ENABLE_UDC, value);
 	} else if (!strcasecmp(name, "boot_on_ac_detect")) {
-		return VbSetNvStorage_WithBackup(VBNV_BOOT_ON_AC_DETECT, value);
+		return vb2_set_nv_storage_with_backup(
+			VB2_NV_BOOT_ON_AC_DETECT, value);
 	} else if (!strcasecmp(name, "try_ro_sync")) {
-		return VbSetNvStorage_WithBackup(VBNV_TRY_RO_SYNC, value);
+		return vb2_set_nv_storage_with_backup(
+			VB2_NV_TRY_RO_SYNC, value);
 	} else if (!strcasecmp(name, "battery_cutoff_request")) {
-		return VbSetNvStorage(VBNV_BATTERY_CUTOFF_REQUEST, value);
+		return vb2_set_nv_storage(VB2_NV_BATTERY_CUTOFF_REQUEST, value);
 	} else if (!strcasecmp(name,"kernel_max_rollforward")) {
-		return VbSetNvStorage(VBNV_KERNEL_MAX_ROLLFORWARD, value);
+		return vb2_set_nv_storage(VB2_NV_KERNEL_MAX_ROLLFORWARD, value);
+	} else if (!strcasecmp(name, "post_ec_sync_delay")) {
+		return vb2_set_nv_storage(VB2_NV_POST_EC_SYNC_DELAY, value);
 	}
 
 	return -1;
@@ -733,26 +681,43 @@ int VbSetSystemPropertyString(const char* name, const char* value)
 
 	if (!strcasecmp(name, "fw_try_next")) {
 		if (!strcasecmp(value, "A"))
-			return VbSetNvStorage(VBNV_FW_TRY_NEXT, 0);
+			return vb2_set_nv_storage(VB2_NV_TRY_NEXT, 0);
 		else if (!strcasecmp(value, "B"))
-			return VbSetNvStorage(VBNV_FW_TRY_NEXT, 1);
+			return vb2_set_nv_storage(VB2_NV_TRY_NEXT, 1);
+		else
+			return -1;
+	} else if (!strcasecmp(name, "minios_priority")) {
+		if (!strcasecmp(value, "A"))
+			return vb2_set_nv_storage(VB2_NV_MINIOS_PRIORITY, 0);
+		else if (!strcasecmp(value, "B"))
+			return vb2_set_nv_storage(VB2_NV_MINIOS_PRIORITY, 1);
 		else
 			return -1;
-
 	} else if (!strcasecmp(name, "fw_result")) {
 		int i;
 
 		for (i = 0; i < ARRAY_SIZE(fw_results); i++) {
 			if (!strcasecmp(value, fw_results[i]))
-				return VbSetNvStorage(VBNV_FW_RESULT, i);
+				return vb2_set_nv_storage(VB2_NV_FW_RESULT, i);
 		}
 		return -1;
 	} else if (!strcasecmp(name, "dev_default_boot")) {
 		int i;
 
+		/* "legacy" term deprecated in favour of "altfw"
+		   (see: b/179458327) */
+		if (!strcasecmp(value, "legacy")) {
+			fprintf(stderr,
+				"!!!\n"
+				"!!! PLEASE USE 'altfw' INSTEAD OF 'legacy'\n"
+				"!!!\n");
+			value = "altfw";
+		}
+
 		for (i = 0; i < ARRAY_SIZE(default_boot); i++) {
 			if (!strcasecmp(value, default_boot[i]))
-				return VbSetNvStorage(VBNV_DEV_DEFAULT_BOOT, i);
+				return vb2_set_nv_storage(
+					VB2_NV_DEV_DEFAULT_BOOT, i);
 		}
 		return -1;
 	}
@@ -760,119 +725,101 @@ int VbSetSystemPropertyString(const char* name, const char* value)
 	return -1;
 }
 
-static int InAndroid(void)
+/**
+ * Get index of the last valid VBNV entry in an EEPROM.
+ *
+ * @param buf		Pointer to the beginning of the EEPROM.
+ * @param buf_sz	Size of the EEPROM.
+ * @param vbnv_size	The size of a single VBNV entry for this device.
+ *
+ * @return The index of the last valid VBNV entry on success, or -1 on
+ * failure.
+ */
+static int vb2_nv_index(const uint8_t *buf, uint32_t buf_sz, int vbnv_size)
 {
-	int fd;
-	struct stat s;
-	int retval = 0;
+	int index;
+	uint8_t blank[VB2_NVDATA_SIZE_V2];
 
-	/*
-	 * In Android, mosys utility located in /system/bin check if file
-	 * exists.  Using fstat because for some reason, stat() was seg
-	 * faulting in Android
-	 */
-	fd = open(MOSYS_ANDROID_PATH, O_RDONLY);
-	if (fd != -1) {
-		if (fstat(fd, &s) == 0)
-			retval = 1;
-		close(fd);
+	/* The size of the buffer should be an even multiple of the
+	   VBNV size. */
+	if (buf_sz % vbnv_size != 0) {
+		VB2_DIE("The VBNV in flash (%u bytes) is not an even multiple "
+			"of the VBNV size (%u bytes).  This is likely a "
+			"firmware bug.\n", buf_sz, vbnv_size);
 	}
-	return retval;
+
+	memset(blank, 0xff, sizeof(blank));
+	for (index = 0; index < buf_sz / vbnv_size; index++) {
+		if (!memcmp(blank, &buf[index * vbnv_size], vbnv_size))
+			break;
+	}
+
+	if (!index) {
+		fprintf(stderr, "VBNV is uninitialized.\n");
+		return -1;
+	}
+
+	return index - 1;
 }
 
-static int ExecuteMosys(char * const argv[], char *buf, size_t bufsize)
+#define VBNV_FMAP_REGION "RW_NVRAM"
+
+int vb2_read_nv_storage_flashrom(struct vb2_context *ctx)
 {
-	int status, mosys_to_crossystem[2];
-	pid_t pid;
-	ssize_t n;
+	int index;
+	int vbnv_size = vb2_nv_get_size(ctx);
+	uint8_t *flash_buf;
+	uint32_t flash_size;
 
-	if (pipe(mosys_to_crossystem) < 0) {
-		fprintf(stderr, "pipe() error\n");
+	if (flashrom_read(FLASHROM_PROGRAMMER_INTERNAL_AP, VBNV_FMAP_REGION,
+			  &flash_buf, &flash_size))
+		return -1;
+
+	index = vb2_nv_index(flash_buf, flash_size, vbnv_size);
+	if (index < 0) {
+		free(flash_buf);
 		return -1;
 	}
 
-	if ((pid = fork()) < 0) {
-		fprintf(stderr, "fork() error\n");
-		close(mosys_to_crossystem[0]);
-		close(mosys_to_crossystem[1]);
-		return -1;
-	} else if (!pid) {  /* Child */
-		close(mosys_to_crossystem[0]);
-		/* Redirect pipe's write-end to mosys' stdout */
-		if (STDOUT_FILENO != mosys_to_crossystem[1]) {
-			if (dup2(mosys_to_crossystem[1], STDOUT_FILENO)
-			    != STDOUT_FILENO) {
-				fprintf(stderr, "stdout dup2() failed (mosys)\n");
-				close(mosys_to_crossystem[1]);
-				exit(1);
-			}
-		}
-		/* Execute mosys */
-		execv(InAndroid() ? MOSYS_ANDROID_PATH : MOSYS_CROS_PATH, argv);
-		/* We shouldn't be here; exit now! */
-		fprintf(stderr, "execv() of mosys failed\n");
-		close(mosys_to_crossystem[1]);
-		exit(1);
-	} else {  /* Parent */
-		close(mosys_to_crossystem[1]);
-		if (bufsize) {
-			bufsize--;  /* Reserve 1 byte for '\0' */
-			while ((n = read(mosys_to_crossystem[0],
-					 buf, bufsize)) > 0) {
-				buf += n;
-				bufsize -= n;
-			}
-			*buf = '\0';
-		} else {
-			n = 0;
-		}
-		close(mosys_to_crossystem[0]);
-		if (n < 0)
-			fprintf(stderr, "read() error on output from mosys\n");
-		if (waitpid(pid, &status, 0) < 0 || status) {
-			fprintf(stderr, "waitpid() or mosys error\n");
-			return -1;
-		}
-		if (n < 0)
-			return -1;
-	}
+	memcpy(ctx->nvdata, &flash_buf[index * vbnv_size], vbnv_size);
+	free(flash_buf);
 	return 0;
 }
 
-int VbReadNvStorage_mosys(VbNvContext *vnc)
+int vb2_write_nv_storage_flashrom(struct vb2_context *ctx)
 {
-	char hexstring[VBNV_BLOCK_SIZE * 2 + 32];  /* Reserve extra 32 bytes */
-	char * const argv[] = {
-		InAndroid() ? MOSYS_ANDROID_PATH : MOSYS_CROS_PATH,
-		"nvram", "vboot", "read", NULL
-	};
-	char hexdigit[3];
-	int i;
+	int rv = 0;
+	int current_index;
+	int next_index;
+	int vbnv_size = vb2_nv_get_size(ctx);
+	uint8_t *flash_buf;
+	uint32_t flash_size;
 
-	if (ExecuteMosys(argv, hexstring, sizeof(hexstring)))
+	if (flashrom_read(FLASHROM_PROGRAMMER_INTERNAL_AP, VBNV_FMAP_REGION,
+			  &flash_buf, &flash_size))
 		return -1;
-	hexdigit[2] = '\0';
-	for (i = 0; i < VBNV_BLOCK_SIZE; i++) {
-		hexdigit[0] = hexstring[i * 2];
-		hexdigit[1] = hexstring[i * 2 + 1];
-		vnc->raw[i] = strtol(hexdigit, NULL, 16);
+
+	current_index = vb2_nv_index(flash_buf, flash_size, vbnv_size);
+	if (current_index < 0) {
+		rv = -1;
+		goto exit;
 	}
-	return 0;
-}
 
-int VbWriteNvStorage_mosys(VbNvContext* vnc)
-{
-	char hexstring[VBNV_BLOCK_SIZE * 2 + 1];
-	char * const argv[] = {
-		InAndroid() ? MOSYS_ANDROID_PATH : MOSYS_CROS_PATH,
-		"nvram", "vboot", "write", hexstring, NULL
-	};
-	int i;
+	next_index = current_index + 1;
+	if (next_index * vbnv_size == flash_size) {
+		/* VBNV is full.  Erase and write at beginning. */
+		memset(flash_buf, 0xff, flash_size);
+		next_index = 0;
+	}
 
-	for (i = 0; i < VBNV_BLOCK_SIZE; i++)
-		snprintf(hexstring + i * 2, 3, "%02x", vnc->raw[i]);
-	hexstring[sizeof(hexstring) - 1] = '\0';
-	if (ExecuteMosys(argv, NULL, 0))
-		return -1;
-	return 0;
+	memcpy(&flash_buf[next_index * vbnv_size], ctx->nvdata, vbnv_size);
+	if (flashrom_write(FLASHROM_PROGRAMMER_INTERNAL_AP, VBNV_FMAP_REGION,
+			   flash_buf, flash_size)) {
+		rv = -1;
+		goto exit;
+	}
+
+ exit:
+	free(flash_buf);
+	return rv;
 }
