@@ -9,42 +9,31 @@
 # Prerequisite tools needed in the system path:
 #
 #  futility (from src/platform/vboot_reference)
-#  vbutil_kernel (from src/platform/vboot_reference)
-#  cgpt (from src/platform/vboot_reference)
-#  dump_kernel_config (from src/platform/vboot_reference)
-#  verity (from src/platform/verity)
+#  verity (from src/platform2/verity)
 #  load_kernel_test (from src/platform/vboot_reference)
 #  dumpe2fs
+#  e2fsck
 #  sha1sum
-#  cbfstool (from src/third_party/coreboot)
+
+MINIOS_KERNEL_GUID="09845860-705f-4bb5-b16c-8a8a099caf52"
 
 # Load common constants and variables.
 . "$(dirname "$0")/common.sh"
 
-# Which futility to run?
-# futility is exception because there are local automated tests for it.
-[ -z "${FUTILITY}" ] && FUTILITY=futility
-
 # Print usage string
 usage() {
   cat <<EOF
 Usage: $PROG <type> input_image /path/to/keys/dir [output_image] [version_file]
 where <type> is one of:
-             ssd  (sign an SSD image)
-             base (sign a base image, similar to an SSD image)
+             base (sign a base image)
              recovery (sign a USB recovery image)
              factory (sign a factory install image)
-             install (old alias to "factory")
              update_payload (sign a delta update hash)
-             kernel (sign a kernel image)
-             recovery_kernel (sign a recovery_kernel image)
              firmware (sign a firmware image)
-             usb  (sign an image to boot directly from USB)
              verify (verify an image including rootfs hashes)
-             nv_lp0_firmware (sign nvidia lp0 firmware)
              accessory_usbpd (sign USB-PD accessory firmware)
              accessory_rwsig (sign accessory RW firmware)
-             oci-container (sign an OCI container)
+             gsc_firmware (sign a GSC firmware image)
 
 output_image: File name of the signed output image
 version_file: File name of where to read the kernel and firmware versions.
@@ -89,8 +78,7 @@ set -e
 PATH=$PATH:/usr/sbin:/sbin
 
 # Make sure the tools we need are available.
-for prereqs in ${FUTILITY} vbutil_kernel cgpt dump_kernel_config verity \
-               cbfstool load_kernel_test dumpe2fs sha1sum e2fsck; do
+for prereqs in ${FUTILITY} verity load_kernel_test dumpe2fs e2fsck sha1sum; do
   type -P "${prereqs}" &>/dev/null || \
     die "${prereqs} tool not found."
 done
@@ -113,19 +101,6 @@ get_verity_arg() {
   echo "$1" | sed -n "s/.*\b$2=\([^ \"]*\).*/\1/p"
 }
 
-is_old_verity_argv() {
-  local depth=$(echo "$1" | cut -f7 -d' ')
-  if [ "$depth" = "0" ]; then
-    return 0
-  fi
-  return 1
-}
-
-# Returns true if given ec.bin is signed or false if not.
-is_ec_rw_signed() {
-  ${FUTILITY} dump_fmap "$1" | grep -q KEY_RO
-}
-
 # Get the dmparams parameters from a kernel config.
 get_dmparams_from_config() {
   local kernel_config=$1
@@ -136,11 +111,7 @@ get_hash_from_config() {
   local kernel_config=$1
   local dm_config=$(get_dmparams_from_config "${kernel_config}")
   local vroot_dev=$(get_dm_slave "${dm_config}" vroot)
-  if is_old_verity_argv "${vroot_dev}"; then
-    echo ${vroot_dev} | cut -f9 -d ' '
-  else
-    echo $(get_verity_arg "${vroot_dev}" root_hexdigest)
-  fi
+  echo $(get_verity_arg "${vroot_dev}" root_hexdigest)
 }
 
 # Get the slave device and its args
@@ -185,33 +156,12 @@ calculate_rootfs_hash() {
   fi
   local vroot_dev=$(get_dm_slave "${dm_config}" vroot)
 
-  local rootfs_sectors
-  local verity_depth
-  local verity_algorithm
-  local root_dev
-  local hash_dev
-  local verity_bin="verity"
-  if is_old_verity_argv "${vroot_dev}"; then
-    # dm="0 2097152 verity ROOT_DEV HASH_DEV 2097152 1 \
-    # sha1 63b7ad16cb9db4b70b28593f825aa6b7825fdcf2"
-    rootfs_sectors=$(echo ${vroot_dev} | cut -f2 -d' ')
-    verity_depth=$(echo ${vroot_dev} | cut -f7 -d' ')
-    verity_algorithm=$(echo ${vroot_dev} | cut -f8 -d' ')
-    root_dev=$(echo ${vroot_dev} | cut -f4 -d ' ')
-    hash_dev=$(echo ${vroot_dev} | cut -f5 -d ' ')
-    # Hack around the fact that the signer needs to use the old version of
-    # verity to generate legacy verity kernel parameters. If we find it,
-    # we use it.
-    type -P "verity-old" &>/dev/null && verity_bin="verity-old"
-  else
-    # Key-value parameters.
-    rootfs_sectors=$(get_verity_arg "${vroot_dev}" hashstart)
-    verity_depth=0
-    verity_algorithm=$(get_verity_arg "${vroot_dev}" alg)
-    root_dev=$(get_verity_arg "${vroot_dev}" payload)
-    hash_dev=$(get_verity_arg "${vroot_dev}" hashtree)
-    salt=$(get_verity_arg "${vroot_dev}" salt)
-  fi
+  # Extract the key-value parameters from the kernel command line.
+  local rootfs_sectors=$(get_verity_arg "${vroot_dev}" hashstart)
+  local verity_algorithm=$(get_verity_arg "${vroot_dev}" alg)
+  local root_dev=$(get_verity_arg "${vroot_dev}" payload)
+  local hash_dev=$(get_verity_arg "${vroot_dev}" hashtree)
+  local salt=$(get_verity_arg "${vroot_dev}" salt)
 
   local salt_arg
   if [ -n "$salt" ]; then
@@ -219,7 +169,7 @@ calculate_rootfs_hash() {
   fi
 
   # Run the verity tool on the rootfs partition.
-  local slave=$(sudo ${verity_bin} mode=create \
+  local slave=$(sudo verity mode=create \
     alg=${verity_algorithm} \
     payload="${rootfs_image}" \
     payload_blocks=$((rootfs_sectors / 8)) \
@@ -257,7 +207,7 @@ update_rootfs_hash() {
   info "Updating rootfs hash and updating config for Kernel partitions"
 
   # If we can't find dm parameters in the kernel config, bail out now.
-  local kernel_config=$(sudo dump_kernel_config "${loop_kern}")
+  local kernel_config=$(sudo ${FUTILITY} dump_kernel_config "${loop_kern}")
   local dm_config=$(get_dmparams_from_config "${kernel_config}")
   if [ -z "${dm_config}" ]; then
     error "Couldn't grab dm_config from kernel ${loop_kern}"
@@ -308,7 +258,7 @@ update_rootfs_hash() {
   for kernelpart in 2 4; do
     loop_kern="${loopdev}p${kernelpart}"
     if ! new_kernel_config="$(
-         sudo dump_kernel_config "${loop_kern}" 2>/dev/null)" &&
+         sudo ${FUTILITY} dump_kernel_config "${loop_kern}" 2>/dev/null)" &&
        [[ "${kernelpart}" == 4 ]]; then
       # Legacy images don't have partition 4.
       info "Skipping empty kernel partition 4 (legacy images)."
@@ -326,7 +276,7 @@ update_rootfs_hash() {
       keyblock="${kern_b_keyblock}"
       priv_key="${kern_b_privkey}"
     fi
-    sudo vbutil_kernel --repack "${loop_kern}" \
+    sudo ${FUTILITY} vbutil_kernel --repack "${loop_kern}" \
       --keyblock ${keyblock} \
       --signprivate ${priv_key} \
       --version "${KERNEL_VERSION}" \
@@ -345,13 +295,13 @@ update_stateful_partition_vblock() {
   local temp_out_vb="$(make_temp_file)"
 
   local loop_kern="${loopdev}p4"
-  if [[ -z "$(sudo dump_kernel_config "${loop_kern}" 2>/dev/null)" ]]; then
+  if [[ -z "$(sudo ${FUTILITY} dump_kernel_config "${loop_kern}" 2>/dev/null)" ]]; then
     info "Building vmlinuz_hd.vblock from legacy image partition 2."
     loop_kern="${loopdev}p2"
   fi
 
   # vblock should always use kernel keyblock.
-  sudo vbutil_kernel --repack "${temp_out_vb}" \
+  sudo ${FUTILITY} vbutil_kernel --repack "${temp_out_vb}" \
     --keyblock "${KEY_DIR}/kernel.keyblock" \
     --signprivate "${KEY_DIR}/kernel_data_key.vbprivk" \
     --oldblob "${loop_kern}" \
@@ -364,7 +314,7 @@ update_stateful_partition_vblock() {
   sudo umount "${stateful_dir}"
 }
 
-# Do a sanity check on the image's rootfs
+# Do a validity check on the image's rootfs
 # ARGS: Image
 verify_image_rootfs() {
   local rootfs=$1
@@ -445,92 +395,13 @@ sign_firmware() {
   info "Signed firmware image output to ${image}"
 }
 
-# Sign nvidia lp0 firmware with the given keys.
-# Args: NV_LP0_FIRMWARE_IMAGE KEY_DIR
-sign_nv_lp0_firmware() {
-  local nv_lp0_fw_image=$1
-  local key_dir=$2
-
-  "${SCRIPT_DIR}/sign_nv_cbootimage.sh" "lp0_firmware" \
-      "${key_dir%/}/nv_pkc.pem" "${nv_lp0_fw_image}" "tegra210"
-  info "Signed nvidia lp0 firmware image output to ${nv_lp0_fw_image}"
-}
-
-# Sign a kernel in-place with the given keys.
-# Args: KERNEL_IMAGE KEY_DIR KERNEL_VERSION
-sign_kernel() {
-  local image=$1
-  local key_dir=$2
-  local kernel_version=$3
-
-  # Note: Although vbutil_kernel may correctly handle specifying the same
-  # output file as the input file, we do not want to rely on it correctly
-  # handing that. Hence, the use of a temporary file.
-  local temp_kernel=$(make_temp_file)
-
-  # Resign the kernel with new keys.
-  vbutil_kernel --repack "${temp_kernel}" \
-    --keyblock "${key_dir}/kernel.keyblock" \
-    --signprivate "${key_dir}/kernel_data_key.vbprivk" \
-    --version "${kernel_version}" \
-    --oldblob "${image}"
-
-  mv "${temp_kernel}" "${image}"
-  info "Signed kernel image output to ${image}"
-}
-
-# Sign a recovery kernel in-place with the given keys.
-# Args: KERNEL_IMAGE KEY_DIR KERNEL_VERSION
-sign_recovery_kernel() {
-  local image=$1
-  local key_dir=$2
-  local kernel_version=$3
-
-  # Note: Although vbutil_kernel may correctly handle specifying the same
-  # output file as the input file, we do not want to rely on it correctly
-  # handing that. Hence, the use of a temporary file.
-  local temp_kernel=$(make_temp_file)
-
-  # Resign the kernel with new recovery keys.
-  vbutil_kernel --repack "${temp_kernel}" \
-    --keyblock "${key_dir}/recovery_kernel.keyblock" \
-    --signprivate "${key_dir}/recovery_kernel_data_key.vbprivk" \
-    --version "${kernel_version}" \
-    --oldblob "${image}"
-
-  mv "${temp_kernel}" "${image}"
-  info "Signed recovery_kernel image output to ${image}"
-}
-
-# Get the compression algorithm used for the given CBFS file.
-# Args: INPUT_CBFS_IMAGE CBFS_FILE_NAME
-get_cbfs_compression() {
-  cbfstool "$1" print -r "FW_MAIN_A" | awk -vname="$2" '$1 == name {print $5}'
-}
-
-# Store a file in CBFS.
-# Args: INPUT_CBFS_IMAGE INPUT_FILE CBFS_FILE_NAME
-store_file_in_cbfs() {
-  local image="$1"
-  local file="$2"
-  local name="$3"
-  local compression=$(get_cbfs_compression "$1" "${name}")
-  cbfstool "${image}" remove -r "FW_MAIN_A,FW_MAIN_B" -n "${name}" || return
-  # This add can fail if
-  # 1. Size of a signature after compression is larger
-  # 2. CBFS is full
-  # These conditions extremely unlikely become true at the same time.
-  cbfstool "${image}" add -r "FW_MAIN_A,FW_MAIN_B" -t "raw" \
-    -c "${compression}" -f "${file}" -n "${name}" || return
-}
-
 # Sign a delta update payload (usually created by paygen).
 # Args: INPUT_IMAGE KEY_DIR OUTPUT_IMAGE
 sign_update_payload() {
   local image=$1
   local key_dir=$2
   local output=$3
-  local key_size key_file="${key_dir}/update_key.pem"
+  local key_output key_size key_file="${key_dir}/update_key.pem"
   # Maps key size to verified boot's algorithm id (for pad_digest_utility).
   # Hashing algorithm is always SHA-256.
   local algo algos=(
@@ -540,11 +411,11 @@ sign_update_payload() {
     [8192]=10
   )
 
-  key_size=$(openssl rsa -text -noout -in "${key_file}" | \
-    sed -n -r '1{s/Private-Key: \(([0-9]*) bit\)/\1/p}')
+  key_output=$(${FUTILITY} show "${key_file}")
+  key_size=$(echo "${key_output}" | sed -n '/Key length/s/[^0-9]*//p')
   algo=${algos[${key_size}]}
   if [[ -z ${algo} ]]; then
-    die "Unknown algorithm specified by key_size=${key_size}"
+    die "Unknown algorithm: futility output=${key_output}"
   fi
 
   pad_digest_utility ${algo} "${image}" | \
@@ -552,9 +423,9 @@ sign_update_payload() {
 }
 
 # Re-sign the firmware AU payload inside the image rootfs with a new keys.
-# Args: IMAGE
+# Args: LOOPDEV
 resign_firmware_payload() {
-  local image=$1
+  local loopdev="$1"
 
   if [ -n "${NO_FWUPDATE}" ]; then
     info "Skipping firmware update."
@@ -563,7 +434,7 @@ resign_firmware_payload() {
 
   # Grab firmware image from the autoupdate bundle (shellball).
   local rootfs_dir=$(make_temp_dir)
-  mount_image_partition ${image} 3 ${rootfs_dir}
+  mount_loop_image_partition "${loopdev}" 3 "${rootfs_dir}"
   local firmware_bundle="${rootfs_dir}/usr/sbin/chromeos-firmwareupdate"
   local shellball_dir=$(make_temp_dir)
 
@@ -617,8 +488,8 @@ resign_firmware_payload() {
         # This is what updater4.sh currently uses to make the decision.
         if [[ -e "${KEY_DIR}/loem.ini" ]]; then
           # loem.ini has the format KEY_ID_VALUE = KEY_INDEX
-          local match="$(grep -E "[0-9]+ = ${key_id}$" "${KEY_DIR}/loem.ini")"
-          local key_index="$(echo "${match}" | cut -d ' ' -f 1)"
+          local match="$(grep -E "[0-9]+ *= *${key_id}$" "${KEY_DIR}/loem.ini")"
+          local key_index="$(echo "${match}" | sed 's/ *= *.*$//g')"
           info "Detected key index from loem.ini as ${key_index} for ${key_id}"
           if [[ -z "${key_index}" ]]; then
             die "Failed to find key_id ${key_id} in loem.ini file for " \
@@ -654,6 +525,9 @@ resign_firmware_payload() {
         # Path to bios.bin.
         local bios_path="${shellball_dir}/${bios_image}"
 
+        echo "Before EC signing ${bios_path}: md5 =" \
+          $(md5sum ${bios_path} | awk '{print $1}')
+
         if [ -n "${ec_image}" ]; then
           # Path to ec.bin.
           local ec_path="${shellball_dir}/${ec_image}"
@@ -664,6 +538,10 @@ resign_firmware_payload() {
             local rw_hash="EC_RW.hash"
             # futility writes byproduct files to CWD, so we cd to temp dir.
             pushd "$(make_temp_dir)" > /dev/null
+
+            echo "Signing EC with:" ${FUTILITY} sign --type rwsig --prikey \
+              "${KEY_DIR}/key_ec_efs.vbprik2" "${ec_path}"
+
             ${FUTILITY} sign --type rwsig --prikey \
               "${KEY_DIR}/key_ec_efs.vbprik2" "${ec_path}" \
               || die "Failed to sign ${ec_path}"
@@ -679,7 +557,20 @@ resign_firmware_payload() {
           fi
         fi
 
+        echo "After EC signing ${bios_path}: md5 =" \
+          $(md5sum ${bios_path} | awk '{print $1}')
+
         # Resign bios.bin.
+        echo "Signing Bios with:" ${FUTILITY} sign \
+          --signprivate "${signprivate}" \
+          --keyblock "${keyblock}" \
+          --devsign "${devsign}" \
+          --devkeyblock "${devkeyblock}" \
+          --kernelkey "${KEY_DIR}/kernel_subkey.vbpubk" \
+          --version "${FIRMWARE_VERSION}" \
+          "${extra_args[@]}" \
+          ${bios_path} \
+          ${temp_fw}
         ${FUTILITY} sign \
           --signprivate "${signprivate}" \
           --keyblock "${keyblock}" \
@@ -691,9 +582,17 @@ resign_firmware_payload() {
           ${bios_path} \
           ${temp_fw}
 
+        echo "After Bios signing ${temp_fw}: md5 =" \
+          $(md5sum ${temp_fw} | awk '{print $1}')
 
         # For development phases, when the GBB can be updated still, set the
         # recovery and root keys in the image.
+        echo "Setting GBB with:" ${FUTILITY} gbb \
+          -s \
+          --recoverykey="${KEY_DIR}/recovery_key.vbpubk" \
+          --rootkey="${rootkey}" \
+          "${temp_fw}" \
+          "${bios_path}"
         ${FUTILITY} gbb \
           -s \
           --recoverykey="${KEY_DIR}/recovery_key.vbpubk" \
@@ -701,6 +600,9 @@ resign_firmware_payload() {
           "${temp_fw}" \
           "${bios_path}"
 
+        echo "After setting GBB on ${bios_path}: md5 =" \
+          $(md5sum ${bios_path} | awk '{print $1}')
+
         info "Signed firmware image output to ${bios_path}"
       done
       unset IFS
@@ -723,7 +625,28 @@ resign_firmware_payload() {
 
   local signer_notes="${shellball_dir}/VERSION.signer"
   echo "" >"$signer_notes"
-  info "Signed with keyset in $(readlink -f "${KEY_DIR}") ." >>"${signer_notes}"
+  echo "Signed with keyset in $(readlink -f "${KEY_DIR}") ." >>"${signer_notes}"
+  # record recovery_key
+  key="${KEY_DIR}/recovery_key.vbpubk"
+  sha1=$(${FUTILITY} vbutil_key --unpack "${key}" \
+    | grep sha1sum | cut -d" " -f9)
+  echo "recovery: ${sha1}" >>"${signer_notes}"
+  # record root_key(s)
+  if [[ -d "${shellball_keyset_dir}"  ]]; then
+    echo "List sha1sum of all loem/model's signatures:" >>"${signer_notes}"
+    for key in "${shellball_keyset_dir}"/rootkey.*; do
+      model="${key##*.}"
+      sha1=$(${FUTILITY} vbutil_key --unpack "${key}" \
+        | grep sha1sum | cut -d" " -f9)
+      echo "  ${model}: ${sha1}" >>"${signer_notes}"
+    done
+  else
+    echo "List sha1sum of single key's signature:" >>"${signer_notes}"
+    key="${KEY_DIR}/root_key.vbpubk"
+    sha1=$(${FUTILITY} vbutil_key --unpack "${key}" \
+      | grep sha1sum | cut -d" " -f9)
+    echo "  root: ${sha1}" >>"${signer_notes}"
+  fi
 
   new_shellball=$(make_temp_file)
   cp -f "${firmware_bundle}" "${new_shellball}"
@@ -733,17 +656,31 @@ resign_firmware_payload() {
   sudo chmod a+rx "${firmware_bundle}"
   # Unmount now to flush changes.
   sudo umount "${rootfs_dir}"
-  info "Re-signed firmware AU payload in ${image}"
+  info "Re-signed firmware AU payload in ${loopdev}"
+}
+
+# Remove old container key if it exists.
+# We can drop this logic once all devices that shipped R78 have gone EOL.
+# So probably in like 2025.
+remove_old_container_key() {
+  local loopdev="$1"
+
+  local rootfs_dir=$(make_temp_dir)
+  mount_loop_image_partition "${loopdev}" 3 "${rootfs_dir}"
+
+  sudo rm -f "${rootfs_dir}/usr/share/misc/oci-container-key-pub.der"
+
+  sudo umount "${rootfs_dir}"
 }
 
 # Re-sign Android image if exists.
 resign_android_image_if_exists() {
-  local image=$1
+  local loopdev="$1"
 
   local rootfs_dir=$(make_temp_dir)
-  mount_image_partition "${image}" 3 "${rootfs_dir}"
+  mount_loop_image_partition "${loopdev}" 3 "${rootfs_dir}"
 
-  local system_img="${rootfs_dir}/opt/google/containers/android/system.raw.img"
+  local system_img="$(echo "${rootfs_dir}"/opt/google/*/android/system.raw.img)"
   local arc_version=$(grep CHROMEOS_ARC_VERSION= \
     "${rootfs_dir}/etc/lsb-release" | cut -d= -f2)
   if [[ ! -e "${system_img}" || -z "${arc_version}" ]]; then
@@ -752,21 +689,97 @@ resign_android_image_if_exists() {
     return
   fi
 
+  info "Found ARC image version '${arc_version}', re-signing APKs."
+  # TODO(crbug.com/1141907): remove set -x and set +x below.
+  set -x
   "${SCRIPT_DIR}/sign_android_image.sh" "${rootfs_dir}" "${KEY_DIR}/android"
 
-  sudo umount "${rootfs_dir}"
+  if ! sudo umount "${rootfs_dir}"; then
+    error "umount ${rootfs_dir} failed"
+    sudo lsof -n "${rootfs_dir}"
+    ps auxf
+    set +x
+    return 1
+  fi
   info "Re-signed Android image"
+  set +x
 }
 
-# Sign an oci container with the given keys.
+# Sign UEFI binaries, if possible.
+# Args: LOOPDEV
+sign_uefi_binaries() {
+  local loopdev="$1"
+
+  if [[ ! -d "${KEY_DIR}/uefi" ]]; then
+    return 0
+  fi
+
+  local esp_dir
+  if ! esp_dir="$(mount_image_esp "${loopdev}")"; then
+    error "Could not mount EFI partition for signing UEFI binaries"
+    return 1
+  elif [[ -z "${esp_dir}" ]]; then
+    return 0
+  fi
+  # TODO(b/199136347): First class this as an arg and pass from config
+  # in the signing repo. This is a temporary fix to unblock reven-release.
+  if [[ "${KEY_DIR}" != *"Reven"* ]]; then
+    "${SCRIPT_DIR}/install_gsetup_certs.sh" "${esp_dir}" "${KEY_DIR}/uefi"
+  fi
+  "${SCRIPT_DIR}/sign_uefi.sh" "${esp_dir}" "${KEY_DIR}/uefi"
+  sudo umount "${esp_dir}"
+
+  local rootfs_dir="$(make_temp_dir)"
+  mount_loop_image_partition "${loopdev}" 3 "${rootfs_dir}"
+  "${SCRIPT_DIR}/sign_uefi.sh" "${rootfs_dir}/boot" "${KEY_DIR}/uefi"
+  sudo umount "${rootfs_dir}"
+
+  info "Signed UEFI binaries"
+  return 0
+}
+
+# Verify the signatures of UEFI binaries.
+# Args: LOOPDEV
+verify_uefi_signatures() {
+  local loopdev="$1"
+  local succeeded=1
+
+  if [[ ! -d "${KEY_DIR}/uefi" ]]; then
+    return 0
+  fi
+
+  local esp_dir
+  if ! esp_dir="$(mount_image_esp "${loopdev}")"; then
+    error "Could not mount EFI partition for verifying UEFI signatures"
+    return 1
+  elif [[ -z "${esp_dir}" ]]; then
+    return 0
+  fi
+  "${SCRIPT_DIR}/verify_uefi.sh" "${esp_dir}" "${esp_dir}" \
+      "${KEY_DIR}/uefi" || succeeded=0
+
+  local rootfs_dir="$(make_temp_dir)"
+  mount_loop_image_partition_ro "${loopdev}" 3 "${rootfs_dir}"
+  "${SCRIPT_DIR}/verify_uefi.sh" "${rootfs_dir}/boot" "${esp_dir}" \
+      "${KEY_DIR}/uefi" || succeeded=0
+  sudo umount "${rootfs_dir}"
+
+  sudo umount "${esp_dir}"
+
+  if [[ "${succeeded}" == "0" ]]; then
+    die "UEFI signature verification failed"
+  fi
+}
+
+# Sign a GSC firmware image with the given keys.
 # Args: CONTAINER KEY_DIR [OUTPUT_CONTAINER]
-sign_oci_container() {
+sign_gsc_firmware() {
   local image=$1
   local key_dir=$2
   local output=$3
 
-  "${SCRIPT_DIR}/sign_oci_container.sh" \
-    "${image}" "${key_dir}" --output "${output}"
+  "${SCRIPT_DIR}/sign_gsc_firmware.sh" \
+    "${image}" "${key_dir}" "${output}"
 }
 
 # Verify an image including rootfs hash using the specified keys.
@@ -784,7 +797,7 @@ verify_image() {
   local partnum
   for partnum in 2 4; do
     info "Considering Kernel partition ${partnum}"
-    kernel_config=$(sudo dump_kernel_config "${loopdev}p${partnum}")
+    kernel_config=$(sudo ${FUTILITY} dump_kernel_config "${loopdev}p${partnum}")
     local hash_image=$(make_temp_file)
     if ! calculate_rootfs_hash "${loop_rootfs}" "${kernel_config}" \
       "${hash_image}"; then
@@ -839,6 +852,8 @@ EOF
 
   verify_image_rootfs "${loop_rootfs}"
 
+  verify_uefi_signatures "${INPUT_IMAGE}"
+
   # TODO(gauravsh): Check embedded firmware AU signatures.
 }
 
@@ -850,8 +865,15 @@ update_recovery_kernel_hash() {
   local loop_kernb="${loopdev}p4"
 
   # Update the Kernel B hash in Kernel A command line
-  local old_kerna_config="$(sudo dump_kernel_config "${loop_kerna}")"
-  local new_kernb_hash=$(sudo sha1sum "${loop_kernb}" | cut -f1 -d' ')
+  local old_kerna_config="$(sudo ${FUTILITY} \
+    dump_kernel_config "${loop_kerna}")"
+  local old_kernb_hash="$(echo "$old_kerna_config" |
+    sed -nEe "s#.*kern_b_hash=([a-z0-9]*).*#\1#p")"
+  if [[ "${#old_kernb_hash}" -lt 64 ]]; then
+    local new_kernb_hash=$(sudo sha1sum "${loop_kernb}" | cut -f1 -d' ')
+  else
+    local new_kernb_hash=$(sudo sha256sum "${loop_kernb}" | cut -f1 -d' ')
+  fi
 
   new_kerna_config=$(make_temp_file)
   echo "$old_kerna_config" |
@@ -861,7 +883,7 @@ update_recovery_kernel_hash() {
   cat ${new_kerna_config}
 
   # Re-calculate kernel partition signature and command line.
-  sudo vbutil_kernel --repack "${loop_kerna}" \
+  sudo ${FUTILITY} vbutil_kernel --repack "${loop_kerna}" \
     --keyblock ${KEY_DIR}/recovery_kernel.keyblock \
     --signprivate ${KEY_DIR}/recovery_kernel_data_key.vbprivk \
     --version "${KERNEL_VERSION}" \
@@ -869,31 +891,75 @@ update_recovery_kernel_hash() {
     --config ${new_kerna_config}
 }
 
+# Re-sign miniOS kernels with new keys.
+# Args: LOOPDEV KEYBLOCK PRIVKEY
+resign_minios_kernels() {
+  local loopdev="$1"
+  local keyblock="$2"
+  local priv_key="$3"
+
+  info "Searching for miniOS kernels to resign..."
+
+  local loop_minios
+  for loop_minios in "${loopdev}p"*; do
+    local part_type_guid
+    part_type_guid=$(sudo lsblk -rnb -o PARTTYPE "${loop_minios}")
+    if [[ "${part_type_guid}" != "${MINIOS_KERNEL_GUID}" ]]; then
+      continue
+    fi
+
+    # Skip miniOS partitions which are empty. This happens when miniOS
+    # kernels aren't written to the partitions because the feature is not
+    # enabled.
+    if ! sudo "${FUTILITY}" dump_kernel_config "${loop_minios}"; then
+      info "Skipping empty miniOS partition ${loop_minios}."
+      continue
+    fi
+
+    # Delay checking that keyblock and private key exist until we are certain
+    # of a valid miniOS partition.  Images that don't support miniOS might not
+    # provide these.  (This check is repeated twice, but that's okay.)
+    if [[ ! -e "${keyblock}" ]]; then
+      error "Resign miniOS: keyblock doesn't exist: ${keyblock}"
+      return 1
+    fi
+    if [[ ! -e "${priv_key}" ]]; then
+      error "Resign miniOS: private key doesn't exist: ${priv_key}"
+      return 1
+    fi
+
+    # Assume this is a miniOS kernel.
+    local minios_kernel_version=$((KERNEL_VERSION >> 24))
+    if sudo ${FUTILITY} vbutil_kernel --repack "${loop_minios}" \
+        --keyblock "${keyblock}" \
+        --signprivate "${priv_key}" \
+        --version "${minios_kernel_version}" \
+        --oldblob "${loop_minios}"; then
+      info "Resign miniOS ${loop_minios}: done"
+    else
+      error "Resign miniOS ${loop_minios}: failed"
+      return 1
+    fi
+  done
+}
+
 # Update the legacy bootloader templates in EFI partition if available.
-# Args: IMAGE_BIN KERNEL
+# Args: LOOPDEV KERNEL
 update_legacy_bootloader() {
-  local image="$1"
+  local loopdev="$1"
   local loop_kern="$2"
 
-  local esp_partnum=12
-  local esp_offset=$(( $(partoffset "${image}" "${esp_partnum}") * 512 ))
-  # Check if the image has an ESP partition.
-  if [[ "${esp_offset}" == "0" ]]; then
-    info "Not updating legacy bootloader configs: ${image}"
+  local esp_dir
+  if ! esp_dir="$(mount_image_esp "${loopdev}")"; then
+    error "Could not mount EFI partition for updating legacy bootloader cfg."
+    return 1
+  elif [[ -z "${esp_dir}" ]]; then
+    info "Not updating legacy bootloader configs: ${loopdev}"
     return 0
   fi
 
-  local esp_dir="$(make_temp_dir)"
-  # We use the 'unsafe' variant because the EFI system partition is vfat type
-  # and can be mounted in RW mode.
-  if ! _mount_image_partition_retry "${image}" "${esp_partnum}" \
-                                    "${esp_dir}"; then
-    error "Could not mount EFI partition for updating legacy bootloader cfg."
-    return 1
-  fi
-
   # If we can't find the dm parameter in the kernel config, bail out now.
-  local kernel_config=$(sudo dump_kernel_config "${loop_kern}")
+  local kernel_config=$(sudo ${FUTILITY} dump_kernel_config "${loop_kern}")
   local root_hexdigest="$(get_hash_from_config "${kernel_config}")"
   if [[ -z "${root_hexdigest}" ]]; then
     error "Couldn't grab root_digest from kernel partition ${loop_kern}"
@@ -924,7 +990,7 @@ update_legacy_bootloader() {
 
 # Sign an image file with proper keys.
 # Args: IMAGE_TYPE INPUT OUTPUT DM_PARTNO KERN_A_KEYBLOCK KERN_A_PRIVKEY \
-#       KERN_B_KEYBLOCK KERN_B_PRIVKEY
+#       KERN_B_KEYBLOCK KERN_B_PRIVKEY MINIOS_KEYBLOCK MINIOS_PRIVKEY
 #
 # A ChromiumOS image file (INPUT) always contains 2 partitions (kernel A & B).
 # This function will rebuild hash data by DM_PARTNO, resign kernel partitions by
@@ -941,15 +1007,20 @@ sign_image_file() {
   local kernA_privkey="$6"
   local kernB_keyblock="$7"
   local kernB_privkey="$8"
+  local minios_keyblock="$9"
+  local minios_privkey="${10}"
 
   info "Preparing ${image_type} image..."
   cp --sparse=always "${input}" "${output}"
 
   local loopdev=$(loopback_partscan "${output}")
   local loop_kern="${loopdev}p${dm_partno}"
+  local loop_rootfs="${loopdev}p3"
 
-  resign_firmware_payload "${output}"
-  resign_android_image_if_exists "${output}"
+  resign_firmware_payload "${loopdev}"
+  remove_old_container_key "${loopdev}"
+  resign_android_image_if_exists "${loopdev}"
+  sign_uefi_binaries "${loopdev}"
   # We do NOT strip /boot for factory installer, since some devices need it to
   # boot EFI. crbug.com/260512 would obsolete this requirement.
   #
@@ -958,11 +1029,11 @@ sign_image_file() {
   # /boot in rootfs to update kernel.  We infer the BIOS type from the kernel
   # config.
   local loop_kerna="${loopdev}p2"
-  local kerna_config="$(sudo dump_kernel_config "${loop_kerna}")"
+  local kerna_config="$(sudo ${FUTILITY} dump_kernel_config "${loop_kerna}")"
   if [[ "${image_type}" != "factory_install" &&
         " ${kerna_config} " != *" cros_legacy "* &&
         " ${kerna_config} " != *" cros_efi "* ]]; then
-    "${SCRIPT_DIR}/strip_boot_from_image.sh" --image "${output}"
+    "${SCRIPT_DIR}/strip_boot_from_image.sh" --image "${loop_rootfs}"
   fi
   update_rootfs_hash "${loopdev}" "${loop_kern}" \
     "${kernA_keyblock}" "${kernA_privkey}" \
@@ -971,7 +1042,11 @@ sign_image_file() {
   if [[ "${image_type}" == "recovery" ]]; then
     update_recovery_kernel_hash "${loopdev}"
   fi
-  if ! update_legacy_bootloader "${output}" "${loop_kern}"; then
+  if ! resign_minios_kernels "${loopdev}" "${minios_keyblock}" \
+      "${minios_privkey}"; then
+    return 1
+  fi
+  if ! update_legacy_bootloader "${loopdev}" "${loop_kern}"; then
     # Error is already logged.
     return 1
   fi
@@ -985,7 +1060,7 @@ dump_config)
   loopdev=$(loopback_partscan "${INPUT_IMAGE}")
   for partnum in 2 4; do
     info "kernel config in partition number ${partnum}:"
-    sudo dump_kernel_config "${loopdev}p${partnum}"
+    sudo ${FUTILITY} dump_kernel_config "${loopdev}p${partnum}"
     echo
   done
   exit 0
@@ -1015,52 +1090,36 @@ info "Using firmware version: ${FIRMWARE_VERSION}"
 info "Using kernel version: ${KERNEL_VERSION}"
 
 # Make all modifications on output copy.
-if [[ "${TYPE}" == "ssd" || "${TYPE}" == "base" ]]; then
-  sign_image_file "SSD" "${INPUT_IMAGE}" "${OUTPUT_IMAGE}" 2 \
-    "${KEY_DIR}/kernel.keyblock" "${KEY_DIR}/kernel_data_key.vbprivk" \
-    "${KEY_DIR}/kernel.keyblock" "${KEY_DIR}/kernel_data_key.vbprivk"
-elif [[ "${TYPE}" == "usb" ]]; then
-  sign_image_file "USB" "${INPUT_IMAGE}" "${OUTPUT_IMAGE}" 2 \
-    "${KEY_DIR}/recovery_kernel.keyblock" \
-    "${KEY_DIR}/recovery_kernel_data_key.vbprivk" \
+if [[ "${TYPE}" == "base" ]]; then
+  sign_image_file "base" "${INPUT_IMAGE}" "${OUTPUT_IMAGE}" 2 \
     "${KEY_DIR}/kernel.keyblock" \
-    "${KEY_DIR}/kernel_data_key.vbprivk"
+    "${KEY_DIR}/kernel_data_key.vbprivk" \
+    "${KEY_DIR}/kernel.keyblock" \
+    "${KEY_DIR}/kernel_data_key.vbprivk" \
+    "${KEY_DIR}/minios_kernel.keyblock" \
+    "${KEY_DIR}/minios_kernel_data_key.vbprivk"
 elif [[ "${TYPE}" == "recovery" ]]; then
   sign_image_file "recovery" "${INPUT_IMAGE}" "${OUTPUT_IMAGE}" 4 \
     "${KEY_DIR}/recovery_kernel.keyblock" \
     "${KEY_DIR}/recovery_kernel_data_key.vbprivk" \
     "${KEY_DIR}/kernel.keyblock" \
-    "${KEY_DIR}/kernel_data_key.vbprivk"
-elif [[ "${TYPE}" == "factory" ]] || [[ "${TYPE}" == "install" ]]; then
+    "${KEY_DIR}/kernel_data_key.vbprivk" \
+    "${KEY_DIR}/minios_kernel.keyblock" \
+    "${KEY_DIR}/minios_kernel_data_key.vbprivk"
+elif [[ "${TYPE}" == "factory" ]]; then
   sign_image_file "factory_install" "${INPUT_IMAGE}" "${OUTPUT_IMAGE}" 2 \
     "${KEY_DIR}/installer_kernel.keyblock" \
     "${KEY_DIR}/installer_kernel_data_key.vbprivk" \
     "${KEY_DIR}/kernel.keyblock" \
-    "${KEY_DIR}/kernel_data_key.vbprivk"
+    "${KEY_DIR}/kernel_data_key.vbprivk" \
+    "${KEY_DIR}/minios_kernel.keyblock" \
+    "${KEY_DIR}/minios_kernel_data_key.vbprivk"
 elif [[ "${TYPE}" == "firmware" ]]; then
   if [[ -e "${KEY_DIR}/loem.ini" ]]; then
     die "LOEM signing not implemented yet for firmware images"
   fi
   cp ${INPUT_IMAGE} ${OUTPUT_IMAGE}
   sign_firmware ${OUTPUT_IMAGE} ${KEY_DIR} ${FIRMWARE_VERSION}
-elif [[ "${TYPE}" == "nv_lp0_firmware" ]]; then
-  if [[ -e "${KEY_DIR}/loem.ini" ]]; then
-    die "LOEM signing not implemented yet for nv_lp0_firmware images"
-  fi
-  cp "${INPUT_IMAGE}" "${OUTPUT_IMAGE}"
-  sign_nv_lp0_firmware "${OUTPUT_IMAGE}" "${KEY_DIR}"
-elif [[ "${TYPE}" == "kernel" ]]; then
-  if [[ -e "${KEY_DIR}/loem.ini" ]]; then
-    die "LOEM signing not implemented yet for kernel images"
-  fi
-  cp "${INPUT_IMAGE}" "${OUTPUT_IMAGE}"
-  sign_kernel "${OUTPUT_IMAGE}" "${KEY_DIR}" "${KERNEL_VERSION}"
-elif [[ "${TYPE}" == "recovery_kernel" ]]; then
-  if [[ -e "${KEY_DIR}/loem.ini" ]]; then
-    die "LOEM signing not implemented yet for recovery_kernel images"
-  fi
-  cp "${INPUT_IMAGE}" "${OUTPUT_IMAGE}"
-  sign_recovery_kernel "${OUTPUT_IMAGE}" "${KEY_DIR}" "${KERNEL_VERSION}"
 elif [[ "${TYPE}" == "update_payload" ]]; then
   sign_update_payload ${INPUT_IMAGE} ${KEY_DIR} ${OUTPUT_IMAGE}
 elif [[ "${TYPE}" == "accessory_usbpd" ]]; then
@@ -1069,17 +1128,23 @@ elif [[ "${TYPE}" == "accessory_usbpd" ]]; then
     KEY_NAME="${KEY_DIR}/key"
   fi
   cp "${INPUT_IMAGE}" "${OUTPUT_IMAGE}"
-  futility sign --type usbpd1 --pem "${KEY_NAME}.pem" "${OUTPUT_IMAGE}"
+  ${FUTILITY} sign --type usbpd1 --pem "${KEY_NAME}.pem" "${OUTPUT_IMAGE}"
 elif [[ "${TYPE}" == "accessory_rwsig" ]]; then
-  KEY_NAME="${KEY_DIR}/key_$(basename $(dirname ${INPUT_IMAGE}))"
+  # If one key is present in this container, assume it's the right one.
+  # See crbug.com/863464
   if [[ ! -e "${KEY_NAME}.vbprik2" ]]; then
-    KEY_NAME="${KEY_DIR}/key"
+    KEYS=( "${KEY_DIR}"/*.vbprik2 )
+    if [[ ${#KEYS[@]} -eq 1 ]]; then
+      KEY_NAME="${KEYS[0]}"
+    else
+      die "Expected exactly one key present in keyset for accessory_rwsig"
+    fi
   fi
   cp "${INPUT_IMAGE}" "${OUTPUT_IMAGE}"
-  futility sign --type rwsig --prikey "${KEY_NAME}.vbprik2" \
+  ${FUTILITY} sign --type rwsig --prikey "${KEY_NAME}" \
            --version "${FIRMWARE_VERSION}" "${OUTPUT_IMAGE}"
-elif [[ "${TYPE}" == "oci-container" ]]; then
-  sign_oci_container "${INPUT_IMAGE}" "${KEY_DIR}" "${OUTPUT_IMAGE}"
+elif [[ "${TYPE}" == "gsc_firmware" ]]; then
+  sign_gsc_firmware "${INPUT_IMAGE}" "${KEY_DIR}" "${OUTPUT_IMAGE}"
 else
   die "Invalid type ${TYPE}"
 fi
