@@ -1,5 +1,4 @@
 /*
- * This file is part of the libpayload project.
  *
  * Copyright (C) 2010 Patrick Georgi
  * Copyright (C) 2013 secunet Security Networks AG
@@ -88,7 +87,7 @@ xhci_init_cycle_ring(transfer_ring_t *const tr, const size_t ring_size)
 }
 
 /* On Panther Point: switch ports shared with EHCI to xHCI */
-#if IS_ENABLED(CONFIG_LP_USB_PCI)
+#if CONFIG(LP_USB_PCI)
 static void
 xhci_switch_ppt_ports(pcidev_t addr)
 {
@@ -112,7 +111,7 @@ xhci_switch_ppt_ports(pcidev_t addr)
 }
 #endif
 
-#if IS_ENABLED(CONFIG_LP_USB_PCI)
+#if CONFIG(LP_USB_PCI)
 /* On Panther Point: switch all ports back to EHCI */
 static void
 xhci_switchback_ppt_ports(pcidev_t addr)
@@ -130,7 +129,12 @@ xhci_switchback_ppt_ports(pcidev_t addr)
 static long
 xhci_handshake(volatile u32 *const reg, u32 mask, u32 wait_for, long timeout_us)
 {
-	while ((*reg & mask) != wait_for && timeout_us--) udelay(1);
+	if (timeout_us <= 0)
+		return 0;
+	while ((*reg & mask) != wait_for && timeout_us != 0) {
+		--timeout_us;
+		udelay(1);
+	}
 	return timeout_us;
 }
 
@@ -185,26 +189,27 @@ xhci_init (unsigned long physical_bar)
 		goto _free_xhci;
 	}
 
-	xhci->capreg	= phys_to_virt(physical_bar);
-	xhci->opreg	= ((void *)xhci->capreg) + xhci->capreg->caplength;
-	xhci->hcrreg	= ((void *)xhci->capreg) + xhci->capreg->rtsoff;
-	xhci->dbreg	= ((void *)xhci->capreg) + xhci->capreg->dboff;
-	xhci_debug("regbase: 0x%"PRIx32"\n", physical_bar);
-	xhci_debug("caplen:  0x%"PRIx32"\n", xhci->capreg->caplength);
+	xhci->capreg = phys_to_virt(physical_bar);
+	xhci->opreg = phys_to_virt(physical_bar) + CAP_GET(CAPLEN, xhci->capreg);
+	xhci->hcrreg = phys_to_virt(physical_bar) + xhci->capreg->rtsoff;
+	xhci->dbreg = phys_to_virt(physical_bar) + xhci->capreg->dboff;
+
+	xhci_debug("regbase: 0x%"PRIxPTR"\n", physical_bar);
+	xhci_debug("caplen:  0x%"PRIx32"\n", CAP_GET(CAPLEN, xhci->capreg));
 	xhci_debug("rtsoff:  0x%"PRIx32"\n", xhci->capreg->rtsoff);
 	xhci_debug("dboff:   0x%"PRIx32"\n", xhci->capreg->dboff);
 
 	xhci_debug("hciversion: %"PRIx8".%"PRIx8"\n",
-		   xhci->capreg->hciver_hi, xhci->capreg->hciver_lo);
-	if ((xhci->capreg->hciversion < 0x96) ||
-			(xhci->capreg->hciversion > 0x110)) {
+		   CAP_GET(CAPVER_HI, xhci->capreg), CAP_GET(CAPVER_LO, xhci->capreg));
+	if ((CAP_GET(CAPVER, xhci->capreg) < 0x96) ||
+	    (CAP_GET(CAPVER, xhci->capreg) > 0x120)) {
 		xhci_debug("Unsupported xHCI version\n");
 		goto _free_xhci;
 	}
 
 	xhci_debug("context size: %dB\n", CTXSIZE(xhci));
-	xhci_debug("maxslots: 0x%02lx\n", xhci->capreg->MaxSlots);
-	xhci_debug("maxports: 0x%02lx\n", xhci->capreg->MaxPorts);
+	xhci_debug("maxslots: 0x%02"PRIx32"\n", CAP_GET(MAXSLOTS, xhci->capreg));
+	xhci_debug("maxports: 0x%02"PRIx32"\n", CAP_GET(MAXPORTS, xhci->capreg));
 	const unsigned pagesize = xhci->opreg->pagesize << 12;
 	xhci_debug("pagesize: 0x%04x\n", pagesize);
 
@@ -213,7 +218,8 @@ xhci_init (unsigned long physical_bar)
 	 * structures at first and can still chicken out easily if we run out
 	 * of memory.
 	 */
-	xhci->max_slots_en = xhci->capreg->MaxSlots & CONFIG_LP_MASK_MaxSlotsEn;
+	xhci->max_slots_en = CAP_GET(MAXSLOTS, xhci->capreg) &
+		CONFIG_LP_MASK_MaxSlotsEn;
 	xhci->dcbaa = xhci_align(64, (xhci->max_slots_en + 1) * sizeof(u64));
 	xhci->dev = malloc((xhci->max_slots_en + 1) * sizeof(*xhci->dev));
 	if (!xhci->dcbaa || !xhci->dev) {
@@ -227,8 +233,9 @@ xhci_init (unsigned long physical_bar)
 	 * Let dcbaa[0] point to another array of pointers, sp_ptrs.
 	 * The pointers therein point to scratchpad buffers (pages).
 	 */
-	const size_t max_sp_bufs = xhci->capreg->Max_Scratchpad_Bufs_Hi << 5 |
-				   xhci->capreg->Max_Scratchpad_Bufs_Lo;
+	const size_t max_sp_bufs =
+		CAP_GET(MAX_SCRATCH_BUFS_HI, xhci->capreg) << 5 |
+		CAP_GET(MAX_SCRATCH_BUFS_LO, xhci->capreg);
 	xhci_debug("max scratchpad bufs: 0x%zx\n", max_sp_bufs);
 	if (max_sp_bufs) {
 		const size_t sp_ptrs_size = max_sp_bufs * sizeof(u64);
@@ -297,23 +304,27 @@ _free_xhci:
 	return NULL;
 }
 
-#if IS_ENABLED(CONFIG_LP_USB_PCI)
+#if CONFIG(LP_USB_PCI)
 hci_t *
 xhci_pci_init (pcidev_t addr)
 {
 	u32 reg_addr;
 	hci_t *controller;
 
-	reg_addr = pci_read_config32 (addr, 0x10) & ~0xf;
-	if (pci_read_config32 (addr, 0x14) > 0) {
+	reg_addr = pci_read_config32(addr, PCI_BASE_ADDRESS_0) &
+		   PCI_BASE_ADDRESS_MEM_MASK;
+	if (pci_read_config32(addr, PCI_BASE_ADDRESS_1) > 0)
 		fatal("We don't do 64bit addressing.\n");
-	}
 
 	controller = xhci_init((unsigned long)reg_addr);
 	if (controller) {
+		xhci_t *xhci = controller->instance;
 		controller->pcidev = addr;
 
 		xhci_switch_ppt_ports(addr);
+
+		/* Set up any quirks for controller root hub */
+		xhci->roothub->quirks = pci_quirk_check(addr);
 	}
 
 	return controller;
@@ -336,7 +347,7 @@ xhci_reset(hci_t *const controller)
 	 * Without this delay, the subsequent HC register access,
 	 * may result in a system hang very rarely.
 	 */
-	if (IS_ENABLED(CONFIG_LP_ARCH_X86))
+	if (CONFIG(LP_ARCH_X86))
 		mdelay(1);
 
 	xhci_debug("Resetting controller... ");
@@ -363,7 +374,7 @@ xhci_reinit (hci_t *controller)
 
 	/* Initialize command ring */
 	xhci_init_cycle_ring(&xhci->cr, COMMAND_RING_SIZE);
-	xhci_debug("command ring @%p (0x%08x)\n",
+	xhci_debug("command ring @%p (0x%08"PRIxPTR")\n",
 		   xhci->cr.ring, virt_to_phys(xhci->cr.ring));
 	xhci->opreg->crcr_lo = virt_to_phys(xhci->cr.ring) | CRCR_RCS;
 	xhci->opreg->crcr_hi = 0;
@@ -373,10 +384,11 @@ xhci_reinit (hci_t *controller)
 
 	/* Initialize event ring */
 	xhci_reset_event_ring(&xhci->er);
-	xhci_debug("event ring @%p (0x%08x)\n",
+	xhci_debug("event ring @%p (0x%08"PRIxPTR")\n",
 		   xhci->er.ring, virt_to_phys(xhci->er.ring));
-	xhci_debug("ERST Max: 0x%lx ->  0x%lx entries\n",
-		   xhci->capreg->ERST_Max, 1 << xhci->capreg->ERST_Max);
+	xhci_debug("ERST Max: 0x%"PRIx32" ->  0x%x entries\n",
+		   CAP_GET(ERST_MAX, xhci->capreg),
+		   1 << CAP_GET(ERST_MAX, xhci->capreg));
 	memset((void*)xhci->ev_ring_table, 0x00, sizeof(erst_entry_t));
 	xhci->ev_ring_table[0].seg_base_lo = virt_to_phys(xhci->er.ring);
 	xhci->ev_ring_table[0].seg_base_hi = 0;
@@ -403,9 +415,12 @@ xhci_reinit (hci_t *controller)
 		xhci_post_command(xhci);
 
 		/* Wait for result in event ring */
-		xhci_wait_for_command_done(xhci, cmd, 1);
-		xhci_debug("Command ring is %srunning\n",
-			   (xhci->opreg->crcr_lo & CRCR_CRR) ? "" : "not ");
+		int cc = xhci_wait_for_command_done(xhci, cmd, 1);
+
+		xhci_debug("Command ring is %srunning: cc: %d\n",
+			   (xhci->opreg->crcr_lo & CRCR_CRR) ? "" : "not ", cc);
+		if (cc != CC_SUCCESS)
+			xhci_debug("noop command failed.\n");
 	}
 #endif
 }
@@ -423,14 +438,15 @@ xhci_shutdown(hci_t *const controller)
 	xhci_t *const xhci = XHCI_INST(controller);
 	xhci_stop(controller);
 
-#if IS_ENABLED(CONFIG_LP_USB_PCI)
+#if CONFIG(LP_USB_PCI)
         if (controller->pcidev)
 		xhci_switchback_ppt_ports(controller->pcidev);
 #endif
 
 	if (xhci->sp_ptrs) {
-		size_t max_sp_bufs = xhci->capreg->Max_Scratchpad_Bufs_Hi << 5 |
-				     xhci->capreg->Max_Scratchpad_Bufs_Lo;
+		const size_t max_sp_bufs =
+			CAP_GET(MAX_SCRATCH_BUFS_HI, xhci->capreg) << 5 |
+			CAP_GET(MAX_SCRATCH_BUFS_LO, xhci->capreg);
 		for (i = 0; i < max_sp_bufs; ++i) {
 			if (xhci->sp_ptrs[i])
 				free(phys_to_virt(xhci->sp_ptrs[i]));
@@ -560,7 +576,7 @@ xhci_enqueue_td(transfer_ring_t *const tr, const int ep, const size_t mps,
 			cur_length = length;
 			packets = 0;
 			length = 0;
-		} else if (!IS_ENABLED(CONFIG_LP_USB_XHCI_MTK_QUIRK)) {
+		} else if (!CONFIG(LP_USB_XHCI_MTK_QUIRK)) {
 			packets -= (residue + cur_length) / mps;
 			residue = (residue + cur_length) % mps;
 			length -= cur_length;
@@ -573,7 +589,7 @@ xhci_enqueue_td(transfer_ring_t *const tr, const int ep, const size_t mps,
 		TRB_SET(TDS, trb, MIN(TRB_MAX_TD_SIZE, packets));
 		TRB_SET(CH, trb, 1);
 
-		if (length && IS_ENABLED(CONFIG_LP_USB_XHCI_MTK_QUIRK)) {
+		if (length && CONFIG(LP_USB_XHCI_MTK_QUIRK)) {
 			/*
 			 * For MTK's xHCI controller, TDS defines a number of
 			 * packets that remain to be transferred for a TD after
