@@ -1,5 +1,4 @@
 /*
- * This file is part of the libpayload project.
  *
  * Copyright (C) 2010 Patrick Georgi
  * Copyright (C) 2013 secunet Security Networks AG
@@ -205,7 +204,7 @@ typedef transfer_ring_t command_ring_t;
 #define SC_UADDR_LEN		8
 #define SC_STATE_FIELD		f4		/* STATE - Slot State */
 #define SC_STATE_START		27
-#define SC_STATE_LEN		8
+#define SC_STATE_LEN		5
 #define SC_MASK(tok)		MASK(SC_##tok##_START, SC_##tok##_LEN)
 #define SC_GET(tok, sc)		(((sc)->SC_##tok##_FIELD & SC_MASK(tok)) \
 				 >> SC_##tok##_START)
@@ -274,7 +273,6 @@ typedef volatile struct epctx {
 } epctx_t;
 
 #define NUM_EPS 32
-#define CTXSIZE(xhci) ((xhci)->capreg->csz ? 64 : 32)
 
 typedef union devctx {
 	/* set of pointers, so we can dynamically adjust Slot/EP context size */
@@ -321,63 +319,62 @@ typedef struct erst_entry {
 	u32 rsvd;
 } erst_entry_t;
 
+#define CAP_CAPLEN_FIELD		hciparams
+#define CAP_CAPLEN_START		0
+#define CAP_CAPLEN_LEN			8
+#define CAP_CAPVER_FIELD		hciparams
+#define CAP_CAPVER_START		16
+#define CAP_CAPVER_LEN			16
+#define CAP_CAPVER_HI_FIELD		hciparams
+#define CAP_CAPVER_HI_START		24
+#define CAP_CAPVER_HI_LEN		8
+#define CAP_CAPVER_LO_FIELD		hciparams
+#define CAP_CAPVER_LO_START		16
+#define CAP_CAPVER_LO_LEN		8
+#define CAP_MAXSLOTS_FIELD		hcsparams1
+#define CAP_MAXSLOTS_START		0
+#define CAP_MAXSLOTS_LEN		7
+#define CAP_MAXINTRS_FIELD		hcsparams1
+#define CAP_MAXINTRS_START		7
+#define CAP_MAXINTRS_LEN		11
+#define CAP_MAXPORTS_FIELD		hcsparams1
+#define CAP_MAXPORTS_START		24
+#define CAP_MAXPORTS_LEN		8
+#define CAP_IST_FIELD			hcsparams2
+#define CAP_IST_START			0
+#define CAP_IST_LEN			4
+#define CAP_ERST_MAX_FIELD		hcsparams2
+#define CAP_ERST_MAX_START		4
+#define CAP_ERST_MAX_LEN		4
+#define CAP_MAX_SCRATCH_BUFS_HI_FIELD	hcsparams2
+#define CAP_MAX_SCRATCH_BUFS_HI_START	21
+#define CAP_MAX_SCRATCH_BUFS_HI_LEN	5
+#define CAP_MAX_SCRATCH_BUFS_LO_FIELD	hcsparams2
+#define CAP_MAX_SCRATCH_BUFS_LO_START	27
+#define CAP_MAX_SCRATCH_BUFS_LO_LEN	5
+#define CAP_U1_LATENCY_FIELD		hcsparams3
+#define CAP_U1_LATENCY_START		0
+#define CAP_U1_LATENCY_LEN		8
+#define CAP_U2_LATENCY_FIELD		hcsparams3
+#define CAP_U2_LATENCY_START		16
+#define CAP_U2_LATENCY_LEN		16
+#define CAP_CSZ_FIELD			hccparams
+#define CAP_CSZ_START			2
+#define CAP_CSZ_LEN			1
+
+#define CAP_MASK(tok)		MASK(CAP_##tok##_START, CAP_##tok##_LEN)
+#define CAP_GET(tok, cap)	((read32(&(cap)->CAP_##tok##_FIELD) & CAP_MASK(tok)) \
+				 >> CAP_##tok##_START)
+
+#define CTXSIZE(xhci) (CAP_GET(CSZ, (xhci)->capreg) ? 64 : 32)
+
 typedef struct xhci {
-	/* capreg is read-only, so no need for volatile,
-	   and thus 32bit accesses can be assumed. */
 	struct capreg {
-		u8 caplength;
-		u8 res1;
-		union {
-			u16 hciversion;
-			struct {
-				u8 hciver_lo;
-				u8 hciver_hi;
-			} __packed;
-		} __packed;
-		union {
-			u32 hcsparams1;
-			struct {
-				unsigned long MaxSlots:7;
-				unsigned long MaxIntrs:11;
-				unsigned long:6;
-				unsigned long MaxPorts:8;
-			} __packed;
-		} __packed;
-		union {
-			u32 hcsparams2;
-			struct {
-				unsigned long IST:4;
-				unsigned long ERST_Max:4;
-				unsigned long:13;
-				unsigned long Max_Scratchpad_Bufs_Hi:5;
-				unsigned long SPR:1;
-				unsigned long Max_Scratchpad_Bufs_Lo:5;
-			} __packed;
-		} __packed;
-		union {
-			u32 hcsparams3;
-			struct {
-				unsigned long u1latency:8;
-				unsigned long:8;
-				unsigned long u2latency:16;
-			} __packed;
-		} __packed;
-		union {
-			u32 hccparams;
-			struct {
-				unsigned long ac64:1;
-				unsigned long bnc:1;
-				unsigned long csz:1;
-				unsigned long ppc:1;
-				unsigned long pind:1;
-				unsigned long lhrc:1;
-				unsigned long ltc:1;
-				unsigned long nss:1;
-				unsigned long:4;
-				unsigned long MaxPSASize:4;
-				unsigned long xECP:16;
-			} __packed;
-		} __packed;
+		u32 hciparams;
+		u32 hcsparams1;
+		u32 hcsparams2;
+		u32 hcsparams3;
+		u32 hccparams;
 		u32 dboff;
 		u32 rtsoff;
 	} __packed *capreg;
@@ -386,34 +383,34 @@ typedef struct xhci {
 	   volatile means that the compiler seeks byte writes if possible,
 	   making bitfields unusable for MMIO register blocks. Yay C :-( */
 	volatile struct opreg {
-		u32 usbcmd;
+		u32 usbcmd; /* 0x00 */
 #define USBCMD_RS (1 << 0)
 #define USBCMD_HCRST (1 << 1)
 #define USBCMD_INTE (1 << 2)
-		u32 usbsts;
+		u32 usbsts; /* 0x04 */
 #define USBSTS_HCH (1 << 0)
 #define USBSTS_HSE (1 << 2)
 #define USBSTS_EINT (1 << 3)
 #define USBSTS_PCD (1 << 4)
 #define USBSTS_CNR (1 << 11)
 #define USBSTS_PRSRV_MASK ((1 << 1) | 0xffffe000)
-		u32 pagesize;
-		u8 res1[0x13-0x0c+1];
-		u32 dnctrl;
-		u32 crcr_lo;
-		u32 crcr_hi;
+		u32 pagesize; /* 0x08 */
+		u8 res1[0x13-0x0c+1]; /* 0x0C */
+		u32 dnctrl; /* 0x14 */
+		u32 crcr_lo; /* 0x18 */
+		u32 crcr_hi; /* 0x1C */
 #define CRCR_RCS (1 << 0)
 #define CRCR_CS (1 << 1)
 #define CRCR_CA (1 << 2)
 #define CRCR_CRR (1 << 3)
-		u8 res2[0x2f-0x20+1];
-		u32 dcbaap_lo;
-		u32 dcbaap_hi;
-		u32 config;
+		u8 res2[0x2f-0x20+1]; /* 0x20 */
+		u32 dcbaap_lo; /* 0x30 */
+		u32 dcbaap_hi; /* 0x34 */
+		u32 config; /* 0x38 */
 #define CONFIG_LP_MASK_MaxSlotsEn 0xff
-		u8 res3[0x3ff-0x3c+1];
+		u8 res3[0x3ff-0x3c+1]; /* 0x3C */
 		struct {
-			u32 portsc;
+			u32 portsc; /* 0x400 + 4 * port */
 #define PORTSC_CCS (1 << 0)
 #define PORTSC_PED (1 << 1)
 	// BIT 2 rsvdZ
@@ -443,9 +440,9 @@ typedef struct xhci {
 #define PORTSC_DR (1 << 30)
 #define PORTSC_WPR (1 << 31)
 #define PORTSC_RW_MASK (PORTSC_PR | PORTSC_PLS_MASK | PORTSC_PP | PORTSC_PIC_MASK | PORTSC_LWS | PORTSC_WCE | PORTSC_WDE | PORTSC_WOE)
-			u32 portpmsc;
-			u32 portli;
-			u32 res;
+			u32 portpmsc; /* 0x404 + 4 * port */
+			u32 portli; /* 0x408 + 4 * port */
+			u32 res; /* 0x40C + 4 * port */
 		} __packed prs[];
 	} __packed *opreg;
 
@@ -518,7 +515,6 @@ static inline int xhci_ep_id(const endpoint_t *const ep) {
 	return ((ep->endpoint & 0x7f) * 2) + (ep->direction != OUT);
 }
 
-
 #ifdef XHCI_DUMPS
 void xhci_dump_slotctx(const slotctx_t *);
 void xhci_dump_epctx(const epctx_t *);
