@@ -1,6 +1,15 @@
+#ifndef _STDDEF_H
+#define _STDDEF_H
 
 #include <arch/types.h>
 
+#ifndef __WCHAR_TYPE__
+#define __WCHAR_TYPE__ int
+#endif
+#ifndef __cplusplus
+typedef __WCHAR_TYPE__ wchar_t;
+#endif
+
 #ifndef __SIZE_TYPE__
 #define __SIZE_TYPE__ unsigned long
 #endif
@@ -29,6 +38,11 @@ typedef __SIZE_TYPE__ ssize_t;
 #define MHz (1000*KHz)
 #define GHz (1000*MHz)
 
+#define NSECS_PER_SEC 1000000000
 #define USECS_PER_SEC 1000000
 #define MSECS_PER_SEC 1000
+#define NSECS_PER_MSEC (NSECS_PER_SEC / MSECS_PER_SEC)
+#define NSECS_PER_USEC (NSECS_PER_SEC / USECS_PER_SEC)
 #define USECS_PER_MSEC (USECS_PER_SEC / MSECS_PER_SEC)
+
+#endif
