@@ -1,21 +1,16 @@
-/*
- * This file is part of the coreboot project.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- */
+/* SPDX-License-Identifier: GPL-2.0-only */
 
+#include <arch/cpu.h>
+#include <arch/exception.h>
+#include <commonlib/helpers.h>
 #include <console/console.h>
 #include <console/streams.h>
+#include <cpu/x86/cr.h>
+#include <cpu/x86/lapic.h>
+#include <stdint.h>
 #include <string.h>
 
-#if IS_ENABLED(CONFIG_GDB_STUB)
+#if CONFIG(GDB_STUB)
 
 /* BUFMAX defines the maximum number of characters in inbound/outbound buffers.
  * At least NUM_REGBYTES*2 are needed for register packets
@@ -185,8 +180,6 @@ static uint32_t gdb_stub_registers[NUM_REGS];
 #define GDB_EXC_SOFTWARE       149 /* Software generated exception */
 #define GDB_EXC_BREAKPOINT     150 /* Breakpoint */
 
-
-
 static unsigned char exception_to_signal[] = {
 	[0]  = GDB_SIGFPE,  /* divide by zero */
 	[1]  = GDB_SIGTRAP, /* debug exception */
@@ -227,7 +220,6 @@ static const char hexchars[] = "0123456789abcdef";
 static char in_buffer[BUFMAX];
 static char out_buffer[BUFMAX];
 
-
 static inline void stub_putc(int ch)
 {
 	gdb_tx_byte(ch);
@@ -288,7 +280,6 @@ static void copy_to_hex(char *buf, void *addr, unsigned long count)
 	*buf = 0;
 }
 
-
 /* convert the hex array pointed to by buf into binary to be placed in mem */
 /* return a pointer to the character AFTER the last byte written */
 static void copy_from_hex(void *addr, char *buf, unsigned long count)
@@ -303,7 +294,6 @@ static void copy_from_hex(void *addr, char *buf, unsigned long count)
 	}
 }
 
-
 /* scan for the sequence $<data>#<checksum>	*/
 
 static int get_packet(char *buffer)
@@ -387,7 +377,7 @@ void x86_exception(struct eregs *info);
 
 void x86_exception(struct eregs *info)
 {
-#if IS_ENABLED(CONFIG_GDB_STUB)
+#if CONFIG(GDB_STUB)
 	int signo;
 	memcpy(gdb_stub_registers, info, 8*sizeof(uint32_t));
 	gdb_stub_registers[PC] = info->eip;
@@ -494,17 +484,52 @@ void x86_exception(struct eregs *info)
 		put_packet(out_buffer);
 	}
 #else /* !CONFIG_GDB_STUB */
-#define MDUMP_SIZE 0x80
+
+	int logical_processor = 0;
+	u32 apic_id = CONFIG(SMP) ? lapicid() : 0;
+
+#if ENV_RAMSTAGE
+	logical_processor = cpu_index();
+#endif
+	u8 *code;
+#if ENV_X86_64
+#define MDUMP_SIZE 0x100
 	printk(BIOS_EMERG,
-		"Unexpected Exception: %d @ %02x:%08x - Halting\n"
-		"Code: %d eflags: %08x\n"
+		"CPU Index %d - APIC %d Unexpected Exception:\n"
+		"%lld @ %02llx:%016llx - Halting\n"
+		"Code: %lld rflags: %016llx cr2: %016llx\n"
+		"rax: %016llx rbx: %016llx\n"
+		"rcx: %016llx rdx: %016llx\n"
+		"rdi: %016llx rsi: %016llx\n"
+		"rbp: %016llx rsp: %016llx\n"
+		"r08: %016llx r09: %016llx\n"
+		"r10: %016llx r11: %016llx\n"
+		"r12: %016llx r13: %016llx\n"
+		"r14: %016llx r15: %016llx\n",
+		logical_processor, apic_id,
+		info->vector, info->cs, info->rip,
+		info->error_code, info->rflags, read_cr2(),
+		info->rax, info->rbx, info->rcx, info->rdx,
+		info->rdi, info->rsi, info->rbp, info->rsp,
+		info->r8, info->r9, info->r10, info->r11,
+		info->r12, info->r13, info->r14, info->r15);
+	code = (u8 *)((uintptr_t)info->rip - (MDUMP_SIZE >> 2));
+#else
+#define MDUMP_SIZE 0x80
+
+	printk(BIOS_EMERG,
+		"CPU Index %d - APIC %d Unexpected Exception:"
+		"%d @ %02x:%08x - Halting\n"
+		"Code: %d eflags: %08x cr2: %08x\n"
 		"eax: %08x ebx: %08x ecx: %08x edx: %08x\n"
 		"edi: %08x esi: %08x ebp: %08x esp: %08x\n",
+		logical_processor, apic_id,
 		info->vector, info->cs, info->eip,
-		info->error_code, info->eflags,
+		info->error_code, info->eflags, read_cr2(),
 		info->eax, info->ebx, info->ecx, info->edx,
 		info->edi, info->esi, info->ebp, info->esp);
-	u8 *code = (u8 *)((uintptr_t)info->eip - (MDUMP_SIZE >> 1));
+	code = (u8 *)((uintptr_t)info->eip - (MDUMP_SIZE >> 1));
+#endif
 	/* Align to 8-byte boundary please, and print eight bytes per row.
 	 * This is done to make DRAM burst timing/reordering errors more
 	 * evident from the looking at the dump */
@@ -515,6 +540,121 @@ void x86_exception(struct eregs *info)
 			printk(BIOS_EMERG, "\n%p:\t", code + i);
 		printk(BIOS_EMERG, "%.2x ", code[i]);
 	}
-	die("");
+
+	/* Align to 4-byte boundary and up the stack. */
+	u32 *ptr = (u32 *)(ALIGN_DOWN((uintptr_t)info->esp, sizeof(u32)) + MDUMP_SIZE - 4);
+	for (i = 0; i < MDUMP_SIZE / sizeof(u32); ++i, --ptr) {
+		printk(BIOS_EMERG, "\n%p:\t0x%08x", ptr, *ptr);
+		if ((uintptr_t)ptr == info->ebp)
+			printk(BIOS_EMERG, " <-ebp");
+		else if ((uintptr_t)ptr == info->esp)
+			printk(BIOS_EMERG, " <-esp");
+	}
+
+	die("\n");
 #endif
 }
+
+#define GATE_P		(1 << 15)
+#define GATE_DPL(x)	(((x) & 0x3) << 13)
+#define GATE_SIZE_16	(0 << 11)
+#define GATE_SIZE_32	(1 << 11)
+
+#define IGATE_FLAGS (GATE_P | GATE_DPL(0) | GATE_SIZE_32 | (0x6 << 8))
+
+struct intr_gate {
+	uint16_t offset_0;
+	uint16_t segsel;
+	uint16_t flags;
+	uint16_t offset_1;
+#if ENV_X86_64
+	uint32_t offset_2;
+	uint32_t reserved;
+#endif
+} __packed;
+
+/* Even though the vecX symbols are interrupt entry points just treat them
+   like data to more easily get the pointer values in C. Because IDT entries
+   format splits the offset field up, one can't use the linker to resolve
+   parts of a relocation on x86 ABI. An array of pointers is used to gather
+   the symbols. The IDT is initialized at runtime when exception_init() is
+   called. */
+extern u8 vec0[], vec1[], vec2[], vec3[], vec4[], vec5[], vec6[], vec7[];
+extern u8 vec8[], vec9[], vec10[], vec11[], vec12[], vec13[], vec14[], vec15[];
+extern u8 vec16[], vec17[], vec18[], vec19[];
+
+static const uintptr_t intr_entries[] = {
+	(uintptr_t)vec0, (uintptr_t)vec1, (uintptr_t)vec2, (uintptr_t)vec3,
+	(uintptr_t)vec4, (uintptr_t)vec5, (uintptr_t)vec6, (uintptr_t)vec7,
+	(uintptr_t)vec8, (uintptr_t)vec9, (uintptr_t)vec10, (uintptr_t)vec11,
+	(uintptr_t)vec12, (uintptr_t)vec13, (uintptr_t)vec14, (uintptr_t)vec15,
+	(uintptr_t)vec16, (uintptr_t)vec17, (uintptr_t)vec18, (uintptr_t)vec19,
+};
+
+static struct intr_gate idt[ARRAY_SIZE(intr_entries)] __aligned(8);
+
+static inline uint16_t get_cs(void)
+{
+	uint16_t segment;
+
+	asm volatile (
+		"mov	%%cs, %0\n"
+		: "=r" (segment)
+		:
+		: "memory"
+	);
+
+	return segment;
+}
+
+struct lidtarg {
+	uint16_t limit;
+#if ENV_X86_32
+	uint32_t base;
+#else
+	uint64_t base;
+#endif
+} __packed;
+
+/* This global is for src/cpu/x86/lapic/secondary.S usage which is only
+   used during ramstage. */
+struct lidtarg idtarg;
+
+static void load_idt(void *table, size_t sz)
+{
+	struct lidtarg lidtarg = {
+		.limit = sz - 1,
+		.base = (uintptr_t)table,
+	};
+
+	asm volatile (
+		"lidt	%0"
+		:
+		: "m" (lidtarg)
+		: "memory"
+	);
+
+	if (ENV_RAMSTAGE)
+		memcpy(&idtarg, &lidtarg, sizeof(idtarg));
+}
+
+asmlinkage void exception_init(void)
+{
+	int i;
+	uint16_t segment;
+
+	segment = get_cs();
+
+	/* Initialize IDT. */
+	for (i = 0; i < ARRAY_SIZE(idt); i++) {
+		idt[i].offset_0 = intr_entries[i];
+		idt[i].segsel = segment;
+		idt[i].flags = IGATE_FLAGS;
+		idt[i].offset_1 = intr_entries[i] >> 16;
+#if ENV_X86_64
+		idt[i].offset_2 = intr_entries[i] >> 32;
+#endif
+	}
+
+	load_idt(idt, sizeof(idt));
+}
