@@ -1,32 +1,21 @@
-/*
- * This file is part of the coreboot project.
- *
- * Copyright (C) 2014 Sage Electronic Engineering, LLC.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- */
+/* SPDX-License-Identifier: GPL-2.0-only */
 
 #include <console/console.h>
 #include <device/path.h>
 #include <device/pci_ids.h>
-#include <cpu/cpu.h>
+#include <arch/ioapic.h>
 #include <arch/smp/mpspec.h>
 #include <string.h>
 #include <arch/cpu.h>
+#include <cpu/cpu.h>
 #include <cpu/x86/lapic.h>
 #include <drivers/generic/ioapic/chip.h>
 
 /* Initialize the specified "mc" struct with initial values. */
-void mptable_init(struct mp_config_table *mc, u32 lapic_addr)
+void mptable_init(struct mp_config_table *mc)
 {
 	int i;
+	u32 lapic_addr = cpu_get_lapic_addr();
 
 	memset(mc, 0, sizeof(*mc));
 
@@ -162,14 +151,12 @@ void smp_write_processors(struct mp_config_table *mc)
 	unsigned int apic_version;
 	unsigned int cpu_features;
 	unsigned int cpu_feature_flags;
-	struct cpuid_result result;
 	struct device *cpu;
 
 	boot_apic_id = lapicid();
 	apic_version = lapic_read(LAPIC_LVR) & 0xff;
-	result = cpuid(1);
-	cpu_features = result.eax;
-	cpu_feature_flags = result.edx;
+	cpu_features = cpu_get_cpuid();
+	cpu_feature_flags = cpu_get_feature_flags_edx();
 	/* order the output of the cpus to fix a bug in kernel 2.6.11 */
 	for (order_id = 0; order_id < 256; order_id++) {
 		for (cpu = all_devices; cpu; cpu = cpu->next) {
@@ -235,6 +222,14 @@ void smp_write_ioapic(struct mp_config_table *mc,
 	smp_add_mpc_entry(mc, sizeof(*mpc));
 }
 
+u8 smp_write_ioapic_from_hw(struct mp_config_table *mc, void *apicaddr)
+{
+	u8 id = get_ioapic_id(apicaddr);
+	u8 ver = get_ioapic_version(apicaddr);
+	smp_write_ioapic(mc, id, ver, apicaddr);
+	return id;
+}
+
 /*
  * Type 3: I/O Interrupt Table Entries:
  * Entry Type, Int Type, Int Polarity, Int Level,
@@ -256,12 +251,6 @@ void smp_write_intsrc(struct mp_config_table *mc,
 	mpc->mpc_dstapic = dstapic;
 	mpc->mpc_dstirq = dstirq;
 	smp_add_mpc_entry(mc, sizeof(*mpc));
-#ifdef DEBUG_MPTABLE
-	printk(BIOS_DEBUG,
-		"add intsrc srcbus 0x%x srcbusirq 0x%x, dstapic 0x%x, dstirq 0x%x\n",
-				srcbus, srcbusirq, dstapic, dstirq);
-	hexdump(__func__, mpc, sizeof(*mpc));
-#endif
 }
 
 /*
@@ -523,38 +512,55 @@ void *mptable_finalize(struct mp_config_table *mc)
 	return smp_next_mpe_entry(mc);
 }
 
-unsigned long __attribute__((weak)) write_smp_table(unsigned long addr)
+static const struct device *find_next_ioapic(unsigned int last_ioapic_id)
+{
+	const struct device *dev;
+	const struct device *result = NULL;
+	unsigned int ioapic_id = MAX_APICS;
+
+	for (dev = all_devices; dev; dev = dev->next) {
+		if (dev->path.type == DEVICE_PATH_IOAPIC &&
+		    dev->path.ioapic.ioapic_id > last_ioapic_id &&
+		    dev->path.ioapic.ioapic_id <= ioapic_id) {
+			result = dev;
+		}
+	}
+	return result;
+}
+
+unsigned long __weak write_smp_table(unsigned long addr)
 {
 	struct drivers_generic_ioapic_config *ioapic_config;
 	struct mp_config_table *mc;
 	int isa_bus, pin, parentpin;
-	struct device *dev;
-	struct device *parent;
-	struct device *oldparent;
+	const struct device *dev;
+	const struct device *parent;
+	const struct device *oldparent;
 	void *tmp, *v;
 	int isaioapic = -1, have_fixed_entries;
-	struct pci_irq_info *pci_irq_info;
+	const struct pci_irq_info *pci_irq_info;
+	unsigned int ioapic_id = 0;
 
 	v = smp_write_floating_table(addr, 0);
 	mc = (void *)(((char *)v) + SMP_FLOATING_TABLE_LEN);
 
-	mptable_init(mc, LOCAL_APIC_ADDR);
+	mptable_init(mc);
 
 	smp_write_processors(mc);
 
 	mptable_write_buses(mc, NULL, &isa_bus);
 
-	for (dev = all_devices; dev; dev = dev->next) {
-		if (dev->path.type != DEVICE_PATH_IOAPIC)
-			continue;
-
+	while ((dev = find_next_ioapic(ioapic_id))) {
 		ioapic_config = dev->chip_info;
 		if (!ioapic_config) {
 			printk(BIOS_ERR, "%s has no config, ignoring\n",
 				dev_path(dev));
+			ioapic_id++;
 			continue;
 		}
-		smp_write_ioapic(mc, dev->path.ioapic.ioapic_id,
+
+		ioapic_id = dev->path.ioapic.ioapic_id;
+		smp_write_ioapic(mc, ioapic_id,
 				     ioapic_config->version,
 				     ioapic_config->base);
 
