@@ -1,15 +1,4 @@
-/*
- * This file is part of the coreboot project.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- */
+/* SPDX-License-Identifier: GPL-2.0-only */
 
 #ifndef COMMONLIB_COREBOOT_TABLES_H
 #define COMMONLIB_COREBOOT_TABLES_H
@@ -44,6 +33,66 @@
  * table entries and be backwards compatible, but it is not required.
  */
 
+enum {
+	LB_TAG_UNUSED			= 0x0000,
+	LB_TAG_MEMORY			= 0x0001,
+	LB_TAG_HWRPB			= 0x0002,
+	LB_TAG_MAINBOARD		= 0x0003,
+	LB_TAG_VERSION			= 0x0004,
+	LB_TAG_EXTRA_VERSION		= 0x0005,
+	LB_TAG_BUILD			= 0x0006,
+	LB_TAG_COMPILE_TIME		= 0x0007,
+	LB_TAG_COMPILE_BY		= 0x0008,
+	LB_TAG_COMPILE_HOST		= 0x0009,
+	LB_TAG_COMPILE_DOMAIN		= 0x000a,
+	LB_TAG_COMPILER			= 0x000b,
+	LB_TAG_LINKER			= 0x000c,
+	LB_TAG_ASSEMBLER		= 0x000d,
+	LB_TAG_SERIAL			= 0x000f,
+	LB_TAG_CONSOLE			= 0x0010,
+	LB_TAG_FORWARD			= 0x0011,
+	LB_TAG_FRAMEBUFFER		= 0x0012,
+	LB_TAG_GPIO			= 0x0013,
+	LB_TAG_TIMESTAMPS		= 0x0016,
+	LB_TAG_CBMEM_CONSOLE		= 0x0017,
+	LB_TAG_MRC_CACHE		= 0x0018,
+	LB_TAG_VBNV			= 0x0019,
+	LB_TAG_VBOOT_HANDOFF		= 0x0020,  /* deprecated */
+	LB_TAG_X86_ROM_MTRR		= 0x0021,
+	LB_TAG_DMA			= 0x0022,
+	LB_TAG_RAM_OOPS			= 0x0023,
+	LB_TAG_ACPI_GNVS		= 0x0024,
+	LB_TAG_BOARD_ID			= 0x0025,  /* deprecated */
+	LB_TAG_VERSION_TIMESTAMP	= 0x0026,
+	LB_TAG_WIFI_CALIBRATION		= 0x0027,
+	LB_TAG_RAM_CODE			= 0x0028,  /* deprecated */
+	LB_TAG_SPI_FLASH		= 0x0029,
+	LB_TAG_SERIALNO			= 0x002a,
+	LB_TAG_MTC			= 0x002b,
+	LB_TAG_VPD			= 0x002c,
+	LB_TAG_SKU_ID			= 0x002d,  /* deprecated */
+	LB_TAG_BOOT_MEDIA_PARAMS	= 0x0030,
+	LB_TAG_CBMEM_ENTRY		= 0x0031,
+	LB_TAG_TSC_INFO			= 0x0032,
+	LB_TAG_MAC_ADDRS		= 0x0033,
+	LB_TAG_VBOOT_WORKBUF		= 0x0034,
+	LB_TAG_MMC_INFO			= 0x0035,
+	LB_TAG_TCPA_LOG			= 0x0036,
+	LB_TAG_FMAP			= 0x0037,
+	LB_TAG_PLATFORM_BLOB_VERSION	= 0x0038,
+	LB_TAG_SMMSTOREV2		= 0x0039,
+	LB_TAG_TPM_PPI_HANDOFF		= 0x003a,
+	LB_TAG_BOARD_CONFIG		= 0x0040,
+	LB_TAG_ACPI_CNVS		= 0x0041,
+	LB_TAG_TYPE_C_INFO		= 0x0042,
+	/* The following options are CMOS-related */
+	LB_TAG_CMOS_OPTION_TABLE	= 0x00c8,
+	LB_TAG_OPTION			= 0x00c9,
+	LB_TAG_OPTION_ENUM		= 0x00ca,
+	LB_TAG_OPTION_DEFAULTS		= 0x00cb,
+	LB_TAG_OPTION_CHECKSUM		= 0x00cc,
+};
+
 /* Since coreboot is usually compiled 32bit, gcc will align 64bit
  * types to 32bit boundaries. If the coreboot table is dumped on a
  * 64bit system, a uint64_t would be aligned to 64bit boundaries,
@@ -97,10 +146,6 @@ struct lb_record {
 	uint32_t size;		/* size of record (in bytes) */
 };
 
-#define LB_TAG_UNUSED		0x0000
-
-#define LB_TAG_MEMORY		0x0001
-
 struct lb_memory_range {
 	struct lb_uint64 start;
 	struct lb_uint64 size;
@@ -120,14 +165,12 @@ struct lb_memory {
 	struct lb_memory_range map[0];
 };
 
-#define LB_TAG_HWRPB		0x0002
 struct lb_hwrpb {
 	uint32_t tag;
 	uint32_t size;
 	uint64_t hwrpb;
 };
 
-#define LB_TAG_MAINBOARD	0x0003
 struct lb_mainboard {
 	uint32_t tag;
 	uint32_t size;
@@ -136,33 +179,20 @@ struct lb_mainboard {
 	uint8_t  strings[0];
 };
 
-#define LB_TAG_VERSION		0x0004
-#define LB_TAG_EXTRA_VERSION	0x0005
-#define LB_TAG_BUILD		0x0006
-#define LB_TAG_COMPILE_TIME	0x0007
-#define LB_TAG_COMPILE_BY	0x0008
-#define LB_TAG_COMPILE_HOST	0x0009
-#define LB_TAG_COMPILE_DOMAIN	0x000a
-#define LB_TAG_COMPILER		0x000b
-#define LB_TAG_LINKER		0x000c
-#define LB_TAG_ASSEMBLER	0x000d
 struct lb_string {
 	uint32_t tag;
 	uint32_t size;
 	uint8_t  string[0];
 };
 
-#define LB_TAG_VERSION_TIMESTAMP	0x0026
 struct lb_timestamp {
 	uint32_t tag;
 	uint32_t size;
 	uint32_t timestamp;
 };
 
-
 /* 0xe is taken by v3 */
 
-#define LB_TAG_SERIAL		0x000f
 struct lb_serial {
 	uint32_t tag;
 	uint32_t size;
@@ -189,7 +219,6 @@ struct lb_serial {
 	uint32_t uart_pci_addr;
 };
 
-#define LB_TAG_CONSOLE		0x0010
 struct lb_console {
 	uint32_t tag;
 	uint32_t size;
@@ -204,14 +233,63 @@ struct lb_console {
 #define LB_TAG_CONSOLE_EHCI		5
 #define LB_TAG_CONSOLE_SERIAL8250MEM	6
 
-#define LB_TAG_FORWARD		0x0011
 struct lb_forward {
 	uint32_t tag;
 	uint32_t size;
 	uint64_t forward;
 };
 
-#define LB_TAG_FRAMEBUFFER	0x0012
+/**
+ * coreboot framebuffer
+ *
+ * The coreboot framebuffer uses a very common format usually referred
+ * to as "linear framebuffer":
+ *
+ * The first pixel of the framebuffer is the upper left corner, its
+ * address is given by `physical_address`.
+ *
+ * Each pixel is represented by exactly `bits_per_pixel` bits. If a
+ * pixel (or a color component therein) doesn't fill a whole byte or
+ * doesn't start on a byte boundary, it starts at the least signifi-
+ * cant bit not occupied by the previous pixel (or color component).
+ * Pixels (or color components) that span multiple bytes always start
+ * in the byte with the lowest address.
+ *
+ * The framebuffer provides a visible rectangle of `x_resolution` *
+ * `y_resolution` pixels. However, the lines always start at a byte
+ * boundary given by `bytes_per_line`, which may leave a gap after
+ * each line of pixels. Thus, the data for a pixel with the coordi-
+ * nates (x, y) from the upper left corner always starts at
+ *
+ *   physical_address + y * bytes_per_line + x * bits_per_pixel / 8
+ *
+ * `bytes_per_line` is always big enough to hold `x_resolution`
+ * pixels. It can, however, be arbitrarily higher (e.g. to fulfill
+ * hardware constraints or for optimization purposes). The size of
+ * the framebuffer is always `y_resolution * bytes_per_line`.
+ *
+ * The coreboot framebuffer only supports RGB color formats. The
+ * position and size of each color component are specified indivi-
+ * dually by <color>_mask_pos and <color>_mask_size. To allow byte
+ * or word aligned pixels, a fourth (padding) component may be
+ * specified by `reserved_mask_pos` and `reserved_mask_size`.
+ *
+ * Software utilizing the coreboot framebuffer shall consider all
+ * fields described above. It may, however, only implement a subset
+ * of the possible color formats.
+ */
+
+/*
+ * Framebuffer orientation, matches drm_connector.h drm_panel_orientation in the
+ * Linux kernel.
+ */
+enum lb_fb_orientation {
+	LB_FB_ORIENTATION_NORMAL = 0,
+	LB_FB_ORIENTATION_BOTTOM_UP = 1,
+	LB_FB_ORIENTATION_LEFT_UP = 2,
+	LB_FB_ORIENTATION_RIGHT_UP = 3,
+};
+
 struct lb_framebuffer {
 	uint32_t tag;
 	uint32_t size;
@@ -229,10 +307,9 @@ struct lb_framebuffer {
 	uint8_t blue_mask_size;
 	uint8_t reserved_mask_pos;
 	uint8_t reserved_mask_size;
+	uint8_t orientation;
 };
 
-#define LB_TAG_GPIO	0x0013
-
 struct lb_gpio {
 	uint32_t port;
 	uint32_t polarity;
@@ -251,12 +328,6 @@ struct lb_gpios {
 	struct lb_gpio gpios[0];
 };
 
-#define LB_TAG_VDAT		0x0015
-#define LB_TAG_VBNV		0x0019
-#define LB_TAB_VBOOT_HANDOFF	0x0020
-#define LB_TAB_DMA		0x0022
-#define LB_TAG_RAM_OOPS		0x0023
-#define LB_TAG_MTC		0x002b
 struct lb_range {
 	uint32_t tag;
 	uint32_t size;
@@ -267,12 +338,6 @@ struct lb_range {
 
 void lb_ramoops(struct lb_header *header);
 
-#define LB_TAG_TIMESTAMPS	0x0016
-#define LB_TAG_CBMEM_CONSOLE	0x0017
-#define LB_TAG_MRC_CACHE	0x0018
-#define LB_TAG_ACPI_GNVS	0x0024
-#define LB_TAG_WIFI_CALIBRATION	0x0027
-#define LB_TAG_VPD		0x002c
 struct lb_cbmem_ref {
 	uint32_t tag;
 	uint32_t size;
@@ -280,7 +345,6 @@ struct lb_cbmem_ref {
 	uint64_t cbmem_addr;
 };
 
-#define LB_TAG_X86_ROM_MTRR	0x0021
 struct lb_x86_rom_mtrr {
 	uint32_t tag;
 	uint32_t size;
@@ -288,26 +352,28 @@ struct lb_x86_rom_mtrr {
 	uint32_t index;
 };
 
-#define LB_TAG_BOARD_ID		0x0025
-#define LB_TAG_RAM_CODE		0x0028
-#define LB_TAG_SKU_ID		0x002d
-
-struct lb_strapping_id {
-	uint32_t tag;
+/* Memory map windows to translate addresses between SPI flash space and host address space. */
+struct flash_mmap_window {
+	uint32_t flash_base;
+	uint32_t host_base;
 	uint32_t size;
-	uint32_t id_code;
 };
 
-#define LB_TAG_SPI_FLASH	0x0029
 struct lb_spi_flash {
 	uint32_t tag;
 	uint32_t size;
 	uint32_t flash_size;
 	uint32_t sector_size;
 	uint32_t erase_cmd;
+	/*
+	 * Number of mmap windows used by the platform to decode addresses between SPI flash
+	 * space and host address space. This determines the number of entries in mmap_table.
+	 */
+
+	uint32_t mmap_count;
+	struct flash_mmap_window mmap_table[0];
 };
 
-#define LB_TAG_BOOT_MEDIA_PARAMS 0x0030
 struct lb_boot_media_params {
 	uint32_t tag;
 	uint32_t size;
@@ -321,7 +387,6 @@ struct lb_boot_media_params {
 /*
  * There can be more than one of these records as there is one per cbmem entry.
  */
-#define LB_TAG_CBMEM_ENTRY 0x0031
 struct lb_cbmem_entry {
 	uint32_t tag;
 	uint32_t size;
@@ -331,7 +396,6 @@ struct lb_cbmem_entry {
 	uint32_t id;
 };
 
-#define LB_TAG_TSC_INFO 0x0032
 struct lb_tsc_info {
 	uint32_t tag;
 	uint32_t size;
@@ -339,12 +403,52 @@ struct lb_tsc_info {
 	uint32_t freq_khz;
 };
 
-#define LB_TAG_MAC_ADDRS	0x0033
 struct mac_address {
 	uint8_t mac_addr[6];
 	uint8_t pad[2];		/* Pad it to 8 bytes to keep it simple. */
 };
 
+struct lb_mmc_info {
+	uint32_t tag;
+	uint32_t size;
+	/*
+	 * Passes the early mmc status to payload to indicate if firmware
+	 * successfully sent CMD0, CMD1 to the card or not. In case of
+	 * success, the payload can skip the first step of the initialization
+	 * sequence which is to send CMD0, and instead start by sending CMD1
+	 * as described in Jedec Standard JESD83-B1 section 6.4.3.
+	 * passes 1 on success
+	 */
+	int32_t early_cmd1_status;
+};
+
+/*
+ * USB Type-C Port Information
+ * This record contains board-specific type-c port information.
+ * There will be one record per type-C port.
+ * Orientation fields should be of type enum type_c_orientation.
+ */
+enum type_c_orientation {
+	/* The orientation of the signal follows the orientation of the CC lines. */
+	TYPEC_ORIENTATION_NONE,
+	/* The orientation of the signal is fixed to follow CC1 */
+	TYPEC_ORIENTATION_NORMAL,
+	/* The orientation of the signal is fixed to follow CC2 */
+	TYPEC_ORIENTATION_REVERSE,
+};
+
+struct type_c_port_info {
+	uint8_t usb2_port_number;
+	uint8_t usb3_port_number;
+	uint8_t sbu_orientation;
+	uint8_t data_orientation;
+};
+
+struct type_c_info {
+	uint32_t port_count;
+	struct type_c_port_info port_info[0];
+};
+
 struct lb_macs {
 	uint32_t tag;
 	uint32_t size;
@@ -352,26 +456,33 @@ struct lb_macs {
 	struct mac_address mac_addrs[0];
 };
 
-#define LB_TAG_SERIALNO		0x002a
+struct lb_board_config {
+	uint32_t tag;
+	uint32_t size;
+
+	struct lb_uint64 fw_config;
+	uint32_t board_id;
+	uint32_t ram_code;
+	uint32_t sku_id;
+};
+
 #define MAX_SERIALNO_LENGTH	32
 
-/* The following structures are for the cmos definitions table */
-#define LB_TAG_CMOS_OPTION_TABLE 200
-/* cmos header record */
+/* The following structures are for the CMOS definitions table */
+/* CMOS header record */
 struct cmos_option_table {
 	uint32_t tag;               /* CMOS definitions table type */
 	uint32_t size;               /* size of the entire table */
 	uint32_t header_length;      /* length of header */
 };
 
-/* cmos entry record
+/* CMOS entry record
  * This record is variable length.  The name field may be
  * shorter than CMOS_MAX_NAME_LENGTH. The entry may start
  * anywhere in the byte, but can not span bytes unless it
  * starts at the beginning of the byte and the length is
  * fills complete bytes.
  */
-#define LB_TAG_OPTION 201
 struct cmos_entries {
 	uint32_t tag;                /* entry type */
 	uint32_t size;               /* length of this record */
@@ -384,12 +495,10 @@ struct cmos_entries {
 					       variable length int aligned */
 };
 
-
-/* cmos enumerations record
+/* CMOS enumerations record
  * This record is variable length.  The text field may be
  * shorter than CMOS_MAX_TEXT_LENGTH.
  */
-#define LB_TAG_OPTION_ENUM 202
 struct cmos_enums {
 	uint32_t tag;		     /* enumeration type */
 	uint32_t size;		     /* length of this record */
@@ -400,10 +509,9 @@ struct cmos_enums {
 						variable length int aligned */
 };
 
-/* cmos defaults record
- * This record contains default settings for the cmos ram.
+/* CMOS defaults record
+ * This record contains default settings for the CMOS ram.
  */
-#define LB_TAG_OPTION_DEFAULTS 203
 struct cmos_defaults {
 	uint32_t tag;                /* default type */
 	uint32_t size;               /* length of this record */
@@ -413,7 +521,6 @@ struct cmos_defaults {
 	uint8_t default_set[CMOS_IMAGE_BUFFER_SIZE]; /* default settings */
 };
 
-#define LB_TAG_OPTION_CHECKSUM 204
 struct	cmos_checksum {
 	uint32_t tag;
 	uint32_t size;
@@ -428,4 +535,43 @@ struct	cmos_checksum {
 #define CHECKSUM_PCBIOS	1
 };
 
+/* SMMSTOREv2 record
+ * This record contains information to use SMMSTOREv2.
+ */
+
+struct lb_smmstorev2 {
+	uint32_t tag;
+	uint32_t size;
+	uint32_t num_blocks;		/* Number of writeable blocks in SMM */
+	uint32_t block_size;		/* Size of a block in byte. Default: 64 KiB */
+	uint32_t mmap_addr;		/* MMIO address of the store for read only access */
+	uint32_t com_buffer;		/* Physical address of the communication buffer */
+	uint32_t com_buffer_size;	/* Size of the communication buffer in bytes */
+	uint8_t apm_cmd;		/* The command byte to write to the APM I/O port */
+	uint8_t unused[3];		/* Set to zero */
+};
+
+enum lb_tmp_ppi_tpm_version {
+	LB_TPM_VERSION_UNSPEC = 0,
+	LB_TPM_VERSION_TPM_VERSION_1_2,
+	LB_TPM_VERSION_TPM_VERSION_2,
+};
+
+/*
+ * Handoff buffer for TPM Physical Presence Interface.
+ * * ppi_address   Pointer to PPI buffer shared with ACPI
+ *                 The layout of the buffer matches the QEMU virtual memory device
+ *                 that is generated by QEMU.
+ *                 See files 'hw/i386/acpi-build.c' and 'include/hw/acpi/tpm.h'
+ *                 for details.
+ * * tpm_version   TPM version: 1 for TPM1.2, 2 for TPM2.0
+ * * ppi_version   BCD encoded version of TPM PPI interface
+ */
+struct lb_tpm_physical_presence {
+	uint32_t tag;
+	uint32_t size;
+	uint32_t ppi_address;	/* Address of ACPI PPI communication buffer */
+	uint8_t tpm_version;	/* 1: TPM1.2, 2: TPM2.0 */
+	uint8_t ppi_version;	/* BCD encoded */
+} __packed;
 #endif
