@@ -1,44 +1,26 @@
+/* SPDX-License-Identifier: GPL-2.0-or-later */
 /*
- * Copyright 2011, Marvell Semiconductor Inc.
- * Lei Wen <leiwen@marvell.com>
- *
- * Copyright 2017 Intel Corporation
- *
  * Secure Digital (SD) Host Controller interface specific code
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of
- * the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
  */
 
-#include <assert.h>
 #include "bouncebuf.h"
-#include <console/console.h>
 #include <commonlib/sd_mmc_ctrlr.h>
 #include <commonlib/sdhci.h>
 #include <commonlib/storage.h>
 #include <delay.h>
 #include <endian.h>
-#include <halt.h>
 #include "sdhci.h"
 #include "sd_mmc.h"
 #include "storage.h"
-#include <string.h>
 #include <timer.h>
 #include <commonlib/stdlib.h>
 
-#define DMA_AVAILABLE	((CONFIG_SDHCI_ADMA_IN_BOOTBLOCK && ENV_BOOTBLOCK) \
-			|| (CONFIG_SDHCI_ADMA_IN_VERSTAGE && ENV_VERSTAGE) \
-			|| (CONFIG_SDHCI_ADMA_IN_ROMSTAGE && ENV_ROMSTAGE) \
+#define DMA_AVAILABLE	((CONFIG(SDHCI_ADMA_IN_BOOTBLOCK) && ENV_BOOTBLOCK) \
+			|| (CONFIG(SDHCI_ADMA_IN_VERSTAGE) && ENV_SEPARATE_VERSTAGE) \
+			|| (CONFIG(SDHCI_ADMA_IN_ROMSTAGE) && ENV_ROMSTAGE) \
 			|| ENV_POSTCAR || ENV_RAMSTAGE)
 
-__attribute__((weak)) void *dma_malloc(size_t length_in_bytes)
+__weak void *dma_malloc(size_t length_in_bytes)
 {
 	return malloc(length_in_bytes);
 }
@@ -153,7 +135,7 @@ static int sdhci_send_command_bounced(struct sd_mmc_ctrlr *ctrlr,
 	sdhci_writel(sdhci_ctrlr, SDHCI_INT_ALL_MASK, SDHCI_INT_STATUS);
 	mask = SDHCI_CMD_INHIBIT | SDHCI_DATA_INHIBIT;
 
-	/* We shouldn't wait for data inihibit for stop commands, even
+	/* We shouldn't wait for data inhibit for stop commands, even
 	   though they might use busy signaling */
 	if (cmd->flags & CMD_FLAG_IGNORE_INHIBIT)
 		mask &= ~SDHCI_DATA_INHIBIT;
@@ -278,20 +260,20 @@ static int sdhci_send_command_bounced(struct sd_mmc_ctrlr *ctrlr,
 	return CARD_COMM_ERR;
 }
 
-__attribute__((weak)) void sdhc_log_command(struct mmc_command *cmd)
+__weak void sdhc_log_command(struct mmc_command *cmd)
 {
 }
 
-__attribute__((weak)) void sdhc_log_command_issued(void)
+__weak void sdhc_log_command_issued(void)
 {
 }
 
-__attribute__((weak)) void sdhc_log_response(uint32_t entries,
+__weak void sdhc_log_response(uint32_t entries,
 	uint32_t *response)
 {
 }
 
-__attribute__((weak)) void sdhc_log_ret(int ret)
+__weak void sdhc_log_ret(int ret)
 {
 }
 
@@ -319,7 +301,7 @@ static int sdhci_send_command(struct sd_mmc_ctrlr *ctrlr,
 
 	sdhc_log_command(cmd);
 
-	if (IS_ENABLED(CONFIG_SDHCI_BOUNCE_BUFFER) && data) {
+	if (CONFIG(SDHCI_BOUNCE_BUFFER) && data) {
 		if (data->flags & DATA_FLAG_READ) {
 			buf = data->dest;
 			bbflags = GEN_BB_WRITE;
@@ -349,7 +331,7 @@ static int sdhci_send_command(struct sd_mmc_ctrlr *ctrlr,
 	sdhci_led_control(ctrlr, 0);
 	sdhc_log_ret(ret);
 
-	if (IS_ENABLED(CONFIG_SDHCI_BOUNCE_BUFFER) && bbstate)
+	if (CONFIG(SDHCI_BOUNCE_BUFFER) && bbstate)
 		bounce_buffer_stop(bbstate);
 
 	return ret;
@@ -595,7 +577,7 @@ static void sdhci_set_ios(struct sd_mmc_ctrlr *ctrlr)
 	}
 
 	/* Set the new bus width */
-	if (IS_ENABLED(CONFIG_SDHC_DEBUG)
+	if (CONFIG(SDHC_DEBUG)
 		&& ((ctrl ^ previous_ctrl) & (SDHCI_CTRL_4BITBUS
 		| ((version >= SDHCI_SPEC_300) ? SDHCI_CTRL_8BITBUS : 0))))
 		sdhc_debug("SDHCI bus width: %d bit%s\n", bus_width,
@@ -715,7 +697,7 @@ static int sdhci_pre_init(struct sdhci_ctrlr *sdhci_ctrlr)
 	return 0;
 }
 
-__attribute__((weak)) void soc_sd_mmc_controller_quirks(struct sd_mmc_ctrlr
+__weak void soc_sd_mmc_controller_quirks(struct sd_mmc_ctrlr
 	*ctrlr)
 {
 }
@@ -729,7 +711,7 @@ static int sdhci_init(struct sdhci_ctrlr *sdhci_ctrlr)
 	if (ctrlr->initialized)
 		return 0;
 
-	sdhc_debug("SDHCI Controller Base Address: 0x%p\n",
+	sdhc_debug("SDHCI Controller Base Address: %p\n",
 			sdhci_ctrlr->ioaddr);
 
 	rv = sdhci_pre_init(sdhci_ctrlr);
