@@ -1,19 +1,4 @@
-/*
- * This file is part of the coreboot project.
- *
- * Copyright (C) 2000 Denis Dowling <dpd@alphalink.com.au>
- * Copyright (C) 2010 Keith Hui <buurin@gmail.com>
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- */
+/* SPDX-License-Identifier: GPL-2.0-or-later */
 
 /*
  * Intel Pentium L2 Cache initialization.
@@ -39,10 +24,8 @@
  */
 
 #include <stdint.h>
+#include <arch/cpu.h>
 #include <console/console.h>
-#include <string.h>
-#include <cpu/cpu.h>
-#include <cpu/x86/mtrr.h>
 #include <cpu/intel/l2_cache.h>
 #include <cpu/x86/cache.h>
 #include <cpu/x86/msr.h>
@@ -207,7 +190,6 @@ int calculate_l2_latency(void)
 	return 0;
 }
 
-
 /* Setup address, data_high:data_low into the L2
  * control registers and then issue command with correct cache way
  */
