@@ -1,30 +1,17 @@
+/* SPDX-License-Identifier: GPL-2.0-or-later */
+
 /*
  * mtrr.c: setting MTRR to decent values for cache initialization on P6
- *
  * Derived from intel_set_mtrr in intel_subr.c and mtrr.c in linux kernel
  *
- * Copyright 2000 Silicon Integrated System Corporation
- * Copyright 2013 Google Inc.
- *
- *	This program is free software; you can redistribute it and/or modify
- *	it under the terms of the GNU General Public License as published by
- *	the Free Software Foundation; either version 2 of the License, or
- *	(at your option) any later version.
- *
- *	This program is distributed in the hope that it will be useful,
- *	but WITHOUT ANY WARRANTY; without even the implied warranty of
- *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *	GNU General Public License for more details.
- *
- *
  * Reference: Intel Architecture Software Developer's Manual, Volume 3: System
  * Programming
  */
 
 #include <stddef.h>
-#include <stdlib.h>
 #include <string.h>
 #include <bootstate.h>
+#include <commonlib/helpers.h>
 #include <console/console.h>
 #include <device/device.h>
 #include <device/pci_ids.h>
@@ -32,21 +19,17 @@
 #include <cpu/x86/msr.h>
 #include <cpu/x86/mtrr.h>
 #include <cpu/x86/cache.h>
-#include <cpu/x86/lapic.h>
-#include <arch/cpu.h>
-#include <arch/acpi.h>
 #include <memrange.h>
-#if IS_ENABLED(CONFIG_X86_AMD_FIXED_MTRRS)
 #include <cpu/amd/mtrr.h>
+#include <assert.h>
+#if CONFIG(X86_AMD_FIXED_MTRRS)
 #define MTRR_FIXED_WRBACK_BITS (MTRR_READ_MEM | MTRR_WRITE_MEM)
 #else
 #define MTRR_FIXED_WRBACK_BITS 0
 #endif
 
-/* 2 MTRRS are reserved for the operating system */
-#define BIOS_MTRRS 6
-#define OS_MTRRS   2
-#define MTRRS      (BIOS_MTRRS + OS_MTRRS)
+#define MIN_MTRRS	8
+
 /*
  * Static storage size for variable MTRRs. It's sized sufficiently large to
  * handle different types of CPUs. Empirically, 16 variable MTRRs has not
@@ -54,8 +37,7 @@
  */
 #define NUM_MTRR_STATIC_STORAGE 16
 
-static int total_mtrrs = MTRRS;
-static int bios_mtrrs = BIOS_MTRRS;
+static int total_mtrrs;
 
 static void detect_var_mtrrs(void)
 {
@@ -71,7 +53,6 @@ static void detect_var_mtrrs(void)
 			total_mtrrs, NUM_MTRR_STATIC_STORAGE);
 		total_mtrrs = NUM_MTRR_STATIC_STORAGE;
 	}
-	bios_mtrrs = total_mtrrs - OS_MTRRS;
 }
 
 void enable_fixed_mtrr(void)
@@ -83,6 +64,30 @@ void enable_fixed_mtrr(void)
 	wrmsr(MTRR_DEF_TYPE_MSR, msr);
 }
 
+void fixed_mtrrs_expose_amd_rwdram(void)
+{
+	msr_t syscfg;
+
+	if (!CONFIG(X86_AMD_FIXED_MTRRS))
+		return;
+
+	syscfg = rdmsr(SYSCFG_MSR);
+	syscfg.lo |= SYSCFG_MSR_MtrrFixDramModEn;
+	wrmsr(SYSCFG_MSR, syscfg);
+}
+
+void fixed_mtrrs_hide_amd_rwdram(void)
+{
+	msr_t syscfg;
+
+	if (!CONFIG(X86_AMD_FIXED_MTRRS))
+		return;
+
+	syscfg = rdmsr(SYSCFG_MSR);
+	syscfg.lo &= ~SYSCFG_MSR_MtrrFixDramModEn;
+	wrmsr(SYSCFG_MSR, syscfg);
+}
+
 static void enable_var_mtrr(unsigned char deftype)
 {
 	msr_t msr;
@@ -95,9 +100,7 @@ static void enable_var_mtrr(unsigned char deftype)
 
 #define MTRR_VERBOSE_LEVEL BIOS_NEVER
 
-/* MTRRs are at a 4KiB granularity. Therefore all address calculations can
- * be done with 32-bit numbers. This allows for the MTRR code to handle
- * up to 2^44 bytes (16 TiB) of address space. */
+/* MTRRs are at a 4KiB granularity. */
 #define RANGE_SHIFT 12
 #define ADDR_SHIFT_TO_RANGE_SHIFT(x) \
 	(((x) > RANGE_SHIFT) ? ((x) - RANGE_SHIFT) : RANGE_SHIFT)
@@ -105,33 +108,19 @@ static void enable_var_mtrr(unsigned char deftype)
 #define RANGE_TO_PHYS_ADDR(x) (((resource_t)(x)) << RANGE_SHIFT)
 #define NUM_FIXED_MTRRS (NUM_FIXED_RANGES / RANGES_PER_FIXED_MTRR)
 
-/* The minimum alignment while handling variable MTRR ranges is 64MiB. */
-#define MTRR_MIN_ALIGN PHYS_TO_RANGE_ADDR(64 << 20)
 /* Helpful constants. */
-#define RANGE_1MB PHYS_TO_RANGE_ADDR(1 << 20)
-#define RANGE_4GB (1 << (ADDR_SHIFT_TO_RANGE_SHIFT(32)))
+#define RANGE_1MB PHYS_TO_RANGE_ADDR(1ULL << 20)
+#define RANGE_4GB (1ULL << (ADDR_SHIFT_TO_RANGE_SHIFT(32)))
 
-/*
- * The default MTRR type selection uses 3 approaches for selecting the
- * optimal number of variable MTRRs.  For each range do 3 calculations:
- *   1. UC as default type with no holes at top of range.
- *   2. UC as default using holes at top of range.
- *   3. WB as default.
- * If using holes is optimal for a range when UC is the default type the
- * tag is updated to direct the commit routine to use a hole at the top
- * of a range.
- */
 #define MTRR_ALGO_SHIFT (8)
 #define MTRR_TAG_MASK ((1 << MTRR_ALGO_SHIFT) - 1)
-/* If the default type is UC use the hole carving algorithm for a range. */
-#define MTRR_RANGE_UC_USE_HOLE (1 << MTRR_ALGO_SHIFT)
 
-static inline uint32_t range_entry_base_mtrr_addr(struct range_entry *r)
+static inline uint64_t range_entry_base_mtrr_addr(struct range_entry *r)
 {
 	return PHYS_TO_RANGE_ADDR(range_entry_base(r));
 }
 
-static inline uint32_t range_entry_end_mtrr_addr(struct range_entry *r)
+static inline uint64_t range_entry_end_mtrr_addr(struct range_entry *r)
 {
 	return PHYS_TO_RANGE_ADDR(range_entry_end(r));
 }
@@ -310,6 +299,8 @@ static void commit_fixed_mtrrs(void)
 	msr_t fixed_msrs[NUM_FIXED_MTRRS];
 	unsigned long msr_index[NUM_FIXED_MTRRS];
 
+	fixed_mtrrs_expose_amd_rwdram();
+
 	memset(&fixed_msrs, 0, sizeof(fixed_msrs));
 
 	msr_num = 0;
@@ -343,6 +334,9 @@ static void commit_fixed_mtrrs(void)
 		}
 	}
 
+	/* Ensure that both arrays were fully initialized */
+	ASSERT(msr_num == NUM_FIXED_MTRRS)
+
 	for (i = 0; i < ARRAY_SIZE(fixed_msrs); i++)
 		printk(BIOS_DEBUG, "MTRR: Fixed MSR 0x%lx 0x%08x%08x\n",
 		       msr_index[i], fixed_msrs[i].hi, fixed_msrs[i].lo);
@@ -351,6 +345,8 @@ static void commit_fixed_mtrrs(void)
 	for (i = 0; i < ARRAY_SIZE(fixed_msrs); i++)
 		wrmsr(msr_index[i], fixed_msrs[i]);
 	enable_cache();
+	fixed_mtrrs_hide_amd_rwdram();
+
 }
 
 void x86_setup_fixed_mtrrs_no_enable(void)
@@ -399,26 +395,33 @@ static void clear_var_mtrr(int index)
 	wrmsr(MTRR_PHYS_MASK(index), msr);
 }
 
+static int get_os_reserved_mtrrs(void)
+{
+	return CONFIG(RESERVE_MTRRS_FOR_OS) ? 2 : 0;
+}
+
 static void prep_var_mtrr(struct var_mtrr_state *var_state,
-			  uint32_t base, uint32_t size, int mtrr_type)
+			  uint64_t base, uint64_t size, int mtrr_type)
 {
 	struct var_mtrr_regs *regs;
 	resource_t rbase;
 	resource_t rsize;
 	resource_t mask;
 
-	/* Some variable MTRRs are attempted to be saved for the OS use.
-	 * However, it's more important to try to map the full address space
-	 * properly. */
-	if (var_state->mtrr_index >= bios_mtrrs)
-		printk(BIOS_WARNING, "Taking a reserved OS MTRR.\n");
 	if (var_state->mtrr_index >= total_mtrrs) {
-		printk(BIOS_ERR, "ERROR: Not enough MTRRs available! MTRR index"
-		       "is %d with %d MTTRs in total.\n",
+		printk(BIOS_ERR, "ERROR: Not enough MTRRs available! MTRR index is %d with %d MTRRs in total.\n",
 		       var_state->mtrr_index, total_mtrrs);
 		return;
 	}
 
+	/*
+	 * If desired, 2 variable MTRRs are attempted to be saved for the OS to
+	 * use. However, it's more important to try to map the full address
+	 * space properly.
+	 */
+	if (var_state->mtrr_index >= total_mtrrs - get_os_reserved_mtrrs())
+		printk(BIOS_WARNING, "Taking a reserved OS MTRR.\n");
+
 	rbase = base;
 	rsize = size;
 
@@ -443,24 +446,51 @@ static void prep_var_mtrr(struct var_mtrr_state *var_state,
 	regs->mask.hi = rsize >> 32;
 }
 
+/*
+ * fls64: find least significant bit set in a 64-bit word
+ * As samples, fls64(0x0) = 64; fls64(0x4400) = 10;
+ * fls64(0x40400000000) = 34.
+ */
+static uint32_t fls64(uint64_t x)
+{
+	uint32_t lo = (uint32_t)x;
+	if (lo)
+		return fls(lo);
+	uint32_t hi = x >> 32;
+	return fls(hi) + 32;
+}
+
+/*
+ * fms64: find most significant bit set in a 64-bit word
+ * As samples, fms64(0x0) = 0; fms64(0x4400) = 14;
+ * fms64(0x40400000000) = 42.
+ */
+static uint32_t fms64(uint64_t x)
+{
+	uint32_t hi = (uint32_t)(x >> 32);
+	if (!hi)
+		return fms((uint32_t)x);
+	return fms(hi) + 32;
+}
+
 static void calc_var_mtrr_range(struct var_mtrr_state *var_state,
-				uint32_t base, uint32_t size, int mtrr_type)
+				uint64_t base, uint64_t size, int mtrr_type)
 {
 	while (size != 0) {
 		uint32_t addr_lsb;
 		uint32_t size_msb;
-		uint32_t mtrr_size;
+		uint64_t mtrr_size;
 
-		addr_lsb = fls(base);
-		size_msb = fms(size);
+		addr_lsb = fls64(base);
+		size_msb = fms64(size);
 
 		/* All MTRR entries need to have their base aligned to the mask
 		 * size. The maximum size is calculated by a function of the
 		 * min base bit set and maximum size bit set. */
 		if (addr_lsb > size_msb)
-			mtrr_size = 1 << size_msb;
+			mtrr_size = 1ULL << size_msb;
 		else
-			mtrr_size = 1 << addr_lsb;
+			mtrr_size = 1ULL << addr_lsb;
 
 		if (var_state->prepare_msrs)
 			prep_var_mtrr(var_state, base, mtrr_size, mtrr_type);
@@ -471,22 +501,77 @@ static void calc_var_mtrr_range(struct var_mtrr_state *var_state,
 	}
 }
 
+static uint64_t optimize_var_mtrr_hole(const uint64_t base,
+				       const uint64_t hole,
+				       const uint64_t limit,
+				       const int carve_hole)
+{
+	/*
+	 * With default type UC, we can potentially optimize a WB
+	 * range with unaligned upper end, by aligning it up and
+	 * carving the added "hole" out again.
+	 *
+	 * To optimize the upper end of the hole, we will test
+	 * how many MTRRs calc_var_mtrr_range() will spend for any
+	 * alignment of the hole's upper end.
+	 *
+	 * We take four parameters, the lower end of the WB range
+	 * `base`, upper end of the WB range as start of the `hole`,
+	 * a `limit` how far we may align the upper end of the hole
+	 * up and a flag `carve_hole` whether we should count MTRRs
+	 * for carving the hole out. We return the optimal upper end
+	 * for the hole (which may be the same as the end of the WB
+	 * range in case we don't gain anything by aligning up).
+	 */
+
+	const int dont_care = 0;
+	struct var_mtrr_state var_state = { 0, };
+
+	unsigned int align, best_count;
+	uint32_t best_end = hole;
+
+	/* calculate MTRR count for the WB range alone (w/o a hole) */
+	calc_var_mtrr_range(&var_state, base, hole - base, dont_care);
+	best_count = var_state.mtrr_index;
+	var_state.mtrr_index = 0;
+
+	for (align = fls(hole) + 1; align <= fms(hole); ++align) {
+		const uint64_t hole_end = ALIGN_UP((uint64_t)hole, 1 << align);
+		if (hole_end > limit)
+			break;
+
+		/* calculate MTRR count for this alignment */
+		calc_var_mtrr_range(
+			&var_state, base, hole_end - base, dont_care);
+		if (carve_hole)
+			calc_var_mtrr_range(
+				&var_state, hole, hole_end - hole, dont_care);
+
+		if (var_state.mtrr_index < best_count) {
+			best_count = var_state.mtrr_index;
+			best_end = hole_end;
+		}
+		var_state.mtrr_index = 0;
+	}
+
+	return best_end;
+}
+
 static void calc_var_mtrrs_with_hole(struct var_mtrr_state *var_state,
 				     struct range_entry *r)
 {
-	uint32_t a1, a2, b1, b2;
-	int mtrr_type;
-	struct range_entry *next;
+	uint64_t a1, a2, b1, b2;
+	int mtrr_type, carve_hole;
 
 	/*
 	 * Determine MTRRs based on the following algorithm for the given entry:
 	 * +------------------+ b2 = ALIGN_UP(end)
 	 * |  0 or more bytes | <-- hole is carved out between b1 and b2
-	 * +------------------+ a2 = b1 = end
+	 * +------------------+ a2 = b1 = original end
 	 * |                  |
 	 * +------------------+ a1 = begin
 	 *
-	 * Thus, there are 3 sub-ranges to configure variable MTRRs for.
+	 * Thus, there are up to 2 sub-ranges to configure variable MTRRs for.
 	 */
 	mtrr_type = range_entry_mtrr_type(r);
 
@@ -513,71 +598,57 @@ static void calc_var_mtrrs_with_hole(struct var_mtrr_state *var_state,
 	if (!var_state->above4gb && a2 > RANGE_4GB)
 		a2 = RANGE_4GB;
 
-	next = memranges_next_entry(var_state->addr_space, r);
-
 	b1 = a2;
+	b2 = a2;
+	carve_hole = 0;
 
-	/* First check if a1 is >= 4GiB and the current entry is the last
-	 * entry. If so perform an optimization of covering a larger range
-	 * defined by the base address' alignment. */
-	if (a1 >= RANGE_4GB && next == NULL) {
-		uint32_t addr_lsb;
-
-		addr_lsb = fls(a1);
-		b2 = (1 << addr_lsb) + a1;
-		if (b2 >= a2) {
-			calc_var_mtrr_range(var_state, a1, b2 - a1, mtrr_type);
-			return;
+	/* We only consider WB type ranges for hole-carving. */
+	if (mtrr_type == MTRR_TYPE_WRBACK) {
+		struct range_entry *next;
+		uint64_t b2_limit;
+		/*
+		 * Depending on the type of the next range, there are three
+		 * different situations to handle:
+		 *
+		 * 1. WB range is last in address space:
+		 *    Aligning up, up to the next power of 2, may gain us
+		 *    something.
+		 *
+		 * 2. The next range is of type UC:
+		 *    We may align up, up to the _end_ of the next range. If
+		 *    there is a gap between the current and the next range,
+		 *    it would have been covered by the default type UC anyway.
+		 *
+		 * 3. The next range is not of type UC:
+		 *    We may align up, up to the _base_ of the next range. This
+		 *    may either be the end of the current range (if the next
+		 *    range follows immediately) or the end of the gap between
+		 *    the ranges.
+		 */
+		next = memranges_next_entry(var_state->addr_space, r);
+		if (next == NULL) {
+			b2_limit = ALIGN_UP((uint64_t)b1, 1 << fms(b1));
+			/* If it's the last range above 4GiB, we won't carve
+			   the hole out. If an OS wanted to move MMIO there,
+			   it would have to override the MTRR setting using
+			   PAT just like it would with WB as default type. */
+			carve_hole = a1 < RANGE_4GB;
+		} else if (range_entry_mtrr_type(next)
+				== MTRR_TYPE_UNCACHEABLE) {
+			b2_limit = range_entry_end_mtrr_addr(next);
+			carve_hole = 1;
+		} else {
+			b2_limit = range_entry_base_mtrr_addr(next);
+			carve_hole = 1;
 		}
-	}
-
-	/* Handle the min alignment roundup case. */
-	b2 = ALIGN_UP(a2, MTRR_MIN_ALIGN);
-
-	/* Check against the next range. If the current range_entry is the
-	 * last entry then carving a hole is no problem. If the current entry
-	 * isn't the last entry then check that the last entry covers the
-	 * entire hole range with the default mtrr type. */
-	if (next != NULL &&
-	    (range_entry_mtrr_type(next) != var_state->def_mtrr_type ||
-	     range_entry_end_mtrr_addr(next) < b2)) {
-		calc_var_mtrr_range(var_state, a1, a2 - a1, mtrr_type);
-		return;
+		b2 = optimize_var_mtrr_hole(a1, b1, b2_limit, carve_hole);
 	}
 
 	calc_var_mtrr_range(var_state, a1, b2 - a1, mtrr_type);
-	calc_var_mtrr_range(var_state, b1, b2 - b1, var_state->def_mtrr_type);
-}
-
-static void calc_var_mtrrs_without_hole(struct var_mtrr_state *var_state,
-					struct range_entry *r)
-{
-	const int mtrr_type = range_entry_mtrr_type(r);
-
-	uint32_t base = range_entry_base_mtrr_addr(r);
-	uint32_t end = range_entry_end_mtrr_addr(r);
-
-	/* The end address is within the first 1MiB. The fixed MTRRs take
-	 * precedence over the variable ones. Therefore this range
-	 * can be ignored. */
-	if (end <= RANGE_1MB)
-		return;
-
-	/* Again, the fixed MTRRs take precedence so the beginning
-	 * of the range can be set to 0 if it starts at or below 1MiB. */
-	if (base <= RANGE_1MB)
-		base = 0;
-
-	/* If the range starts above 4GiB the processing is done. */
-	if (!var_state->above4gb && base >= RANGE_4GB)
-		return;
-
-	/* Clip the upper address to 4GiB if addresses above 4GiB
-	 * are not being processed. */
-	if (!var_state->above4gb && end > RANGE_4GB)
-		end = RANGE_4GB;
-
-	calc_var_mtrr_range(var_state, base, end - base, mtrr_type);
+	if (carve_hole && b2 != b1) {
+		calc_var_mtrr_range(var_state, b1, b2 - b1,
+				    MTRR_TYPE_UNCACHEABLE);
+	}
 }
 
 static void __calc_var_mtrrs(struct memranges *addr_space,
@@ -601,16 +672,14 @@ static void __calc_var_mtrrs(struct memranges *addr_space,
 	uc_deftype_count = 0;
 
 	/*
-	 * For each range do 3 calculations:
-	 *   1. UC as default type with no holes at top of range.
-	 *   2. UC as default using holes at top of range.
-	 *   3. WB as default.
+	 * For each range do 2 calculations:
+	 *   1. UC as default type with possible holes at top of range.
+	 *   2. WB as default.
 	 * The lowest count is then used as default after totaling all
-	 * MTRRs. Note that the optimal algorithm for UC default is marked in
-	 * the tag of each range regardless of final decision.  UC takes
-	 * precedence in the MTRR architecture. Therefore, only holes can be
-	 * used when the type of the region is MTRR_TYPE_WRBACK with
-	 * MTRR_TYPE_UNCACHEABLE as the default type.
+	 * MTRRs. UC takes precedence in the MTRR architecture. There-
+	 * fore, only holes can be used when the type of the region is
+	 * MTRR_TYPE_WRBACK with MTRR_TYPE_UNCACHEABLE as the default
+	 * type.
 	 */
 	memranges_each_entry(r, var_state.addr_space) {
 		int mtrr_type;
@@ -618,46 +687,20 @@ static void __calc_var_mtrrs(struct memranges *addr_space,
 		mtrr_type = range_entry_mtrr_type(r);
 
 		if (mtrr_type != MTRR_TYPE_UNCACHEABLE) {
-			int uc_hole_count;
-			int uc_no_hole_count;
-
-			var_state.def_mtrr_type = MTRR_TYPE_UNCACHEABLE;
 			var_state.mtrr_index = 0;
-
-			/* No hole calculation. */
-			calc_var_mtrrs_without_hole(&var_state, r);
-			uc_no_hole_count = var_state.mtrr_index;
-
-			/* Hole calculation only if type is WB. The 64 number
-			 * is a count that is unachievable, thus making it
-			 * a default large number in the case of not doing
-			 * the hole calculation. */
-			uc_hole_count = 64;
-			if (mtrr_type == MTRR_TYPE_WRBACK) {
-				var_state.mtrr_index = 0;
-				calc_var_mtrrs_with_hole(&var_state, r);
-				uc_hole_count = var_state.mtrr_index;
-			}
-
-			/* Mark the entry with the optimal algorithm. */
-			if (uc_no_hole_count < uc_hole_count) {
-				uc_deftype_count += uc_no_hole_count;
-			} else {
-				unsigned long new_tag;
-
-				new_tag = mtrr_type | MTRR_RANGE_UC_USE_HOLE;
-				range_entry_update_tag(r, new_tag);
-				uc_deftype_count += uc_hole_count;
-			}
+			var_state.def_mtrr_type = MTRR_TYPE_UNCACHEABLE;
+			calc_var_mtrrs_with_hole(&var_state, r);
+			uc_deftype_count += var_state.mtrr_index;
 		}
 
 		if (mtrr_type != MTRR_TYPE_WRBACK) {
 			var_state.mtrr_index = 0;
 			var_state.def_mtrr_type = MTRR_TYPE_WRBACK;
-			calc_var_mtrrs_without_hole(&var_state, r);
+			calc_var_mtrrs_with_hole(&var_state, r);
 			wb_deftype_count += var_state.mtrr_index;
 		}
 	}
+
 	*num_def_wb_mtrrs = wb_deftype_count;
 	*num_def_uc_mtrrs = uc_deftype_count;
 }
@@ -671,6 +714,7 @@ static int calc_var_mtrrs(struct memranges *addr_space,
 	__calc_var_mtrrs(addr_space, above4gb, address_bits, &wb_deftype_count,
 			 &uc_deftype_count);
 
+	const int bios_mtrrs = total_mtrrs - get_os_reserved_mtrrs();
 	if (wb_deftype_count > bios_mtrrs && uc_deftype_count > bios_mtrrs) {
 		printk(BIOS_DEBUG, "MTRR: Removing WRCOMB type. "
 		       "WB/UC MTRR counts: %d/%d > %d.\n",
@@ -711,12 +755,7 @@ static void prepare_var_mtrrs(struct memranges *addr_space, int def_type,
 	memranges_each_entry(r, var_state.addr_space) {
 		if (range_entry_mtrr_type(r) == def_type)
 			continue;
-
-		if (def_type == MTRR_TYPE_UNCACHEABLE &&
-		    (range_entry_tag(r) & MTRR_RANGE_UC_USE_HOLE))
-			calc_var_mtrrs_with_hole(&var_state, r);
-		else
-			calc_var_mtrrs_without_hole(&var_state, r);
+		calc_var_mtrrs_with_hole(&var_state, r);
 	}
 
 	/* Update the solution. */
@@ -766,7 +805,7 @@ void x86_setup_var_mtrrs(unsigned int address_bits, unsigned int above4gb)
 	commit_var_mtrrs(sol);
 }
 
-void x86_setup_mtrrs(void)
+static void _x86_setup_mtrrs(unsigned int above4gb)
 {
 	int address_size;
 
@@ -774,14 +813,28 @@ void x86_setup_mtrrs(void)
 	address_size = cpu_phys_address_size();
 	printk(BIOS_DEBUG, "CPU physical address size: %d bits\n",
 		address_size);
+	x86_setup_var_mtrrs(address_size, above4gb);
+}
+
+void x86_setup_mtrrs(void)
+{
+	/* Without detect, assume the minimum */
+	total_mtrrs = MIN_MTRRS;
 	/* Always handle addresses above 4GiB. */
-	x86_setup_var_mtrrs(address_size, 1);
+	_x86_setup_mtrrs(1);
 }
 
 void x86_setup_mtrrs_with_detect(void)
 {
 	detect_var_mtrrs();
-	x86_setup_mtrrs();
+	/* Always handle addresses above 4GiB. */
+	_x86_setup_mtrrs(1);
+}
+
+void x86_setup_mtrrs_with_detect_no_above_4gb(void)
+{
+	detect_var_mtrrs();
+	_x86_setup_mtrrs(0);
 }
 
 void x86_mtrr_check(void)
@@ -827,9 +880,6 @@ void mtrr_use_temp_range(uintptr_t begin, size_t size, int type)
 	memranges_each_entry(r, orig) {
 		unsigned long tag = range_entry_tag(r);
 
-		/* Remove any special tags from original solution. */
-		tag &= ~MTRR_RANGE_UC_USE_HOLE;
-
 		/* Remove any write combining MTRRs from the temporary
 		 * solution as it just fragments the address space. */
 		if (tag == MTRR_TYPE_WRCOMB)
