@@ -1,122 +1,20 @@
-/*
- * This file is part of the coreboot project.
- *
- * Copyright (C) 2003-2004 Linux Networx
- * (Written by Eric Biederman <ebiederman@lnxi.com> for Linux Networx)
- * Copyright (C) 2003 Greg Watson <jarrah@users.sourceforge.net>
- * Copyright (C) 2004 Li-Ta Lo <ollie@lanl.gov>
- * Copyright (C) 2005-2006 Tyan
- * (Written by Yinghai Lu <yhlu@tyan.com> for Tyan)
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- */
+/* SPDX-License-Identifier: GPL-2.0-only */
 
 #include <console/console.h>
 #include <device/device.h>
 #include <device/path.h>
 #include <device/pci_def.h>
 #include <device/resource.h>
+#include <stdlib.h>
 #include <string.h>
 
-/**
- * See if a device structure exists for path.
- *
- * @param parent The bus to find the device on.
- * @param path The relative path from the bus to the appropriate device.
- * @return Pointer to a device structure for the device on bus at path
- *         or 0/NULL if no device is found.
- */
-struct device *find_dev_path(struct bus *parent, struct device_path *path)
-{
-	struct device *child;
-	for (child = parent->children; child; child = child->sibling) {
-		if (path_eq(path, &child->path))
-			break;
-	}
-	return child;
-}
-
-/**
- * Given a PCI bus and a devfn number, find the device structure.
- *
- * @param bus The bus number.
- * @param devfn A device/function number.
- * @return Pointer to the device structure (if found), 0 otherwise.
- */
-struct device *dev_find_slot(unsigned int bus, unsigned int devfn)
-{
-	struct device *dev, *result;
-
-	result = 0;
-	for (dev = all_devices; dev; dev = dev->next) {
-		if ((dev->path.type == DEVICE_PATH_PCI) &&
-		    (dev->bus->secondary == bus) &&
-		    (dev->path.pci.devfn == devfn)) {
-			result = dev;
-			break;
-		}
-	}
-	return result;
-}
-
-/**
- * Given an SMBus bus and a device number, find the device structure.
- *
- * @param bus The bus number.
- * @param addr A device number.
- * @return Pointer to the device structure (if found), 0 otherwise.
- */
-struct device *dev_find_slot_on_smbus(unsigned int bus, unsigned int addr)
-{
-	struct device *dev, *result;
-
-	result = 0;
-	for (dev = all_devices; dev; dev = dev->next) {
-		if ((dev->path.type == DEVICE_PATH_I2C) &&
-		    (dev->bus->secondary == bus) &&
-		    (dev->path.i2c.device == addr)) {
-			result = dev;
-			break;
-		}
-	}
-	return result;
-}
-
-/**
- * Given a PnP port and a device number, find the device structure.
- *
- * @param port The I/O port.
- * @param device Logical device number.
- * @return Pointer to the device structure (if found), 0 otherwise.
- */
-struct device *dev_find_slot_pnp(u16 port, u16 device)
-{
-	struct device *dev;
-
-	for (dev = all_devices; dev; dev = dev->next) {
-		if ((dev->path.type == DEVICE_PATH_PNP) &&
-		    (dev->path.pnp.port == port) &&
-		    (dev->path.pnp.device == device)) {
-			return dev;
-		}
-	}
-	return 0;
-}
-
 /**
  * Given a Local APIC ID, find the device structure.
  *
  * @param apic_id The Local APIC ID number.
  * @return Pointer to the device structure (if found), 0 otherwise.
  */
-struct device *dev_find_lapic(unsigned apic_id)
+struct device *dev_find_lapic(unsigned int apic_id)
 {
 	struct device *dev;
 	struct device *result = NULL;
@@ -131,33 +29,6 @@ struct device *dev_find_lapic(unsigned apic_id)
 	return result;
 }
 
-/**
- * Given a Device Path Type, find the device structure.
- *
- * @param prev_match The previously matched device instance.
- * @param path_type The Device Path Type.
- * @return Pointer to the device structure (if found), 0 otherwise.
- */
-struct device *dev_find_path(device_t prev_match,
-				enum device_path_type path_type)
-{
-	struct device *dev;
-	struct device *result = NULL;
-
-	if (prev_match == NULL)
-		prev_match = all_devices;
-	else
-		prev_match = prev_match->next;
-
-	for (dev = prev_match; dev; dev = dev->next) {
-		if (dev->path.type == path_type) {
-			result = dev;
-			break;
-		}
-	}
-	return result;
-}
-
 /**
  * Find a device of a given vendor and type.
  *
@@ -209,7 +80,7 @@ struct device *dev_find_class(unsigned int class, struct device *from)
  * @param dev The device path to encode.
  * @return Device path encoded into lower 3 bytes of dword.
  */
-u32 dev_path_encode(struct device *dev)
+u32 dev_path_encode(const struct device *dev)
 {
 	u32 ret;
 
@@ -256,7 +127,14 @@ u32 dev_path_encode(struct device *dev)
 	case DEVICE_PATH_SPI:
 		ret |= dev->path.spi.cs;
 		break;
+	case DEVICE_PATH_USB:
+		ret |= dev->path.usb.port_type << 8 | dev->path.usb.port_id;
+		break;
+	case DEVICE_PATH_GPIO:
+		ret |= dev->path.gpio.id;
+		break;
 	case DEVICE_PATH_NONE:
+	case DEVICE_PATH_MMIO:  /* don't care */
 	default:
 		break;
 	}
@@ -268,70 +146,81 @@ u32 dev_path_encode(struct device *dev)
  * Warning: This function uses a static buffer. Don't call it more than once
  * from the same print statement!
  */
-const char *dev_path(struct device *dev)
+const char *dev_path(const struct device *dev)
 {
 	static char buffer[DEVICE_PATH_MAX];
 
 	buffer[0] = '\0';
 	if (!dev) {
-		memcpy(buffer, "<null>", 7);
+		strcpy(buffer, "<null>");
 	} else {
-		switch(dev->path.type) {
+		switch (dev->path.type) {
 		case DEVICE_PATH_NONE:
-			memcpy(buffer, "NONE", 5);
+			strcpy(buffer, "NONE");
 			break;
 		case DEVICE_PATH_ROOT:
-			memcpy(buffer, "Root Device", 12);
+			strcpy(buffer, "Root Device");
 			break;
 		case DEVICE_PATH_PCI:
-			snprintf(buffer, sizeof (buffer),
+			snprintf(buffer, sizeof(buffer),
 				 "PCI: %02x:%02x.%01x",
 				 dev->bus->secondary,
 				 PCI_SLOT(dev->path.pci.devfn),
 				 PCI_FUNC(dev->path.pci.devfn));
 			break;
 		case DEVICE_PATH_PNP:
-			snprintf(buffer, sizeof (buffer), "PNP: %04x.%01x",
+			snprintf(buffer, sizeof(buffer), "PNP: %04x.%01x",
 				 dev->path.pnp.port, dev->path.pnp.device);
 			break;
 		case DEVICE_PATH_I2C:
-			snprintf(buffer, sizeof (buffer), "I2C: %02x:%02x",
+			snprintf(buffer, sizeof(buffer), "I2C: %02x:%02x",
 				 dev->bus->secondary,
 				 dev->path.i2c.device);
 			break;
 		case DEVICE_PATH_APIC:
-			snprintf(buffer, sizeof (buffer), "APIC: %02x",
+			snprintf(buffer, sizeof(buffer), "APIC: %02x",
 				 dev->path.apic.apic_id);
 			break;
 		case DEVICE_PATH_IOAPIC:
-			snprintf(buffer, sizeof (buffer), "IOAPIC: %02x",
+			snprintf(buffer, sizeof(buffer), "IOAPIC: %02x",
 				 dev->path.ioapic.ioapic_id);
 			break;
 		case DEVICE_PATH_DOMAIN:
-			snprintf(buffer, sizeof (buffer), "DOMAIN: %04x",
+			snprintf(buffer, sizeof(buffer), "DOMAIN: %04x",
 				dev->path.domain.domain);
 			break;
 		case DEVICE_PATH_CPU_CLUSTER:
-			snprintf(buffer, sizeof (buffer), "CPU_CLUSTER: %01x",
+			snprintf(buffer, sizeof(buffer), "CPU_CLUSTER: %01x",
 				dev->path.cpu_cluster.cluster);
 			break;
 		case DEVICE_PATH_CPU:
-			snprintf(buffer, sizeof (buffer),
+			snprintf(buffer, sizeof(buffer),
 				 "CPU: %02x", dev->path.cpu.id);
 			break;
 		case DEVICE_PATH_CPU_BUS:
-			snprintf(buffer, sizeof (buffer),
+			snprintf(buffer, sizeof(buffer),
 				 "CPU_BUS: %02x", dev->path.cpu_bus.id);
 			break;
 		case DEVICE_PATH_GENERIC:
-			snprintf(buffer, sizeof (buffer),
+			snprintf(buffer, sizeof(buffer),
 				 "GENERIC: %d.%d", dev->path.generic.id,
 				 dev->path.generic.subid);
 			break;
 		case DEVICE_PATH_SPI:
-			snprintf(buffer, sizeof (buffer), "SPI: %02x",
+			snprintf(buffer, sizeof(buffer), "SPI: %02x",
 				 dev->path.spi.cs);
 			break;
+		case DEVICE_PATH_USB:
+			snprintf(buffer, sizeof(buffer), "USB%u port %u",
+				 dev->path.usb.port_type, dev->path.usb.port_id);
+			break;
+		case DEVICE_PATH_MMIO:
+			snprintf(buffer, sizeof(buffer), "MMIO: %08lx",
+				 dev->path.mmio.addr);
+			break;
+		case DEVICE_PATH_GPIO:
+			snprintf(buffer, sizeof(buffer), "GPIO: %d", dev->path.gpio.id);
+			break;
 		default:
 			printk(BIOS_ERR, "Unknown device path type: %d\n",
 			       dev->path.type);
@@ -341,7 +230,7 @@ const char *dev_path(struct device *dev)
 	return buffer;
 }
 
-const char *dev_name(struct device *dev)
+const char *dev_name(const struct device *dev)
 {
 	if (dev->name)
 		return dev->name;
@@ -354,66 +243,11 @@ const char *dev_name(struct device *dev)
 const char *bus_path(struct bus *bus)
 {
 	static char buffer[BUS_PATH_MAX];
-	snprintf(buffer, sizeof (buffer),
+	snprintf(buffer, sizeof(buffer),
 		 "%s,%d", dev_path(bus->dev), bus->link_num);
 	return buffer;
 }
 
-int path_eq(struct device_path *path1, struct device_path *path2)
-{
-	int equal = 0;
-
-	if (path1->type != path2->type)
-		return 0;
-
-	switch (path1->type) {
-	case DEVICE_PATH_NONE:
-		break;
-	case DEVICE_PATH_ROOT:
-		equal = 1;
-		break;
-	case DEVICE_PATH_PCI:
-		equal = (path1->pci.devfn == path2->pci.devfn);
-		break;
-	case DEVICE_PATH_PNP:
-		equal = (path1->pnp.port == path2->pnp.port) &&
-			(path1->pnp.device == path2->pnp.device);
-		break;
-	case DEVICE_PATH_I2C:
-		equal = (path1->i2c.device == path2->i2c.device) &&
-			(path1->i2c.mode_10bit == path2->i2c.mode_10bit);
-		break;
-	case DEVICE_PATH_APIC:
-		equal = (path1->apic.apic_id == path2->apic.apic_id);
-		break;
-	case DEVICE_PATH_DOMAIN:
-		equal = (path1->domain.domain == path2->domain.domain);
-		break;
-	case DEVICE_PATH_CPU_CLUSTER:
-		equal = (path1->cpu_cluster.cluster
-			 == path2->cpu_cluster.cluster);
-		break;
-	case DEVICE_PATH_CPU:
-		equal = (path1->cpu.id == path2->cpu.id);
-		break;
-	case DEVICE_PATH_CPU_BUS:
-		equal = (path1->cpu_bus.id == path2->cpu_bus.id);
-		break;
-	case DEVICE_PATH_GENERIC:
-		equal = (path1->generic.id == path2->generic.id) &&
-			(path1->generic.subid == path2->generic.subid);
-		break;
-	case DEVICE_PATH_SPI:
-		equal = (path1->spi.cs == path2->spi.cs);
-		break;
-	default:
-		printk(BIOS_ERR, "Unknown device type: %d\n", path1->type);
-		break;
-	}
-
-	return equal;
-}
-
 /**
  * Allocate 64 more resources to the free list.
  *
@@ -486,7 +320,7 @@ void compact_resources(struct device *dev)
  * @param index The index of the resource on the device.
  * @return The resource, if it already exists.
  */
-struct resource *probe_resource(struct device *dev, unsigned index)
+struct resource *probe_resource(const struct device *dev, unsigned int index)
 {
 	struct resource *res;
 
@@ -509,7 +343,7 @@ struct resource *probe_resource(struct device *dev, unsigned index)
  * @param index The index of the resource on the device.
  * @return TODO.
  */
-struct resource *new_resource(struct device *dev, unsigned index)
+struct resource *new_resource(struct device *dev, unsigned int index)
 {
 	struct resource *resource, *tail;
 
@@ -528,7 +362,8 @@ struct resource *new_resource(struct device *dev, unsigned index)
 		resource->next = NULL;
 		tail = dev->resource_list;
 		if (tail) {
-			while (tail->next) tail = tail->next;
+			while (tail->next)
+				tail = tail->next;
 			tail->next = resource;
 		} else {
 			dev->resource_list = resource;
@@ -556,7 +391,7 @@ struct resource *new_resource(struct device *dev, unsigned index)
  * @param index The index of the resource on the device.
  * return TODO.
  */
-struct resource *find_resource(struct device *dev, unsigned index)
+struct resource *find_resource(const struct device *dev, unsigned int index)
 {
 	struct resource *resource;
 
@@ -607,7 +442,7 @@ static resource_t align_down(resource_t val, unsigned long gran)
  * @param resource The resource whose limit is desired.
  * @return The end.
  */
-resource_t resource_end(struct resource *resource)
+resource_t resource_end(const struct resource *resource)
 {
 	resource_t base, end;
 
@@ -633,7 +468,7 @@ resource_t resource_end(struct resource *resource)
  * @param resource The resource whose maximum is desired.
  * @return The maximum.
  */
-resource_t resource_max(struct resource *resource)
+resource_t resource_max(const struct resource *resource)
 {
 	resource_t max;
 
@@ -648,10 +483,10 @@ resource_t resource_max(struct resource *resource)
  * @param resource The resource type to decode.
  * @return TODO.
  */
-const char *resource_type(struct resource *resource)
+const char *resource_type(const struct resource *resource)
 {
 	static char buffer[RESOURCE_TYPE_MAX];
-	snprintf(buffer, sizeof (buffer), "%s%s%s%s",
+	snprintf(buffer, sizeof(buffer), "%s%s%s%s",
 		 ((resource->flags & IORESOURCE_READONLY) ? "ro" : ""),
 		 ((resource->flags & IORESOURCE_PREFETCH) ? "pref" : ""),
 		 ((resource->flags == 0) ? "unused" :
@@ -670,7 +505,7 @@ const char *resource_type(struct resource *resource)
  * @param resource The resource that was just stored.
  * @param comment TODO
  */
-void report_resource_stored(struct device *dev, struct resource *resource,
+void report_resource_stored(struct device *dev, const struct resource *resource,
 			    const char *comment)
 {
 	char buf[10];
@@ -683,8 +518,8 @@ void report_resource_stored(struct device *dev, struct resource *resource,
 	end = resource_end(resource);
 	buf[0] = '\0';
 
-	if (resource->flags & IORESOURCE_PCI_BRIDGE) {
-		snprintf(buf, sizeof (buf),
+	if (dev->link_list && (resource->flags & IORESOURCE_PCI_BRIDGE)) {
+		snprintf(buf, sizeof(buf),
 			 "bus %02x ", dev->link_list->secondary);
 	}
 	printk(BIOS_DEBUG, "%s %02lx <- [0x%010llx - 0x%010llx] size 0x%08llx "
@@ -713,7 +548,7 @@ void search_bus_resources(struct bus *bus, unsigned long type_mask,
 
 			/* If it is a subtractive resource recurse. */
 			if (res->flags & IORESOURCE_SUBTRACTIVE) {
-				struct bus * subbus;
+				struct bus *subbus;
 				for (subbus = curdev->link_list; subbus;
 				     subbus = subbus->next)
 					if (subbus->link_num
@@ -762,11 +597,10 @@ void dev_set_enabled(struct device *dev, int enable)
 		return;
 
 	dev->enabled = enable;
-	if (dev->ops && dev->ops->enable) {
+	if (dev->ops && dev->ops->enable)
 		dev->ops->enable(dev);
-	} else if (dev->chip_ops && dev->chip_ops->enable_dev) {
+	else if (dev->chip_ops && dev->chip_ops->enable_dev)
 		dev->chip_ops->enable_dev(dev);
-	}
 }
 
 void disable_children(struct bus *bus)
@@ -783,7 +617,7 @@ void disable_children(struct bus *bus)
 
 /*
  * Returns true if the device is an enabled bridge that has at least
- * one enabled device on its secondary bus.
+ * one enabled device on its secondary bus that is not of type NONE.
  */
 bool dev_is_active_bridge(struct device *dev)
 {
@@ -798,6 +632,9 @@ bool dev_is_active_bridge(struct device *dev)
 
 	for (link = dev->link_list; link; link = link->next) {
 		for (child = link->children; child; child = child->sibling) {
+			if (child->path.type == DEVICE_PATH_NONE)
+				continue;
+
 			if (child->enabled)
 				return 1;
 		}
@@ -806,7 +643,54 @@ bool dev_is_active_bridge(struct device *dev)
 	return 0;
 }
 
-static void resource_tree(struct device *root, int debug_level, int depth)
+/**
+ * Ensure the device has a minimum number of bus links.
+ *
+ * @param dev The device to add links to.
+ * @param total_links The minimum number of links to have.
+ */
+void add_more_links(struct device *dev, unsigned int total_links)
+{
+	struct bus *link, *last = NULL;
+	int link_num = -1;
+
+	for (link = dev->link_list; link; link = link->next) {
+		if (link_num < link->link_num)
+			link_num = link->link_num;
+		last = link;
+	}
+
+	if (last) {
+		int links = total_links - (link_num + 1);
+		if (links > 0) {
+			link = malloc(links * sizeof(*link));
+			if (!link)
+				die("Couldn't allocate more links!\n");
+			memset(link, 0, links * sizeof(*link));
+			last->next = link;
+		} else {
+			/* No more links to add */
+			return;
+		}
+	} else {
+		link = malloc(total_links * sizeof(*link));
+		if (!link)
+			die("Couldn't allocate more links!\n");
+		memset(link, 0, total_links * sizeof(*link));
+		dev->link_list = link;
+	}
+
+	for (link_num = link_num + 1; link_num < total_links; link_num++) {
+		link->link_num = link_num;
+		link->dev = dev;
+		link->next = link + 1;
+		last = link;
+		link = link->next;
+	}
+	last->next = NULL;
+}
+
+static void resource_tree(const struct device *root, int debug_level, int depth)
 {
 	int i = 0;
 	struct device *child;
@@ -818,14 +702,14 @@ static void resource_tree(struct device *root, int debug_level, int depth)
 		indent[i] = ' ';
 	indent[i] = '\0';
 
-	do_printk(BIOS_DEBUG, "%s%s", indent, dev_path(root));
+	printk(BIOS_DEBUG, "%s%s", indent, dev_path(root));
 	if (root->link_list && root->link_list->children)
-		do_printk(BIOS_DEBUG, " child on link 0 %s",
+		printk(BIOS_DEBUG, " child on link 0 %s",
 			  dev_path(root->link_list->children));
-	do_printk(BIOS_DEBUG, "\n");
+	printk(BIOS_DEBUG, "\n");
 
 	for (res = root->resource_list; res; res = res->next) {
-		do_printk(debug_level, "%s%s resource base %llx size %llx "
+		printk(debug_level, "%s%s resource base %llx size %llx "
 			  "align %d gran %d limit %llx flags %lx index %lx\n",
 			  indent, dev_path(root), res->base, res->size,
 			  res->align, res->gran, res->limit, res->flags,
@@ -838,23 +722,24 @@ static void resource_tree(struct device *root, int debug_level, int depth)
 	}
 }
 
-void print_resource_tree(struct device *root, int debug_level, const char *msg)
+void print_resource_tree(const struct device *root, int debug_level,
+			 const char *msg)
 {
 	/* Bail if root is null. */
 	if (!root) {
-		do_printk(debug_level, "%s passed NULL for root!\n", __func__);
+		printk(debug_level, "%s passed NULL for root!\n", __func__);
 		return;
 	}
 
 	/* Bail if not printing to screen. */
-	if (!do_printk(debug_level, "Show resources in subtree (%s)...%s\n",
+	if (!printk(debug_level, "Show resources in subtree (%s)...%s\n",
 		       dev_path(root), msg))
 		return;
 
 	resource_tree(root, debug_level, 0);
 }
 
-void show_devs_tree(struct device *dev, int debug_level, int depth)
+void show_devs_tree(const struct device *dev, int debug_level, int depth)
 {
 	char depth_str[20];
 	int i;
@@ -865,7 +750,7 @@ void show_devs_tree(struct device *dev, int debug_level, int depth)
 		depth_str[i] = ' ';
 	depth_str[i] = '\0';
 
-	do_printk(debug_level, "%s%s: enabled %d\n",
+	printk(debug_level, "%s%s: enabled %d\n",
 		  depth_str, dev_path(dev), dev->enabled);
 
 	for (link = dev->link_list; link; link = link->next) {
@@ -878,7 +763,7 @@ void show_devs_tree(struct device *dev, int debug_level, int depth)
 void show_all_devs_tree(int debug_level, const char *msg)
 {
 	/* Bail if not printing to screen. */
-	if (!do_printk(debug_level, "Show all devs in tree form... %s\n", msg))
+	if (!printk(debug_level, "Show all devs in tree form... %s\n", msg))
 		return;
 	show_devs_tree(all_devices, debug_level, 0);
 }
@@ -886,10 +771,10 @@ void show_all_devs_tree(int debug_level, const char *msg)
 void show_devs_subtree(struct device *root, int debug_level, const char *msg)
 {
 	/* Bail if not printing to screen. */
-	if (!do_printk(debug_level, "Show all devs in subtree %s... %s\n",
+	if (!printk(debug_level, "Show all devs in subtree %s... %s\n",
 		       dev_path(root), msg))
 		return;
-	do_printk(debug_level, "%s\n", msg);
+	printk(debug_level, "%s\n", msg);
 	show_devs_tree(root, debug_level, 0);
 }
 
@@ -898,10 +783,10 @@ void show_all_devs(int debug_level, const char *msg)
 	struct device *dev;
 
 	/* Bail if not printing to screen. */
-	if (!do_printk(debug_level, "Show all devs... %s\n", msg))
+	if (!printk(debug_level, "Show all devs... %s\n", msg))
 		return;
 	for (dev = all_devices; dev; dev = dev->next) {
-		do_printk(debug_level, "%s: enabled %d\n",
+		printk(debug_level, "%s: enabled %d\n",
 			  dev_path(dev), dev->enabled);
 	}
 }
@@ -915,22 +800,22 @@ void show_one_resource(int debug_level, struct device *dev,
 	end = resource_end(resource);
 	buf[0] = '\0';
 
-	do_printk(debug_level, "%s %02lx <- [0x%010llx - 0x%010llx] "
+	printk(debug_level, "%s %02lx <- [0x%010llx - 0x%010llx] "
 		  "size 0x%08llx gran 0x%02x %s%s%s\n", dev_path(dev),
 		  resource->index, base, end, resource->size, resource->gran,
 		  buf, resource_type(resource), comment);
 }
 
-void show_all_devs_resources(int debug_level, const char* msg)
+void show_all_devs_resources(int debug_level, const char *msg)
 {
 	struct device *dev;
 
-	if (!do_printk(debug_level, "Show all devs with resources... %s\n", msg))
+	if (!printk(debug_level, "Show all devs with resources... %s\n", msg))
 		return;
 
 	for (dev = all_devices; dev; dev = dev->next) {
 		struct resource *res;
-		do_printk(debug_level, "%s: enabled %d\n",
+		printk(debug_level, "%s: enabled %d\n",
 			  dev_path(dev), dev->enabled);
 		for (res = dev->resource_list; res; res = res->next)
 			show_one_resource(debug_level, dev, res, "");
@@ -938,7 +823,8 @@ void show_all_devs_resources(int debug_level, const char* msg)
 }
 
 void fixed_mem_resource(struct device *dev, unsigned long index,
-		  unsigned long basek, unsigned long sizek, unsigned long type)
+			unsigned long basek, unsigned long sizek,
+			unsigned long type)
 {
 	struct resource *resource;
 
@@ -954,24 +840,31 @@ void fixed_mem_resource(struct device *dev, unsigned long index,
 	resource->flags |= type;
 }
 
-void mmconf_resource_init(struct resource *resource, resource_t base,
-	int buses)
+void fixed_io_resource(struct device *dev, unsigned long index,
+			unsigned long base, unsigned long size)
 {
-	resource->base = base;
-	resource->size = buses * MiB;
-	resource->flags = IORESOURCE_MEM | IORESOURCE_RESERVE |
-		IORESOURCE_FIXED | IORESOURCE_STORED | IORESOURCE_ASSIGNED;
+	struct resource *resource;
 
-	printk(BIOS_DEBUG, "Adding PCIe enhanced config space BAR "
-			"0x%08lx-0x%08lx.\n", (unsigned long)(resource->base),
-			(unsigned long)(resource->base + resource->size));
+	resource = new_resource(dev, index);
+	resource->base = (resource_t)base;
+	resource->size = (resource_t)size;
+	resource->limit = resource->base + resource->size - 1;
+	resource->flags = IORESOURCE_IO | IORESOURCE_FIXED |
+		 IORESOURCE_STORED | IORESOURCE_ASSIGNED |
+		 IORESOURCE_RESERVE;
 }
 
 void mmconf_resource(struct device *dev, unsigned long index)
 {
 	struct resource *resource = new_resource(dev, index);
-	mmconf_resource_init(resource, CONFIG_MMCONF_BASE_ADDRESS,
-		CONFIG_MMCONF_BUS_NUMBER);
+	resource->base = CONFIG_MMCONF_BASE_ADDRESS;
+	resource->size = CONFIG_MMCONF_LENGTH;
+	resource->flags = IORESOURCE_MEM | IORESOURCE_RESERVE |
+		IORESOURCE_FIXED | IORESOURCE_STORED | IORESOURCE_ASSIGNED;
+
+	printk(BIOS_DEBUG, "Adding PCIe enhanced config space BAR 0x%08lx-0x%08lx.\n",
+			(unsigned long)(resource->base),
+			(unsigned long)(resource->base + resource->size));
 }
 
 void tolm_test(void *gp, struct device *dev, struct resource *new)
@@ -981,6 +874,13 @@ void tolm_test(void *gp, struct device *dev, struct resource *new)
 
 	best = *best_p;
 
+	/*
+	 * If resource is not allocated any space i.e. size is zero,
+	 * then do not consider this resource in tolm calculations.
+	 */
+	if (new->size == 0)
+		return;
+
 	if (!best || (best->base > new->base))
 		best = new;
 
@@ -991,9 +891,9 @@ u32 find_pci_tolm(struct bus *bus)
 {
 	struct resource *min = NULL;
 	u32 tolm;
+	unsigned long mask_match = IORESOURCE_MEM | IORESOURCE_ASSIGNED;
 
-	search_bus_resources(bus, IORESOURCE_MEM, IORESOURCE_MEM,
-			     tolm_test, &min);
+	search_bus_resources(bus, mask_match, mask_match, tolm_test, &min);
 
 	tolm = 0xffffffffUL;
 
