@@ -1,18 +1,4 @@
-/*
- * This file is part of the coreboot project.
- *
- * Copyright (C) 2011-2013  Alexandru Gagniuc <mr.nuke.me@gmail.com>
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- */
+/* SPDX-License-Identifier: GPL-2.0-or-later */
 
 /**
  * @file ddr3.c
@@ -23,7 +9,12 @@
 #include <console/console.h>
 #include <device/device.h>
 #include <device/dram/ddr3.h>
+#include <device/dram/common.h>
 #include <string.h>
+#include <memory_info.h>
+#include <cbmem.h>
+#include <smbios.h>
+#include <types.h>
 
 /*==============================================================================
  * = DDR3 SPD decoding helpers
@@ -36,34 +27,16 @@
  *
  * @param type DIMM type. This is byte[3] of the SPD.
  */
-int spd_dimm_is_registered_ddr3(enum spd_dimm_type type)
+int spd_dimm_is_registered_ddr3(enum spd_dimm_type_ddr3 type)
 {
-	if ((type == SPD_DIMM_TYPE_RDIMM)
-	    | (type == SPD_DIMM_TYPE_MINI_RDIMM)
-	    | (type == SPD_DIMM_TYPE_72B_SO_RDIMM))
+	if ((type == SPD_DDR3_DIMM_TYPE_RDIMM)
+	    | (type == SPD_DDR3_DIMM_TYPE_MINI_RDIMM)
+	    | (type == SPD_DDR3_DIMM_TYPE_72B_SO_RDIMM))
 		return 1;
 
 	return 0;
 }
 
-static u16 crc16(const u8 *ptr, int n_crc)
-{
-	int i;
-	u16 crc = 0;
-
-	while (--n_crc >= 0) {
-		crc = crc ^ ((int)*ptr++ << 8);
-		for (i = 0; i < 8; ++i)
-			if (crc & 0x8000) {
-				crc = (crc << 1) ^ 0x1021;
-			} else {
-				crc = crc << 1;
-			}
-	}
-
-	return crc;
-}
-
 /**
  * \brief Calculate the CRC of a DDR3 SPD
  *
@@ -87,7 +60,7 @@ u16 spd_ddr3_calc_crc(u8 *spd, int len)
 		/* Not enough bytes available to get the CRC */
 		return 0;
 
-	return crc16(spd, n_crc);
+	return ddr_crc16(spd, n_crc);
 }
 
 /**
@@ -104,7 +77,7 @@ u16 spd_ddr3_calc_unique_crc(u8 *spd, int len)
 		/* Not enough bytes available to get the CRC */
 		return 0;
 
-	return crc16(&spd[117], 11);
+	return ddr_crc16(&spd[117], 11);
 }
 
 /**
@@ -115,7 +88,7 @@ u16 spd_ddr3_calc_unique_crc(u8 *spd, int len)
  * array, and passed to this function.
  *
  * @param dimm pointer to @ref dimm_attr structure where the decoded data is to
- * 	       be stored
+ *	       be stored
  * @param spd array of raw data previously read from the SPD.
  *
  * @return @ref spd_status enumerator
@@ -123,9 +96,9 @@ u16 spd_ddr3_calc_unique_crc(u8 *spd, int len)
  *		SPD_STATUS_INVALID -- invalid SPD or not a DDR3 SPD
  *		SPD_STATUS_CRC_ERROR -- CRC did not verify
  *		SPD_STATUS_INVALID_FIELD -- A field with an invalid value was
- * 					    detected.
+ *					    detected.
  */
-int spd_decode_ddr3(dimm_attr * dimm, spd_raw_data spd)
+int spd_decode_ddr3(struct dimm_attr_ddr3_st *dimm, spd_raw_data spd)
 {
 	int ret;
 	u16 crc, spd_crc;
@@ -133,7 +106,7 @@ int spd_decode_ddr3(dimm_attr * dimm, spd_raw_data spd)
 	u8 reg8;
 	u32 mtb;		/* medium time base */
 	u32 ftb;		/* fine time base */
-	unsigned int val, param;
+	unsigned int val;
 
 	ret = SPD_STATUS_OK;
 
@@ -170,8 +143,7 @@ int spd_decode_ddr3(dimm_attr * dimm, spd_raw_data spd)
 		printram("  Invalid number of memory banks\n");
 		ret = SPD_STATUS_INVALID_FIELD;
 	}
-	param = 1 << (val + 3);
-	printram("  Banks              : %u\n", param);
+	printram("  Banks              : %u\n", 1 << (val + 3));
 	/* SDRAM capacity */
 	capacity_shift = reg8 & 0x0f;
 	if (capacity_shift > 0x06) {
@@ -295,6 +267,11 @@ int spd_decode_ddr3(dimm_attr * dimm, spd_raw_data spd)
 	dimm->tRTP = spd[27] * mtb;
 	/* Minimum Four Activate Window Delay Time (tFAWmin) */
 	dimm->tFAW = (((spd[28] & 0x0f) << 8) + spd[29]) * mtb;
+	/* Minimum CAS Write Latency Time (tCWLmin)
+	 * - not present in standard SPD */
+	dimm->tCWL = 0;
+	/* System CMD Rate Mode - not present in standard SPD */
+	dimm->tCMD = 0;
 
 	printram("  FTB timings        :");
 	/* FTB is introduced in SPD revision 1.1 */
@@ -374,8 +351,7 @@ int spd_decode_ddr3(dimm_attr * dimm, spd_raw_data spd)
 		 dimm->flags.therm_sensor ? "yes" : "no");
 
 	/*  SDRAM Device Type */
-	reg8 = spd[33];
-	printram("  Standard SDRAM     : %s\n", (reg8 & 0x80) ? "no" : "yes");
+	printram("  Standard SDRAM     : %s\n", (spd[33] & 0x80) ? "no" : "yes");
 
 	if (spd[63] & 0x01) {
 		dimm->flags.pins_mirrored = 1;
@@ -393,6 +369,8 @@ int spd_decode_ddr3(dimm_attr * dimm, spd_raw_data spd)
 	memcpy(dimm->part_number, &spd[128], 16);
 	printram("  Part number        : %s\n", dimm->part_number);
 
+	memcpy(dimm->serial, &spd[SPD_DIMM_SERIAL_NUM], SPD_DIMM_SERIAL_LEN);
+
 	return ret;
 }
 
@@ -416,7 +394,7 @@ int spd_decode_ddr3(dimm_attr * dimm, spd_raw_data spd)
  *		SPD_STATUS_INVALID_FIELD -- A field with an invalid value was
  *					    detected.
  */
-int spd_xmp_decode_ddr3(dimm_attr *dimm,
+int spd_xmp_decode_ddr3(struct dimm_attr_ddr3_st *dimm,
 		       spd_raw_data spd,
 		       enum ddr3_xmp_profile profile)
 {
@@ -484,7 +462,7 @@ int spd_xmp_decode_ddr3(dimm_attr *dimm,
 	/* SDRAM Minimum Cycle Time (tCKmin) */
 	dimm->tCK = xmp[1] * mtb;
 	/* CAS Latencies Supported */
-	dimm->cas_supported = (xmp[9] << 8) + xmp[8];
+	dimm->cas_supported = ((xmp[4] << 8) + xmp[3]) & 0x7fff;
 	/* Minimum CAS Latency Time (tAAmin) */
 	dimm->tAA = xmp[2] * mtb;
 	/* Minimum Write Recovery Time (tWRmin) */
@@ -507,10 +485,95 @@ int spd_xmp_decode_ddr3(dimm_attr *dimm,
 	dimm->tRTP = xmp[16] * mtb;
 	/* Minimum Four Activate Window Delay Time (tFAWmin) */
 	dimm->tFAW = (((xmp[18] & 0x0f) << 8) + xmp[19]) * mtb;
+	/* Minimum CAS Write Latency Time (tCWLmin) */
+	dimm->tCWL = xmp[5] * mtb;
+	/* System CMD Rate Mode */
+	dimm->tCMD = xmp[23] * mtb;
 
 	return ret;
 }
 
+/**
+ * Fill cbmem with information for SMBIOS type 17.
+ *
+ * @param channel Corresponding channel of provided @info
+ * @param slot Corresponding slot of provided @info
+ * @param selected_freq The actual frequency the DRAM is running on
+ * @param info DIMM parameters read from SPD
+ *
+ * @return CB_SUCCESS if DIMM info was written
+ */
+enum cb_err spd_add_smbios17(const u8 channel, const u8 slot,
+			     const u16 selected_freq,
+			     const struct dimm_attr_ddr3_st *info)
+{
+	struct memory_info *mem_info;
+	struct dimm_info *dimm;
+
+	/*
+	 * Allocate CBMEM area for DIMM information used to populate SMBIOS
+	 * table 17
+	 */
+	mem_info = cbmem_find(CBMEM_ID_MEMINFO);
+	if (!mem_info) {
+		mem_info = cbmem_add(CBMEM_ID_MEMINFO, sizeof(*mem_info));
+
+		printk(BIOS_DEBUG, "CBMEM entry for DIMM info: %p\n",
+				mem_info);
+		if (!mem_info)
+			return CB_ERR;
+
+		memset(mem_info, 0, sizeof(*mem_info));
+	}
+
+	if (mem_info->dimm_cnt >= ARRAY_SIZE(mem_info->dimm)) {
+		printk(BIOS_WARNING, "BUG: Too many DIMM infos for %s.\n",
+			__func__);
+		return CB_ERR;
+	}
+
+	dimm = &mem_info->dimm[mem_info->dimm_cnt];
+	if (info->size_mb) {
+		dimm->ddr_type = MEMORY_TYPE_DDR3;
+		dimm->ddr_frequency = selected_freq;
+		dimm->dimm_size = info->size_mb;
+		dimm->channel_num = channel;
+		dimm->rank_per_dimm = info->ranks;
+		dimm->dimm_num = slot;
+		memcpy(dimm->module_part_number, info->part_number, 16);
+		dimm->mod_id = info->manufacturer_id;
+
+		switch (info->dimm_type) {
+		case SPD_DDR3_DIMM_TYPE_SO_DIMM:
+			dimm->mod_type = SPD_SODIMM;
+			break;
+		case SPD_DDR3_DIMM_TYPE_72B_SO_CDIMM:
+			dimm->mod_type = SPD_72B_SO_CDIMM;
+			break;
+		case SPD_DDR3_DIMM_TYPE_72B_SO_RDIMM:
+			dimm->mod_type = SPD_72B_SO_RDIMM;
+			break;
+		case SPD_DDR3_DIMM_TYPE_UDIMM:
+			dimm->mod_type = SPD_UDIMM;
+			break;
+		case SPD_DDR3_DIMM_TYPE_RDIMM:
+			dimm->mod_type = SPD_RDIMM;
+			break;
+		case SPD_DDR3_DIMM_TYPE_UNDEFINED:
+		default:
+			dimm->mod_type = SPD_UNDEFINED;
+			break;
+		}
+
+		dimm->bus_width = MEMORY_BUS_WIDTH_64; // non-ECC only
+		memcpy(dimm->serial, info->serial,
+		       MIN(sizeof(dimm->serial), sizeof(info->serial)));
+		mem_info->dimm_cnt++;
+	}
+
+	return CB_SUCCESS;
+}
+
 /*
  * The information printed below has a more informational character, and is not
  * necessarily tied in to RAM init debugging. Hence, we stop using printram(),
@@ -529,12 +592,12 @@ static void print_ns(const char *msg, u32 val)
 /**
 * \brief Print the info in DIMM
 *
-* Print info about the DIMM. Useful to use when CONFIG_DEBUG_RAM_SETUP is
+* Print info about the DIMM. Useful to use when CONFIG(DEBUG_RAM_SETUP) is
 * selected, or for a purely informative output.
 *
 * @param dimm pointer to already decoded @ref dimm_attr structure
 */
-void dram_print_spd_ddr3(const dimm_attr * dimm)
+void dram_print_spd_ddr3(const struct dimm_attr_ddr3_st *dimm)
 {
 	u16 val16;
 	int i;
@@ -568,227 +631,10 @@ void dram_print_spd_ddr3(const dimm_attr * dimm)
 	print_ns("  tWTRmin           : ", dimm->tWTR);
 	print_ns("  tRTPmin           : ", dimm->tRTP);
 	print_ns("  tFAWmin           : ", dimm->tFAW);
-}
-
-/*==============================================================================
- *= DDR3 MRS helpers
- *----------------------------------------------------------------------------*/
-
-/*
- * MRS command structure:
- * cmd[15:0] = Address pins MA[15:0]
- * cmd[18:16] = Bank address BA[2:0]
- */
-
-/* Map tWR value to a bitmask of the MR0 cycle */
-static u16 ddr3_twr_to_mr0_map(u8 twr)
-{
-	if ((twr >= 5) && (twr <= 8))
-		return (twr - 4) << 9;
-
-	/*
-	 * From 8T onwards, we can only use even values. Round up if we are
-	 * given an odd value.
-	 */
-	if ((twr >= 9) && (twr <= 14))
-		return ((twr + 1) >> 1) << 9;
-
-	/* tWR == 16T is [000] */
-	return 0;
-}
-
-/* Map the CAS latency to a bitmask for the MR0 cycle */
-static u16 ddr3_cas_to_mr0_map(u8 cas)
-{
-	u16 mask = 0;
-	/* A[6:4] are bits [2:0] of (CAS - 4) */
-	mask = ((cas - 4) & 0x07) << 4;
-
-	/* A2 is the MSB of (CAS - 4) */
-	if ((cas - 4) & (1 << 3))
-		mask |= (1 << 2);
-
-	return mask;
-}
-
-/**
- * \brief Get command address for a DDR3 MR0 command
- *
- * The DDR3 specification only covers odd write_recovery up to 7T. If an odd
- * write_recovery greater than 7 is specified, it will be rounded up. If a tWR
- * greater than 8 is specified, it is recommended to explicitly round it up or
- * down before calling this function.
- *
- * write_recovery and cas are given in clock cycles. For example, a CAS of 7T
- * should be given as 7.
- *
- * @param precharge_pd
- * @param write_recovery Write recovery latency, tWR in clock cycles.
- * @param dll_reset
- * @param mode
- * @param cas CAS latency in clock cycles.
- * @param burst_type
- * @param burst_length
- */
-mrs_cmd_t ddr3_get_mr0(enum ddr3_mr0_precharge precharge_pd,
-		       u8 write_recovery,
-		       enum ddr3_mr0_dll_reset dll_reset,
-		       enum ddr3_mr0_mode mode,
-		       u8 cas,
-		       enum ddr3_mr0_burst_type burst_type,
-		       enum ddr3_mr0_burst_length burst_length)
-{
-	mrs_cmd_t cmd = 0 << 16;
-
-	if (precharge_pd == DDR3_MR0_PRECHARGE_FAST)
-		cmd |= (1 << 12);
-
-	cmd |= ddr3_twr_to_mr0_map(write_recovery);
-
-	if (dll_reset == DDR3_MR0_DLL_RESET_YES)
-		cmd |= (1 << 8);
-
-	if (mode == DDR3_MR0_MODE_TEST)
-		cmd |= (1 << 7);
-
-	cmd |= ddr3_cas_to_mr0_map(cas);
-
-	if (burst_type == DDR3_MR0_BURST_TYPE_INTERLEAVED)
-		cmd |= (1 << 3);
-
-	cmd |= (burst_length & 0x03) << 0;
-
-	return cmd;
-}
-
-static u16 ddr3_rtt_nom_to_mr1_map(enum ddr3_mr1_rtt_nom rtt_nom)
-{
-	u16 mask = 0;
-	/* A9 <-> rtt_nom[2] */
-	if (rtt_nom & (1 << 2))
-		mask |= (1 << 9);
-	/* A6 <-> rtt_nom[1] */
-	if (rtt_nom & (1 << 1))
-		mask |= (1 << 6);
-	/* A2 <-> rtt_nom[0] */
-	if (rtt_nom & (1 << 0))
-		mask |= (1 << 2);
-
-	return mask;
-}
-
-static u16 ddr3_ods_to_mr1_map(enum ddr3_mr1_ods ods)
-{
-	u16 mask = 0;
-	/* A5 <-> ods[1] */
-	if (ods & (1 << 1))
-		mask |= (1 << 5);
-	/* A1 <-> ods[0] */
-	if (ods & (1 << 0))
-		mask |= (1 << 1);
-
-	return mask;
-}
-
-/**
- * \brief Get command address for a DDR3 MR1 command
- */
-mrs_cmd_t ddr3_get_mr1(enum ddr3_mr1_qoff qoff,
-		       enum ddr3_mr1_tqds tqds,
-		       enum ddr3_mr1_rtt_nom rtt_nom,
-		       enum ddr3_mr1_write_leveling write_leveling,
-		       enum ddr3_mr1_ods ods,
-		       enum ddr3_mr1_additive_latency additive_latency,
-		       enum ddr3_mr1_dll dll_disable)
-{
-	mrs_cmd_t cmd = 1 << 16;
-
-	if (qoff == DDR3_MR1_QOFF_DISABLE)
-		cmd |= (1 << 12);
-
-	if (tqds == DDR3_MR1_TQDS_ENABLE)
-		cmd |= (1 << 11);
-
-	cmd |= ddr3_rtt_nom_to_mr1_map(rtt_nom);
-
-	if (write_leveling == DDR3_MR1_WRLVL_ENABLE)
-		cmd |= (1 << 7);
-
-	cmd |= ddr3_ods_to_mr1_map(ods);
-
-	cmd |= (additive_latency & 0x03) << 3;
-
-	if (dll_disable == DDR3_MR1_DLL_DISABLE)
-		cmd |= (1 << 0);
-
-	return cmd;
-}
-
-/**
- * \brief Get command address for a DDR3 MR2 command
- *
- * cas_cwl is given in clock cycles. For example, a cas_cwl of 7T should be
- * given as 7.
- *
- * @param rtt_wr
- * @param extended_temp
- * @param self_refresh
- * @param cas_cwl CAS write latency in clock cycles.
- */
-
-mrs_cmd_t ddr3_get_mr2(enum ddr3_mr2_rttwr rtt_wr,
-		       enum ddr3_mr2_srt_range extended_temp,
-		       enum ddr3_mr2_asr self_refresh, u8 cas_cwl)
-{
-	mrs_cmd_t cmd = 2 << 16;
-
-	cmd |= (rtt_wr & 0x03) << 9;
-
-	if (extended_temp == DDR3_MR2_SRT_EXTENDED)
-		cmd |= (1 << 7);
-
-	if (self_refresh == DDR3_MR2_ASR_AUTO)
-		cmd |= (1 << 6);
-
-	cmd |= ((cas_cwl - 5) & 0x07) << 3;
-
-	return cmd;
-}
-
-/**
- * \brief Get command address for a DDR3 MR3 command
- *
- * @param dataflow_from_mpr Specify a non-zero value to put DRAM in read
- *			    leveling mode. Zero for normal operation.
- */
-mrs_cmd_t ddr3_get_mr3(char dataflow_from_mpr)
-{
-	mrs_cmd_t cmd = 3 << 16;
-
-	if (dataflow_from_mpr)
-		cmd |= (1 << 2);
-
-	return cmd;
-}
-
-/**
- * \brief Mirror the address bits for this MRS command
- *
- * Swap the following bits in the MRS command:
- *	- MA3 <-> MA4
- *	- MA5 <-> MA6
- *	- MA7 <-> MA8
- *	- BA0 <-> BA1
- */
-mrs_cmd_t ddr3_mrs_mirror_pins(mrs_cmd_t cmd)
-{
-	u32 downshift, upshift;
-	/* High bits=    A4    |    A6    |    A8    |    BA1 */
-	/* Low bits =    A3    |    A5    |    A7    |    BA0 */
-	u32 lowbits = (1 << 3) | (1 << 5) | (1 << 7) | (1 << 16);
-	downshift = (cmd & (lowbits << 1));
-	upshift = (cmd & lowbits);
-	cmd &= ~(lowbits | (lowbits << 1));
-	cmd |= (downshift >> 1) | (upshift << 1);
-	return cmd;
+	/* Those values are only relevant if an XMP profile sets them */
+	if (dimm->tCWL)
+		print_ns("  tCWLmin           : ", dimm->tCWL);
+	if (dimm->tCMD)
+		printk(BIOS_INFO, "  tCMDmin           : %3u\n",
+		       DIV_ROUND_UP(dimm->tCMD, 256));
 }
