@@ -1,25 +1,5 @@
+/* SPDX-License-Identifier: MIT */
 /*
- * Copyright © 2006 Intel Corporation
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE.
- *
  * Authors:
  *    Eric Anholt <eric@anholt.net>
  *
@@ -28,8 +8,6 @@
 #ifndef _I830_BIOS_H_
 #define _I830_BIOS_H_
 
-#include <compiler.h>
-
 struct vbt_header {
 	u8 signature[20];		/**< Always starts with 'VBT$' */
 	u16 version;			/**< decimal */
@@ -108,14 +86,14 @@ struct vbios_data {
 #define BDB_SKIP		254 /* VBIOS private block, ignore */
 
 struct bdb_general_features {
-        /* bits 1 */
+	/* bits 1 */
 	u8 panel_fitting:2;
 	u8 flexaim:1;
 	u8 msg_enable:1;
 	u8 clear_screen:3;
 	u8 color_flip:1;
 
-        /* bits 2 */
+	/* bits 2 */
 	u8 download_ext_vbt:1;
 	u8 enable_ssc:1;
 	u8 ssc_freq:1;
@@ -125,17 +103,17 @@ struct bdb_general_features {
 	u8 display_clock_mode:1;
 	u8 rsvd8:1; /* finish byte */
 
-        /* bits 3 */
+	/* bits 3 */
 	u8 disable_smooth_vision:1;
 	u8 single_dvi:1;
 	u8 rsvd9:1;
 	u8 fdi_rx_polarity_inverted:1;
 	u8 rsvd10:4; /* finish byte */
 
-        /* bits 4 */
+	/* bits 4 */
 	u8 legacy_monitor_detect;
 
-        /* bits 5 */
+	/* bits 5 */
 	u8 int_crt_support:1;
 	u8 int_tv_support:1;
 	u8 int_efp_support:1;
@@ -228,8 +206,8 @@ struct old_child_dev_config {
 } __packed;
 
 /* This one contains field offsets that are known to be common for all BDB
- * versions. Notice that the meaning of the contents contents may still change,
- * but at least the offsets are consistent. */
+ * versions. Notice that the meaning of the contents may still change, but
+ * at least the offsets are consistent. */
 struct common_child_dev_config {
 	u16 handle;
 	u16 device_type;
@@ -418,7 +396,6 @@ struct bdb_sdvo_lvds_options {
 	u8 panel_misc_bits_4;
 } __packed;
 
-
 #define BDB_DRIVER_FEATURE_NO_LVDS		0
 #define BDB_DRIVER_FEATURE_INT_LVDS		1
 #define BDB_DRIVER_FEATURE_SDVO_LVDS		2
@@ -603,7 +580,7 @@ struct bdb_edp {
 #define   SWF14_PM_APM_12	(0x2 << 16)
 #define   SWF14_PM_APM_11	(0x1 << 16)
 #define SWF14_HK_REQUEST_MASK	0x0000ffff /* see GR18 6:3 for event type */
-          /* if GR18 indicates a display switch */
+	  /* if GR18 indicates a display switch */
 #define   SWF14_DS_PIPEB_LFP2_EN (1<<15)
 #define   SWF14_DS_PIPEB_EFP2_EN (1<<14)
 #define   SWF14_DS_PIPEB_TV2_EN  (1<<13)
@@ -620,9 +597,9 @@ struct bdb_edp {
 #define   SWF14_DS_PIPEA_EFP_EN  (1<<2)
 #define   SWF14_DS_PIPEA_TV_EN   (1<<1)
 #define   SWF14_DS_PIPEA_CRT_EN  (1<<0)
-          /* if GR18 indicates a panel fitting request */
+	  /* if GR18 indicates a panel fitting request */
 #define   SWF14_PFIT_EN		(1<<0) /* 0 means disable */
-          /* if GR18 indicates an APM change request */
+	  /* if GR18 indicates an APM change request */
 #define   SWF14_APM_HIBERNATE	0x4
 #define   SWF14_APM_SUSPEND	0x3
 #define   SWF14_APM_STANDBY	0x1
