@@ -1,13 +1,9 @@
-/*
- * Copyright 2016 The Chromium OS Authors. All rights reserved.
- * Use of this source code is governed by a BSD-style license that can be
- * found in the LICENSE file.
- *
- * This is a driver for a SPI interfaced TPM2 device.
+/* SPDX-License-Identifier: BSD-3-Clause */
+/* This is a driver for a SPI interfaced TPM2 device.
  *
  * It assumes that the required SPI interface has been initialized before the
  * driver is started. A 'sruct spi_slave' pointer passed at initialization is
- * used to direct traffic to the correct SPI interface. This dirver does not
+ * used to direct traffic to the correct SPI interface. This driver does not
  * provide a way to instantiate multiple TPM devices. Also, to keep things
  * simple, the driver unconditionally uses of TPM locality zero.
  *
@@ -15,7 +11,6 @@
  * Specification Revision 00.43".
  */
 
-#include <arch/early_variables.h>
 #include <assert.h>
 #include <commonlib/endian.h>
 #include <console/console.h>
@@ -23,7 +18,7 @@
 #include <endian.h>
 #include <string.h>
 #include <timer.h>
-#include <tpm.h>
+#include <security/tpm/tis.h>
 
 #include "tpm.h"
 
@@ -36,20 +31,30 @@
 #define TPM_DID_VID_REG   (TPM_LOCALITY_0_SPI_BASE + 0xf00)
 #define TPM_RID_REG       (TPM_LOCALITY_0_SPI_BASE + 0xf04)
 #define TPM_FW_VER	  (TPM_LOCALITY_0_SPI_BASE + 0xf90)
+#define CR50_BOARD_CFG     (TPM_LOCALITY_0_SPI_BASE + 0xfe0)
+
+#define CR50_BOARD_CFG_LOCKBIT_MASK 0x80000000U
+#define CR50_BOARD_CFG_FEATUREBITS_MASK 0x3FFFFFFFU
+
+#define CR50_BOARD_CFG_100US_READY_PULSE 0x00000001U
+#define CR50_BOARD_CFG_VALUE \
+		(CONFIG(CR50_USE_LONG_INTERRUPT_PULSES) \
+		 ? CR50_BOARD_CFG_100US_READY_PULSE : 0)
 
 #define CR50_TIMEOUT_INIT_MS 30000 /* Very long timeout for TPM init */
 
 /* SPI slave structure for TPM device. */
-static struct spi_slave g_spi_slave CAR_GLOBAL;
+static struct spi_slave spi_slave;
 
 /* Cached TPM device identification. */
-static struct tpm2_info g_tpm_info CAR_GLOBAL;
+static struct tpm2_info tpm_info;
+static struct cr50_firmware_version cr50_firmware_version;
 
 /*
- * TODO(vbendeb): make CONFIG_DEBUG_TPM an int to allow different level of
+ * TODO(vbendeb): make CONFIG(DEBUG_TPM) an int to allow different level of
  * debug traces. Right now it is either 0 or 1.
  */
-static const int debug_level_ = CONFIG_DEBUG_TPM;
+static const int debug_level_ = CONFIG(DEBUG_TPM);
 
 /*
  * SPI frame header for TPM transactions is 4 bytes in size, it is described
@@ -61,16 +66,17 @@ typedef struct {
 
 void tpm2_get_info(struct tpm2_info *info)
 {
-	*info = car_get_var(g_tpm_info);
+	*info = tpm_info;
 }
 
-__attribute__((weak)) int tis_plat_irq_status(void)
+__weak int tis_plat_irq_status(void)
 {
-	static int warning_displayed CAR_GLOBAL;
+	static int warning_displayed;
 
-	if (!car_get_var(warning_displayed)) {
-		printk(BIOS_WARNING, "WARNING: tis_plat_irq_status() not implemented, wasting 10ms to wait on Cr50!\n");
-		car_set_var(warning_displayed, 1);
+	if (!warning_displayed) {
+		printk(BIOS_WARNING, "WARNING: %s() not implemented, wasting 10ms to wait on"
+		       " Cr50!\n", __func__);
+		warning_displayed = 1;
 	}
 	mdelay(10);
 
@@ -78,8 +84,8 @@ __attribute__((weak)) int tis_plat_irq_status(void)
 }
 
 /*
- * TPM may trigger a irq after finish processing previous transfer.
- * Waiting for this irq to sync tpm status.
+ * TPM may trigger a IRQ after finish processing previous transfer.
+ * Waiting for this IRQ to sync TPM status.
  *
  * Returns 1 on success, 0 on failure (timeout).
  */
@@ -90,7 +96,7 @@ static int tpm_sync(void)
 	stopwatch_init_msecs_expire(&sw, 10);
 	while (!tis_plat_irq_status()) {
 		if (stopwatch_expired(&sw)) {
-			printk(BIOS_ERR, "Timeout wait for tpm irq!\n");
+			printk(BIOS_ERR, "Timeout wait for TPM IRQ!\n");
 			return 0;
 		}
 	}
@@ -103,54 +109,57 @@ static int tpm_sync(void)
  *
  * Returns 1 on success, 0 on failure (TPM SPI flow control timeout.)
  */
-static int start_transaction(int read_write, size_t bytes, unsigned addr)
+static int start_transaction(int read_write, size_t bytes, unsigned int addr)
 {
-	spi_frame_header header;
+	spi_frame_header header, header_resp;
 	uint8_t byte;
 	int i;
+	int ret;
 	struct stopwatch sw;
-	static int tpm_sync_needed CAR_GLOBAL;
-	static struct stopwatch wake_up_sw CAR_GLOBAL;
-	struct spi_slave *spi_slave = car_get_var_ptr(&g_spi_slave);
-	/*
-	 * First Cr50 access in each coreboot stage where TPM is used will be
-	 * prepended by a wake up pulse on the CS line.
-	 */
-	int wakeup_needed = 1;
+	static int tpm_sync_needed;
+	static struct stopwatch wake_up_sw;
 
-	/* Wait for tpm to finish previous transaction if needed */
-	if (car_get_var(tpm_sync_needed)) {
-		tpm_sync();
+	if (CONFIG(TPM_CR50)) {
 		/*
-		 * During the first invocation of this function on each stage
-		 * this if () clause code does not run (as tpm_sync_needed
-		 * value is zero), during all following invocations the
-		 * stopwatch below is guaranteed to be started.
+		 * First Cr50 access in each coreboot stage where TPM is used will be
+		 * prepended by a wake up pulse on the CS line.
 		 */
-		if (!stopwatch_expired(car_get_var_ptr(&wake_up_sw)))
-			wakeup_needed = 0;
-	} else {
-		car_set_var(tpm_sync_needed, 1);
-	}
+		int wakeup_needed = 1;
 
-	if (wakeup_needed) {
-		/* Just in case Cr50 is asleep. */
-		spi_claim_bus(spi_slave);
-		udelay(1);
-		spi_release_bus(spi_slave);
-		udelay(100);
-	}
+		/* Wait for TPM to finish previous transaction if needed */
+		if (tpm_sync_needed) {
+			tpm_sync();
+			/*
+			 * During the first invocation of this function on each stage
+			 * this if () clause code does not run (as tpm_sync_needed
+			 * value is zero), during all following invocations the
+			 * stopwatch below is guaranteed to be started.
+			 */
+			if (!stopwatch_expired(&wake_up_sw))
+				wakeup_needed = 0;
+		} else {
+			tpm_sync_needed = 1;
+		}
 
-	/*
-	 * The Cr50 on H1 does not go to sleep for 1 second after any
-	 * SPI slave activity, let's be conservative and limit the
-	 * window to 900 ms.
-	 */
-	stopwatch_init_msecs_expire(car_get_var_ptr(&wake_up_sw), 900);
+		if (wakeup_needed) {
+			/* Just in case Cr50 is asleep. */
+			spi_claim_bus(&spi_slave);
+			udelay(1);
+			spi_release_bus(&spi_slave);
+			udelay(100);
+		}
+
+		/*
+		 * The Cr50 on H1 does not go to sleep for 1 second after any
+		 * SPI slave activity, let's be conservative and limit the
+		 * window to 900 ms.
+		 */
+		stopwatch_init_msecs_expire(&wake_up_sw, 900);
+	}
 
 	/*
 	 * The first byte of the frame header encodes the transaction type
-	 * (read or write) and transfer size (set to lentgh - 1), limited to
+	 * (read or write) and transfer size (set to length - 1), limited to
 	 * 64 bytes.
 	 */
 	header.body[0] = (read_write ? 0x80 : 0) | 0x40 | (bytes - 1);
@@ -160,7 +169,7 @@ static int start_transaction(int read_write, size_t bytes, unsigned addr)
 		header.body[i + 1] = (addr >> (8 * (2 - i))) & 0xff;
 
 	/* CS assert wakes up the slave. */
-	spi_claim_bus(spi_slave);
+	spi_claim_bus(&spi_slave);
 
 	/*
 	 * The TCG TPM over SPI specification introduces the notion of SPI
@@ -179,30 +188,45 @@ static int start_transaction(int read_write, size_t bytes, unsigned addr)
 	 * the last clock of the byte) is set to 1.
 	 *
 	 * Due to some SPI controllers' shortcomings (Rockchip comes to
-	 * mind...) we trasmit the 4 byte header without checking the byte
+	 * mind...) we transmit the 4 byte header without checking the byte
 	 * transmitted by the TPM during the transaction's last byte.
 	 *
 	 * We know that cr50 is guaranteed to set the flow control bit to 0
-	 * during the header transfer, but real TPM2 might be fast enough not
-	 * to require to stall the master, this would present an issue.
+	 * during the header transfer. Real TPM2 are fast enough to not require
+	 * to stall the master. They might still use this feature, so test the
+	 * last bit after shifting in the address bytes.
 	 * crosbug.com/p/52132 has been opened to track this.
 	 */
-	spi_xfer(spi_slave, header.body, sizeof(header.body), NULL, 0);
+
+	header_resp.body[3] = 0;
+	if (CONFIG(TPM_CR50))
+		ret = spi_xfer(&spi_slave, header.body, sizeof(header.body), NULL, 0);
+	else
+		ret = spi_xfer(&spi_slave, header.body, sizeof(header.body),
+			       header_resp.body, sizeof(header_resp.body));
+	if (ret) {
+		printk(BIOS_ERR, "SPI-TPM: transfer error\n");
+		spi_release_bus(&spi_slave);
+		return 0;
+	}
+
+	if (header_resp.body[3] & 1)
+		return 1;
 
 	/*
 	 * Now poll the bus until TPM removes the stall bit. Give it up to 100
-	 * ms to sort it out - it could be saving stuff in nvram at some
-	 * point.
+	 * ms to sort it out - it could be saving stuff in nvram at some point.
 	 */
 	stopwatch_init_msecs_expire(&sw, 100);
 	do {
 		if (stopwatch_expired(&sw)) {
 			printk(BIOS_ERR, "TPM flow control failure\n");
-			spi_release_bus(spi_slave);
+			spi_release_bus(&spi_slave);
 			return 0;
 		}
-		spi_xfer(spi_slave, NULL, 0, &byte, 1);
+		spi_xfer(&spi_slave, NULL, 0, &byte, 1);
 	} while (!(byte & 1));
+
 	return 1;
 }
 
@@ -214,11 +238,10 @@ static void trace_dump(const char *prefix, uint32_t reg,
 		       size_t bytes, const uint8_t *buffer,
 		       int force)
 {
-	static char prev_prefix CAR_GLOBAL;
-	static unsigned prev_reg CAR_GLOBAL;
-	static int current_char CAR_GLOBAL;
+	static char prev_prefix;
+	static unsigned int prev_reg;
+	static int current_char;
 	const int BYTES_PER_LINE = 32;
-	int *current_char_ptr = car_get_var_ptr(&current_char);
 
 	if (!force) {
 		if (!debug_level_)
@@ -232,12 +255,11 @@ static void trace_dump(const char *prefix, uint32_t reg,
 	 * Do not print register address again if the last dump print was for
 	 * that register.
 	 */
-	if ((car_get_var(prev_prefix) != *prefix) ||
-		(car_get_var(prev_reg) != reg)) {
-		car_set_var(prev_prefix, *prefix);
-		car_set_var(prev_reg, reg);
+	if (prev_prefix != *prefix || (prev_reg != reg)) {
+		prev_prefix = *prefix;
+		prev_reg = reg;
 		printk(BIOS_DEBUG, "\n%s %2.2x:", prefix, reg);
-		*current_char_ptr = 0;
+		current_char = 0;
 	}
 
 	if ((reg != TPM_DATA_FIFO_REG) && (bytes == 4)) {
@@ -254,12 +276,12 @@ static void trace_dump(const char *prefix, uint32_t reg,
 		 * quantiites is printed byte at a time.
 		 */
 		for (i = 0; i < bytes; i++) {
-			if (*current_char_ptr &&
-				!(*current_char_ptr % BYTES_PER_LINE)) {
+			if (current_char &&
+				!(current_char % BYTES_PER_LINE)) {
 				printk(BIOS_DEBUG, "\n     ");
-				*current_char_ptr = 0;
+				current_char = 0;
 			}
-			(*current_char_ptr)++;
+			(current_char)++;
 			printk(BIOS_DEBUG, " %2.2x", buffer[i]);
 		}
 	}
@@ -271,8 +293,7 @@ static void trace_dump(const char *prefix, uint32_t reg,
  */
 static void write_bytes(const void *buffer, size_t bytes)
 {
-	struct spi_slave *spi_slave = car_get_var_ptr(&g_spi_slave);
-	spi_xfer(spi_slave, buffer, bytes, NULL, 0);
+	spi_xfer(&spi_slave, buffer, bytes, NULL, 0);
 }
 
 /*
@@ -281,8 +302,7 @@ static void write_bytes(const void *buffer, size_t bytes)
  */
 static void read_bytes(void *buffer, size_t bytes)
 {
-	struct spi_slave *spi_slave = car_get_var_ptr(&g_spi_slave);
-	spi_xfer(spi_slave, NULL, 0, buffer, bytes);
+	spi_xfer(&spi_slave, NULL, 0, buffer, bytes);
 }
 
 /*
@@ -291,14 +311,13 @@ static void read_bytes(void *buffer, size_t bytes)
  *
  * Returns one to indicate success, zero to indicate failure.
  */
-static int tpm2_write_reg(unsigned reg_number, const void *buffer, size_t bytes)
+static int tpm2_write_reg(unsigned int reg_number, const void *buffer, size_t bytes)
 {
-	struct spi_slave *spi_slave = car_get_var_ptr(&g_spi_slave);
 	trace_dump("W", reg_number, bytes, buffer, 0);
 	if (!start_transaction(false, bytes, reg_number))
 		return 0;
 	write_bytes(buffer, bytes);
-	spi_release_bus(spi_slave);
+	spi_release_bus(&spi_slave);
 	return 1;
 }
 
@@ -309,15 +328,14 @@ static int tpm2_write_reg(unsigned reg_number, const void *buffer, size_t bytes)
  * Returns one to indicate success, zero to indicate failure. In case of
  * failure zero out the user buffer.
  */
-static int tpm2_read_reg(unsigned reg_number, void *buffer, size_t bytes)
+static int tpm2_read_reg(unsigned int reg_number, void *buffer, size_t bytes)
 {
-	struct spi_slave *spi_slave = car_get_var_ptr(&g_spi_slave);
 	if (!start_transaction(true, bytes, reg_number)) {
 		memset(buffer, 0, bytes);
 		return 0;
 	}
 	read_bytes(buffer, bytes);
-	spi_release_bus(spi_slave);
+	spi_release_bus(&spi_slave);
 	trace_dump("R", reg_number, bytes, buffer, 0);
 	return 1;
 }
@@ -331,7 +349,7 @@ static int read_tpm_sts(uint32_t *status)
 	return tpm2_read_reg(TPM_STS_REG, status, sizeof(*status));
 }
 
-static int write_tpm_sts(uint32_t status)
+static int __must_check write_tpm_sts(uint32_t status)
 {
 	return tpm2_write_reg(TPM_STS_REG, &status, sizeof(status));
 }
@@ -414,22 +432,118 @@ static int tpm2_claim_locality(void)
 	return 0;
 }
 
+static int cr50_parse_fw_version(const char *version_str, struct cr50_firmware_version *ver)
+{
+	int epoch, major, minor;
+
+	char *number = strstr(version_str, " RW_A:");
+	if (!number)
+		number = strstr(version_str, " RW_B:");
+	if (!number)
+		return -1;
+	number += 6; /* Skip past the colon. */
+
+	epoch = skip_atoi(&number);
+	if (*number++ != '.')
+		return -2;
+	major = skip_atoi(&number);
+	if (*number++ != '.')
+		return -2;
+	minor = skip_atoi(&number);
+
+	ver->epoch = epoch;
+	ver->major = major;
+	ver->minor = minor;
+	return 0;
+}
+
+static int cr50_fw_supports_board_cfg(struct cr50_firmware_version *version)
+{
+	/* Cr50 supports the CR50_BOARD_CFG register from version 0.5.5 / 0.6.5
+	 * and onwards. */
+	if (version->epoch > 0 || version->major >= 7
+	    || (version->major >= 5 && version->minor >= 5))
+		return 1;
+	printk(BIOS_INFO, "Cr50 firmware does not support CR50_BOARD_CFG, version: %d.%d.%d\n",
+	       version->epoch, version->major, version->minor);
+	return 0;
+}
+
+/**
+ * Set the BOARD_CFG register on the TPM chip to a particular compile-time constant value.
+ */
+static void cr50_set_board_cfg(void)
+{
+	uint32_t board_cfg_value;
+	if (!cr50_fw_supports_board_cfg(&cr50_firmware_version))
+		return;
+	/* Set the CR50_BOARD_CFG register, for e.g. asking cr50 to use longer ready pulses. */
+	if (!tpm2_read_reg(CR50_BOARD_CFG, &board_cfg_value, sizeof(board_cfg_value))) {
+		printk(BIOS_INFO, "Error reading from cr50\n");
+		return;
+	}
+	if ((board_cfg_value & CR50_BOARD_CFG_FEATUREBITS_MASK) == CR50_BOARD_CFG_VALUE) {
+		printk(BIOS_INFO,
+		       "Current CR50_BOARD_CFG = 0x%08x, matches desired = 0x%08x\n",
+		       board_cfg_value, CR50_BOARD_CFG_VALUE);
+		return;
+	}
+	if (board_cfg_value & CR50_BOARD_CFG_LOCKBIT_MASK) {
+		/* The high bit is set, meaning that the Cr50 is already locked on a particular
+		 * value for the register, but not the one we wanted. */
+		printk(BIOS_ERR,
+		       "ERROR: Current CR50_BOARD_CFG = 0x%08x, does not match desired = 0x%08x\n",
+		       board_cfg_value, CR50_BOARD_CFG_VALUE);
+		return;
+	}
+	printk(BIOS_INFO, "Current CR50_BOARD_CFG = 0x%08x, setting to 0x%08x\n",
+	       board_cfg_value, CR50_BOARD_CFG_VALUE);
+	board_cfg_value = CR50_BOARD_CFG_VALUE;
+	if (!tpm2_write_reg(CR50_BOARD_CFG, &board_cfg_value, sizeof(board_cfg_value)))
+		printk(BIOS_INFO, "Error writing to cr50\n");
+}
+
+/*
+ * Expose method to read the CR50_BOARD_CFG register, will return zero if
+ * register not supported by Cr50 firmware.
+ */
+static uint32_t cr50_get_board_cfg(void)
+{
+	uint32_t board_cfg_value;
+	if (!cr50_fw_supports_board_cfg(&cr50_firmware_version))
+		return 0;
+	if (!tpm2_read_reg(CR50_BOARD_CFG, &board_cfg_value, sizeof(board_cfg_value))) {
+		printk(BIOS_INFO, "Error reading from cr50\n");
+		return 0;
+	}
+	return board_cfg_value & CR50_BOARD_CFG_FEATUREBITS_MASK;
+}
+
+bool cr50_is_long_interrupt_pulse_enabled(void)
+{
+	return cr50_get_board_cfg() & CR50_BOARD_CFG_100US_READY_PULSE;
+}
+
 /* Device/vendor ID values of the TPM devices this driver supports. */
 static const uint32_t supported_did_vids[] = {
-	0x00281ae0  /* H1 based Cr50 security chip. */
+	0x00281ae0,  /* H1 based Cr50 security chip. */
+	0x0000104a   /* ST33HTPH2E32 */
 };
 
+static int first_access_this_boot(void)
+{
+	return ENV_SEPARATE_VERSTAGE || ENV_BOOTBLOCK || !CONFIG(VBOOT);
+}
+
 int tpm2_init(struct spi_slave *spi_if)
 {
 	uint32_t did_vid, status;
 	uint8_t cmd;
 	int retries;
-	struct tpm2_info *tpm_info = car_get_var_ptr(&g_tpm_info);
-	struct spi_slave *spi_slave = car_get_var_ptr(&g_spi_slave);
 
-	memcpy(spi_slave, spi_if, sizeof(*spi_if));
+	memcpy(&spi_slave, spi_if, sizeof(*spi_if));
 
-	/* clear any pending irqs */
+	/* clear any pending IRQs */
 	tis_plat_irq_status();
 
 	/*
@@ -441,12 +555,12 @@ int tpm2_init(struct spi_slave *spi_if)
 	for (retries = 15; retries > 0; retries--) {
 		int i;
 
-		/* In case of falure to read div_vid is set to zero. */
+		/* In case of failure to read div_vid is set to zero. */
 		tpm2_read_reg(TPM_DID_VID_REG, &did_vid, sizeof(did_vid));
 
 		for (i = 0; i < ARRAY_SIZE(supported_did_vids); i++)
 			if (did_vid == supported_did_vids[i])
-				break; /* Tpm is up and ready. */
+				break; /* TPM is up and ready. */
 
 		if (i < ARRAY_SIZE(supported_did_vids))
 			break;
@@ -464,7 +578,8 @@ int tpm2_init(struct spi_slave *spi_if)
 
 	printk(BIOS_INFO, " done!\n");
 
-	if (ENV_VERSTAGE || ENV_BOOTBLOCK)
+	// FIXME: Move this to tpm_setup()
+	if (first_access_this_boot())
 		/*
 		 * Claim locality 0, do it only during the first
 		 * initialization after reset.
@@ -472,7 +587,10 @@ int tpm2_init(struct spi_slave *spi_if)
 		if (!tpm2_claim_locality())
 			return -1;
 
-	read_tpm_sts(&status);
+	if (!read_tpm_sts(&status)) {
+		printk(BIOS_ERR, "Reading status reg failed\n");
+		return -1;
+	}
 	if ((status & TPM_STS_FAMILY_MASK) != TPM_STS_FAMILY_TPM_2_0) {
 		printk(BIOS_ERR, "unexpected TPM family value, status: %#x\n",
 		       status);
@@ -484,26 +602,18 @@ int tpm2_init(struct spi_slave *spi_if)
 	 * structure.
 	 */
 	tpm2_read_reg(TPM_RID_REG, &cmd, sizeof(cmd));
-	tpm_info->vendor_id = did_vid & 0xffff;
-	tpm_info->device_id = did_vid >> 16;
-	tpm_info->revision = cmd;
+	tpm_info.vendor_id = did_vid & 0xffff;
+	tpm_info.device_id = did_vid >> 16;
+	tpm_info.revision = cmd;
 
 	printk(BIOS_INFO, "Connected to device vid:did:rid of %4.4x:%4.4x:%2.2x\n",
-	       tpm_info->vendor_id, tpm_info->device_id, tpm_info->revision);
+	       tpm_info.vendor_id, tpm_info.device_id, tpm_info.revision);
 
 	/* Let's report device FW version if available. */
-	if (tpm_info->vendor_id == 0x1ae0) {
+	if (CONFIG(TPM_CR50) && tpm_info.vendor_id == 0x1ae0) {
 		int chunk_count = 0;
-		size_t chunk_size;
-		/*
-		 * let's read 50 bytes at a time; leave room for the trailing
-		 * zero.
-		 */
-		char vstr[51];
-
-		chunk_size = sizeof(vstr) - 1;
-
-		printk(BIOS_INFO, "Firmware version: ");
+		size_t chunk_size = 50;
+		char version_str[301];
 
 		/*
 		 * Does not really matter what's written, this just makes sure
@@ -511,20 +621,28 @@ int tpm2_init(struct spi_slave *spi_if)
 		 */
 		tpm2_write_reg(TPM_FW_VER, &chunk_size, 1);
 
-		/* Print it out in sizeof(vstr) - 1 byte chunks. */
-		vstr[chunk_size] = 0;
+		/*
+		 * Read chunk_size bytes at a time, last chunk will be zero padded.
+		 */
 		do {
-			tpm2_read_reg(TPM_FW_VER, vstr, chunk_size);
-			printk(BIOS_INFO, "%s", vstr);
-
-			/*
-			 * While string is not over, and is no longer than 300
-			 * characters.
-			 */
-		} while (vstr[chunk_size - 1] &&
-			 (chunk_count++ < (300 / chunk_size)));
-
-		printk(BIOS_INFO, "\n");
+			tpm2_read_reg(TPM_FW_VER,
+				      version_str + chunk_count * chunk_size,
+				      chunk_size);
+			if (!version_str[++chunk_count * chunk_size - 1])
+				/* Zero padding detected: end of string. */
+				break;
+			/* Check if there is enough room for reading one more chunk. */
+		} while (chunk_count * chunk_size < sizeof(version_str) - chunk_size);
+		version_str[chunk_count * chunk_size] = '\0';
+		printk(BIOS_INFO, "Firmware version: %s\n", version_str);
+		if (cr50_parse_fw_version(version_str, &cr50_firmware_version)) {
+			printk(BIOS_ERR, "Did not recognize Cr50 version format\n");
+			return -1;
+		}
+		if (CR50_BOARD_CFG_VALUE) {
+			if (first_access_this_boot())
+				cr50_set_board_cfg();
+		}
 	}
 	return 0;
 }
@@ -571,9 +689,9 @@ union fifo_transfer_buffer {
  * Transfer requested number of bytes to or from TPM FIFO, accounting for the
  * current burst count value.
  */
-static void fifo_transfer(size_t transfer_size,
-			  union fifo_transfer_buffer buffer,
-			  enum fifo_transfer_direction direction)
+static int __must_check fifo_transfer(size_t transfer_size,
+				      union fifo_transfer_buffer buffer,
+				      enum fifo_transfer_direction direction)
 {
 	size_t transaction_size;
 	size_t burst_count;
@@ -594,18 +712,23 @@ static void fifo_transfer(size_t transfer_size,
 		 */
 		transaction_size = MIN(transaction_size, 64);
 
-		if (direction == fifo_receive)
-			tpm2_read_reg(TPM_DATA_FIFO_REG,
-				      buffer.rx_buffer + handled_so_far,
-				      transaction_size);
-		else
-			tpm2_write_reg(TPM_DATA_FIFO_REG,
-				       buffer.tx_buffer + handled_so_far,
-				       transaction_size);
+		if (direction == fifo_receive) {
+			if (!tpm2_read_reg(TPM_DATA_FIFO_REG,
+					   buffer.rx_buffer + handled_so_far,
+					   transaction_size))
+				return 0;
+		} else {
+			if (!tpm2_write_reg(TPM_DATA_FIFO_REG,
+					    buffer.tx_buffer + handled_so_far,
+					    transaction_size))
+				return 0;
+		}
 
 		handled_so_far += transaction_size;
 
 	} while (handled_so_far != transfer_size);
+
+	return 1;
 }
 
 size_t tpm2_process_command(const void *tpm2_command, size_t command_size,
@@ -619,10 +742,9 @@ size_t tpm2_process_command(const void *tpm2_command, size_t command_size,
 	uint8_t *rsp_body = tpm2_response;
 	union fifo_transfer_buffer fifo_buffer;
 	const int HEADER_SIZE = 6;
-	struct tpm2_info *tpm_info = car_get_var_ptr(&g_tpm_info);
 
 	/* Do not try using an uninitialized TPM. */
-	if (!tpm_info->vendor_id)
+	if (!tpm_info.vendor_id)
 		return 0;
 
 	/* Skip the two byte tag, read the size field. */
@@ -639,10 +761,13 @@ size_t tpm2_process_command(const void *tpm2_command, size_t command_size,
 	}
 
 	/* Let the TPM know that the command is coming. */
-	write_tpm_sts(TPM_STS_COMMAND_READY);
+	if (!write_tpm_sts(TPM_STS_COMMAND_READY)) {
+		printk(BIOS_ERR, "TPM_STS_COMMAND_READY failed\n");
+		return 0;
+	}
 
 	/*
-	 * Tpm commands and responses written to and read from the FIFO
+	 * TPM commands and responses written to and read from the FIFO
 	 * register (0x24) are datagrams of variable size, prepended by a 6
 	 * byte header.
 	 *
@@ -653,10 +778,17 @@ size_t tpm2_process_command(const void *tpm2_command, size_t command_size,
 	 * burst count or the maximum PDU size, whatever is smaller.
 	 */
 	fifo_buffer.tx_buffer = cmd_body;
-	fifo_transfer(command_size, fifo_buffer, fifo_transmit);
+	if (!fifo_transfer(command_size, fifo_buffer, fifo_transmit)) {
+		printk(BIOS_ERR, "fifo_transfer %zd command bytes failed\n",
+		       command_size);
+		return 0;
+	}
 
 	/* Now tell the TPM it can start processing the command. */
-	write_tpm_sts(TPM_STS_GO);
+	if (!write_tpm_sts(TPM_STS_GO)) {
+		printk(BIOS_ERR, "TPM_STS_GO failed\n");
+		return 0;
+	}
 
 	/* Now wait for it to report that the response is ready. */
 	expected_status_bits = TPM_STS_VALID | TPM_STS_DATA_AVAIL;
@@ -686,7 +818,7 @@ size_t tpm2_process_command(const void *tpm2_command, size_t command_size,
 		 * TODO(vbendeb): at least drain the FIFO here or somehow let
 		 * the TPM know that the response can be dropped.
 		 */
-		printk(BIOS_ERR, " tpm response too long (%zd bytes)",
+		printk(BIOS_ERR, " TPM response too long (%zd bytes)",
 		       payload_size);
 		return 0;
 	}
@@ -699,7 +831,11 @@ size_t tpm2_process_command(const void *tpm2_command, size_t command_size,
 	 */
 	bytes_to_go = payload_size - 1 - HEADER_SIZE;
 	fifo_buffer.rx_buffer = rsp_body + HEADER_SIZE;
-	fifo_transfer(bytes_to_go, fifo_buffer, fifo_receive);
+	if (!fifo_transfer(bytes_to_go, fifo_buffer, fifo_receive)) {
+		printk(BIOS_ERR, "fifo_transfer %zd receive bytes failed\n",
+		       bytes_to_go);
+		return 0;
+	}
 
 	/* Verify that there is still data to read. */
 	read_tpm_sts(&status);
@@ -724,7 +860,15 @@ size_t tpm2_process_command(const void *tpm2_command, size_t command_size,
 	}
 
 	/* Move the TPM back to idle state. */
-	write_tpm_sts(TPM_STS_COMMAND_READY);
+	if (!write_tpm_sts(TPM_STS_COMMAND_READY)) {
+		printk(BIOS_ERR, "TPM_STS_COMMAND_READY failed\n");
+		return 0;
+	}
 
 	return payload_size;
 }
+
+void cr50_get_firmware_version(struct cr50_firmware_version *version)
+{
+	memcpy(version, &cr50_firmware_version, sizeof(*version));
+}
