@@ -1,7 +1,7 @@
 #ifndef SMP_ATOMIC_H
 #define SMP_ATOMIC_H
 
-#if IS_ENABLED(CONFIG_SMP)
+#if CONFIG(SMP)
 #include <arch/smp/atomic.h>
 #else
 
@@ -31,7 +31,6 @@ typedef struct { int counter; } atomic_t;
  */
 #define atomic_set(v, i)	(((v)->counter) = (i))
 
-
 /**
  * atomic_inc - increment atomic variable
  * @param v: pointer of type atomic_t
@@ -41,7 +40,6 @@ typedef struct { int counter; } atomic_t;
  */
 #define atomic_inc(v)	(((v)->counter)++)
 
-
 /**
  * atomic_dec - decrement atomic variable
  * @param v: pointer of type atomic_t
@@ -51,7 +49,6 @@ typedef struct { int counter; } atomic_t;
  */
 #define atomic_dec(v)	(((v)->counter)--)
 
-
 #endif /* CONFIG_SMP */
 
 #endif /* SMP_ATOMIC_H */
