@@ -2,12 +2,8 @@
 #define STDDEF_H
 
 #include <commonlib/helpers.h>
-#include <rules.h>
 
-typedef long ptrdiff_t;
-#ifndef __SIZE_TYPE__
-#define __SIZE_TYPE__ unsigned long
-#endif
+typedef __PTRDIFF_TYPE__ ptrdiff_t;
 typedef __SIZE_TYPE__ size_t;
 /* There is a GCC macro for a size_t type, but not
  * for a ssize_t type. Below construct tricks GCC
@@ -17,14 +13,14 @@ typedef __SIZE_TYPE__ size_t;
 typedef __SIZE_TYPE__ ssize_t;
 #undef unsigned
 
-typedef int wchar_t;
-typedef unsigned int wint_t;
+typedef __WCHAR_TYPE__ wchar_t;
+typedef __WINT_TYPE__ wint_t;
 
 #define NULL ((void *)0)
 
 /* The devicetree data structures are only mutable in ramstage. All other
    stages have a constant devicetree. */
-#if !ENV_RAMSTAGE
+#if !ENV_PAYLOAD_LOADER
 #define DEVTREE_EARLY 1
 #else
 #define DEVTREE_EARLY 0
@@ -36,19 +32,16 @@ typedef unsigned int wint_t;
 #define DEVTREE_CONST
 #endif
 
-/* Work around non-writable data segment in execute-in-place romstage on x86. */
-#if defined(__PRE_RAM__) && IS_ENABLED(CONFIG_ARCH_X86)
-#define MAYBE_STATIC
+#if ENV_STAGE_HAS_DATA_SECTION
+#define MAYBE_STATIC_NONZERO static
 #else
-#define MAYBE_STATIC static
+#define MAYBE_STATIC_NONZERO
 #endif
 
-#ifndef __ROMCC__
 /* Provide a pointer to address 0 that thwarts any "accessing this is
  * undefined behaviour and do whatever" trickery in compilers.
  * Use when you _really_ need to read32(zeroptr) (ie. read address 0).
  */
 extern char zeroptr[];
-#endif
 
 #endif /* STDDEF_H */
