@@ -1,30 +1,15 @@
-/*
- * This file is part of the coreboot project.
- *
- * Copyright 2016 Google Inc.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- */
+/* SPDX-License-Identifier: GPL-2.0-only */
 
 #include <delay.h>
 #include <gpio.h>
-#include <soc/verstage.h>
-
+#include <security/vboot/vboot_common.h>
 
 #define TPM_RESET_GPIO		19
 
 static void ipq_setup_tpm(void)
 {
 
-	if (IS_ENABLED(CONFIG_I2C_TPM)) {
+	if (CONFIG(I2C_TPM)) {
 		gpio_tlmm_config_set(TPM_RESET_GPIO, FUNC_SEL_GPIO,
 				     GPIO_PULL_UP, GPIO_6MA, 1);
 		gpio_set(TPM_RESET_GPIO, 0);
