@@ -1,17 +1,4 @@
-/*
- * This file is part of the coreboot project.
- *
- * Copyright 2017 Google Inc.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- */
+/* SPDX-License-Identifier: GPL-2.0-only */
 
 #include <baseboard/variants.h>
 #include <gpio.h>
@@ -37,7 +24,7 @@ static const u16 rcomp_resistor[] = { 200, 81, 162 };
 /* Rcomp target */
 static const u16 rcomp_target[] = { 100, 40, 40, 23, 40 };
 
-void __attribute__((weak)) variant_memory_params(struct memory_params *p)
+void __weak variant_memory_params(struct memory_params *p)
 {
 	p->type = MEMORY_LPDDR3;
 	p->dq_map = dq_map;
@@ -50,7 +37,7 @@ void __attribute__((weak)) variant_memory_params(struct memory_params *p)
 	p->rcomp_target_size = sizeof(rcomp_target);
 }
 
-int __attribute__((weak)) variant_memory_sku(void)
+int __weak variant_memory_sku(void)
 {
 	gpio_t spd_gpios[] = {
 		GPIO_MEM_CONFIG_0,
