@@ -1,19 +1,7 @@
-/*
- * This file is part of the coreboot project.
- *
- * Copyright 2017 Google Inc.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- */
+/* SPDX-License-Identifier: GPL-2.0-only */
 
 #include <baseboard/variants.h>
+#include <variant/sku.h>
 
 /* DQ byte map */
 static const u8 dq_map[][12] = {
@@ -46,4 +34,13 @@ void variant_memory_params(struct memory_params *p)
 	p->rcomp_resistor_size = sizeof(rcomp_resistor);
 	p->rcomp_target = rcomp_target;
 	p->rcomp_target_size = sizeof(rcomp_target);
+
+	switch (variant_board_sku()) {
+	case SKU_0_NAUTILUS:
+		/* Bumping VCC_SA voltage offset 75mV to allow a
+		 * tolerance to PLLGT on Nautilus-Wifi sku */
+		p->enable_sa_oc_support = 1;
+		p->sa_voltage_offset_val = 75;
+		break;
+	}
 }
