@@ -1,23 +1,8 @@
-/*
- * This file is part of the coreboot project.
- *
- * Copyright 2014 Rockchip Inc.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- */
+/* SPDX-License-Identifier: GPL-2.0-only */
 
+#include <bootmode.h>
 #include <boot/coreboot_tables.h>
-#include <console/console.h>
 #include <gpio.h>
-#include <string.h>
-#include <vendorcode/google/chromeos/chromeos.h>
 
 #include "board.h"
 
@@ -26,7 +11,6 @@
 #define GPIO_RECOVERY_SERVO	GPIO(0, B, 1)
 #define GPIO_RECOVERY_PUSHKEY	GPIO(7, B, 1)
 
-
 void setup_chromeos_gpios(void)
 {
 	gpio_input(GPIO_WP);
@@ -38,11 +22,10 @@ void setup_chromeos_gpios(void)
 void fill_lb_gpios(struct lb_gpios *gpios)
 {
 	struct lb_gpio chromeos_gpios[] = {
-		{GPIO_WP.raw, ACTIVE_LOW, gpio_get(GPIO_WP), "write protect"},
 		/* Note for early development, we want to support both servo
 		 * and pushkey recovery buttons in firmware boot stages. */
 		{GPIO_RECOVERY_PUSHKEY.raw, ACTIVE_LOW,
-			!get_recovery_mode_switch(), "recovery"},
+			!get_recovery_mode_switch(), "presence"},
 		{GPIO_POWER.raw, ACTIVE_LOW, -1, "power"},
 		{GPIO_RESET.raw, ACTIVE_HIGH, -1, "reset"},
 	};
