@@ -1,21 +1,8 @@
-/*
- * This file is part of the coreboot project.
- *
- * Copyright (C) 2014 Google Inc.
- * Copyright (C) 2015 Intel Corporation.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- */
+/* SPDX-License-Identifier: GPL-2.0-only */
 
 #ifndef MAINBOARD_SPD_H
 
+#include <fsp/soc_binding.h>
 #include <gpio.h>
 #include "../gpio.h"
 
@@ -40,7 +27,6 @@
 
 /* PCH_MEM_CFG[3:0] */
 #define MAX_MEMORY_CONFIG	0x10
-#define RCOMP_TARGET_PARAMS	0x5
 #define K4E6E304EE_MEM_ID	0x3
 
 static inline int get_spd_index(void) {
@@ -53,8 +39,8 @@ static inline int get_spd_index(void) {
 	};
 	return (gpio_base2_value(spd_gpios, ARRAY_SIZE(spd_gpios)));
 }
-void mainboard_fill_dq_map_data(void *dq_map_ptr);
-void mainboard_fill_dqs_map_data(void *dqs_map_ptr);
+void mainboard_fill_dq_map_data(void *dq_map_ch0, void *dq_map_ch1);
+void mainboard_fill_dqs_map_data(void *dqs_map_ch0, void *dqs_map_ch1);
 void mainboard_fill_rcomp_res_data(void *rcomp_ptr);
 void mainboard_fill_rcomp_strength_data(void *rcomp_strength_ptr);
 uintptr_t mainboard_get_spd_data(void);
