@@ -1,23 +1,10 @@
-/*
- * This file is part of the coreboot project.
- *
- * Copyright (C) 2011 Advanced Micro Devices, Inc.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- */
+/* SPDX-License-Identifier: GPL-2.0-only */
 
 #ifndef _AGESA_HELPER_H_
 #define _AGESA_HELPER_H_
 
 #include <stddef.h>
-#include <arch/cpu.h>
+#include <arch/romstage.h>
 
 enum {
 	PICK_DMI,       /* DMI Interface */
@@ -36,7 +23,6 @@ void agesawrapper_setlateinitptr (void *Late);
 void *agesawrapper_getlateinitptr (int pick);
 
 void amd_initcpuio(void);
-void amd_initmmio(void);
 void amd_initenv(void);
 
 void *GetHeapBase(void);
@@ -44,13 +30,8 @@ void EmptyHeap(void);
 
 #define BSP_STACK_BASE_ADDR		0x30000
 
-#if 1
 /* This covers node 0 only. */
 #define HIGH_ROMSTAGE_STACK_SIZE	(0x48000 - BSP_STACK_BASE_ADDR)
-#else
-/* This covers total of 8 nodes. */
-#define HIGH_ROMSTAGE_STACK_SIZE	(0xA0000 - BSP_STACK_BASE_ADDR)
-#endif
 
 #define HIGH_MEMORY_SCRATCH		0x30000
 
