@@ -1,6 +1,10 @@
+/* SPDX-License-Identifier: GPL-2.0-only */
+
 #ifndef RAMINIT_H
 #define RAMINIT_H
 
+#include <stdint.h>
+
 #define MAX_DIMM_SOCKETS_PER_CHANNEL 4
 #define MAX_NUM_CHANNELS 2
 #define MAX_DIMM_SOCKETS (MAX_NUM_CHANNELS * MAX_DIMM_SOCKETS_PER_CHANNEL)
@@ -15,13 +19,6 @@ struct mem_controller {
 	uint16_t channel1[MAX_DIMM_SOCKETS_PER_CHANNEL];
 };
 
-void e7505_mch_init(const struct mem_controller *memctrl);
-void e7505_mch_scrub_ecc(unsigned long ret_addr);
-void e7505_mch_done(const struct mem_controller *memctrl);
-int e7505_mch_is_ready(void);
-
-
-/* Mainboard exports this. */
-int spd_read_byte(unsigned device, unsigned address);
+void sdram_initialize(void);
 
 #endif /* RAMINIT_H */
